
#ifndef _ROHC_V2_H
#define _ROHC_V2_H

#define ROHCV2_DEBUG   1

#define MAX_IP_HEADERS   5

#define PACKET_TYPE_IR   0xFD

/* Moved in rohc.h
// See RFC5225 page 46

#define ROHCv2_PROFILE_RTP	          0x0101
#define ROHCv2_PROFILE_UDP	          0x0102
#define ROHCv2_PROFILE_ESP	          0x0103
#define ROHCv2_PROFILE_IP	          0x0104
#define ROHCv2_PROFILE_UDPLITE_RTP	  0x0107
#define ROHCv2_PROFILE_UDPLITE	          0x0108

*/

// IP-ID behavior constants
#define IP_ID_BEHAVIOR_SEQUENTIAL          0
#define IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED  1
#define IP_ID_BEHAVIOR_RANDOM              2
#define IP_ID_BEHAVIOR_ZERO                3
#define IP_ID_BEHAVIOR_UNKNOWN             0xFE

// UDP-lite checksum coverage behavior constants
#define UDP_LITE_COVERAGE_INFERRED         0
#define UDP_LITE_COVERAGE_STATIC           1
#define UDP_LITE_COVERAGE_IRREGULAR        2
#define UDP_LITE_COVERAGE_UNKNOWN          0xFF

// Variable reordering offset
#define REORDERING_NONE                    0
#define REORDERING_QUATER                  1
#define REORDERING_HALF                    2
#define REORDERING_THREEQUARTERS           3

// Profile names and versions
#define PROFILE_RTP_0101                   0x0101
#define PROFILE_UDP_0102                   0x0102
#define PROFILE_ESP_0103                   0x0103
#define PROFILE_IP_0104                    0x0104
#define PROFILE_RTP_0107                   0x0107
#define PROFILE_UDPLITE_0108               0x0108

// Default values for RTP timestamp encoding
#define TS_STRIDE_DEFAULT                  160
#define TIME_STRIDE_DEFAULT                0


/**
 * @brief Define the IPv6 option header.
 *
 */

typedef struct __attribute__((packed)) ipv6_opt
{
	u_int8_t next_header;
	u_int8_t length;
	u_int8_t value[1];
} ipv6_opt_t;

/**
 * @brief Define the static part of IPv6 option header.
 *
 */

typedef struct __attribute__((packed)) ip_opt_static
{
	u_int8_t next_header;
	u_int8_t length;
} ip_opt_static_t;

/**
 * @brief Define the dynamic part of IPv6 option header.
 *
 */

typedef struct __attribute__((packed)) ip_opt_dynamic
{
	u_int8_t value[1];
} ip_opt_dynamic_t;

/**
 * @brief Define the IPv6 destination option header.
 *
 */

typedef struct __attribute__((packed)) ip_dest_opt
{
	u_int8_t next_header;
	u_int8_t length;
	u_int8_t value[1];
} ip_dest_opt_t;

/**
 * @brief Define the static part of IPv6 Destination option header.
 *
 */

typedef struct __attribute__((packed)) ip_dest_opt_static
{
	u_int8_t next_header;
	u_int8_t length;
} ip_dest_opt_static_t;

/**
 * @brief Define the dynamic part of IPv6 Destination option header.
 *
 */

typedef struct __attribute__((packed)) ip_dest_opt_dynamic
{
	u_int8_t value[1];
} ip_dest_opt_dynamic_t;

/**
 * @brief Define the IPv6 Hop-by-Hop option header.
 *
 */

typedef struct __attribute__((packed)) ip_hop_opt
{
	u_int8_t next_header;
	u_int8_t length;
	u_int8_t value[1];
} ip_hop_opt_t;

/**
 * @brief Define the static part of IPv6 Hop-by-Hop option header.
 *
 */

typedef struct __attribute__((packed)) ip_hop_opt_static
{
	u_int8_t next_header;
	u_int8_t length;
} ip_hop_opt_static_t;

/**
 * @brief Define the dynamic part of IPv6 Hop-by-Hop option header.
 *
 */

typedef struct __attribute__((packed)) ip_hop_opt_dynamic
{
	u_int8_t value[1];
} ip_hop_opt_dynamic_t;

/**
 * @brief Define the IPv6 Routing option header.
 *
 */

typedef struct __attribute__((packed)) ip_rout_opt
{
	u_int8_t next_header;
	u_int8_t length;
	u_int8_t value[1];
} ip_rout_opt_t;

/**
 * @brief Define the static part of IPv6 Routing option header.
 *
 */

typedef struct __attribute__((packed)) ip_rout_opt_static
{
	u_int8_t next_header;
	u_int8_t length;
	u_int8_t value[1];
} ip_rout_opt_static_t;

/**
 * @brief Define the IPv6 GRE option header.
 *
 * See RFC5225 page 55
 */

typedef struct __attribute__((packed)) ip_gre_opt
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reserved2 : 4;
	u_int8_t s_flag : 1;
	u_int8_t k_flag : 1;
	u_int8_t r_flag : 1;
	u_int8_t c_flag : 1;

	u_int8_t version : 3;
	u_int8_t reserved1 : 5;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t c_flag : 1;
	u_int16_t r_flag : 1;
	u_int16_t k_flag : 1;
	u_int16_t s_flag : 1;
	u_int16_t reserved0 : 9;
	u_int16_t version : 3;
#endif
	u_int16_t protocol;
	u_int32_t datas[1];
} ip_gre_opt_t;

/**
 * @brief Define the static part of IPv6 GRE option header.
 *
 */

typedef struct __attribute__((packed)) ip_gre_opt_static
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t padding : 4;
	u_int8_t s_flag : 1;
	u_int8_t k_flag : 1;
	u_int8_t c_flag : 1;
	u_int8_t protocol : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t protocol : 1;
	u_int8_t c_flag : 1;
	u_int8_t k_flag : 1;
	u_int8_t s_flag : 1;
	u_int8_t padding : 4;
#endif
	u_int32_t key;     // if k_flag
} ip_gre_opt_static_t;

/**
 * @brief Define the IPv6 MIME option header.
 *
 */

typedef struct __attribute__((packed)) ip_mime_opt
{
	u_int8_t next_header;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t res_bits : 7;
	u_int8_t s_bit : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t s_bit : 1;
	u_int8_t res_bits : 7;
#endif
	u_int16_t checksum;
	u_int32_t orig_dest;
	u_int32_t orig_src;     // if s_bit set
} ip_mime_opt_t;

/**
 * @brief Define the static part of IPv6 MIME option header.
 *
 */

typedef struct __attribute__((packed)) ip_mime_opt_static
{
	u_int8_t next_header;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t res_bits : 7;
	u_int8_t s_bit : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t s_bit : 1;
	u_int8_t res_bits : 7;
#endif
	u_int32_t orig_dest;
	u_int32_t orig_src;         // if s_bit set
} ip_mime_opt_static_t;

/**
 * @brief Define the IPv6 Authentication option header.
 *
 */

typedef struct __attribute__((packed)) ip_ah_opt
{
	u_int8_t next_header;
	u_int8_t length;
	u_int16_t res_bits;
	u_int32_t spi;
	u_int32_t sequence_number;
	u_int32_t icv[1];
} ip_ah_opt_t;

/**
 * @brief Define the static part of IPv6 Authentication option header.
 *
 */

typedef struct __attribute__((packed)) ip_ah_opt_static
{
	u_int8_t next_header;
	u_int8_t length;
	u_int32_t spi;
} ip_ah_opt_static_t;

/**
 * @brief Define the dynamic part of IPv6 Authentication option header.
 *
 */

typedef struct __attribute__((packed)) ip_ah_opt_dynamic
{
	u_int32_t sequence_number;
	u_int32_t icv[1];
} ip_ah_opt_dynamic_t;

/**
 * @brief Define the common IP v4/v6 header.
 *
 */

typedef struct __attribute__((packed)) base_header_ipvx
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reserved : 4;
	u_int8_t version : 4;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t version : 4;
	u_int8_t reserved : 4;
#endif
} base_header_ipvx_t;

/***********************************************************/
/*                IPv6                                     */
/***********************************************************/

/**
 * @brief Define the IP v6 header.
 *
 * See RFC5225 page 58
 */

typedef struct __attribute__((packed)) base_header_ipv6
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t tos_tc1 : 4;
	u_int8_t version : 4;

	u_int8_t flow_label1 : 4;
	u_int8_t tos_tc2 : 4;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t version : 4;
	u_int16_t tos_tc : 8;
	u_int16_t flow_label1 : 4;
#endif
	u_int16_t flow_label2;
	u_int16_t payload_length;
	u_int8_t next_header;
	u_int8_t ttl_hopl;
	u_int32_t src_addr[4];
	u_int32_t dest_addr[4];

} base_header_ipv6_t;

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define TOS_TC_V6(ptr)       (((ptr->tos_tc1) << 4) | ptr->tos_tc2)
#else
#define TOS_TC_V6(ptr)       (ptr->tos_tc)
#endif
#define FLOW_LABEL_V6(ptr) (((ptr->flow_label1) << 16) | ptr->flow_label2)

/**
 * @brief Define the IP v6 static part, null flow_label encoded with 1 bit
 *
 * See RFC5225 page 58
 */

typedef struct __attribute__((packed)) ipv6_static1
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t flow_label1 : 4;
	u_int8_t flow_label_enc_discriminator : 1;
	u_int8_t reserved : 1;
	u_int8_t innermost_ip : 1;
	u_int8_t version_flag : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t version_flag : 1;
	u_int8_t innermost_ip : 1;
	u_int8_t reserved : 1;
	u_int8_t flow_label_enc_discriminator : 1;
	u_int8_t flow_label1 : 4;
#endif
	u_int8_t next_header;
	u_int32_t src_addr[4];
	u_int32_t dst_addr[4];
} ipv6_static1_t;

/**
 * @brief Define the IP v6 static part, flow_label encoded with 1+20 bits
 *
 * See RFC5225 page 58
 */

typedef struct __attribute__((packed)) ipv6_static2
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t flow_label1 : 4;
	u_int8_t flow_label_enc_discriminator : 1;
	u_int8_t reserved : 1;
	u_int8_t innermost_ip : 1;
	u_int8_t version_flag : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t version_flag : 1;
	u_int8_t innermost_ip : 1;
	u_int8_t reserved : 1;
	u_int8_t flow_label_enc_discriminator : 1;
	u_int8_t flow_label1 : 4;
#endif
	u_int16_t flow_label2;
	u_int8_t next_header;
	u_int32_t src_addr[4];
	u_int32_t dst_addr[4];
} ipv6_static2_t;

/**
 * @brief Define the IP v6 endpoint dynamic part
 *
 * See RFC5225 page 59
 */

typedef struct __attribute__((packed)) ipv6_endpoint_dynamic
{
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reorder_ratio : 2;
	u_int8_t reserved : 6;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 6;
	u_int8_t reorder_ratio : 2;
#endif
	u_int16_t msn;
} ipv6_endpoint_dynamic_t;

/**
 * @brief Define the IP v6 regular dynamic part
 *
 * See RFC5225 page 59
 */

typedef struct __attribute__((packed)) ipv6_regular_dynamic
{
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
} ipv6_regular_dynamic_t;

typedef struct __attribute__((packed)) ipv6_outer_irregular
{
	u_int8_t datas[1];
} ipv6_outer_irregular_t;

/***********************************************************/
/*                IPv4                                     */
/***********************************************************/

/**
 * @brief Define the IP v4 header
 *
 * See RFC5225 page 61
 */

typedef struct __attribute__((packed)) base_header_ipv4
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t hdr_length : 4;
	u_int8_t version : 4;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t version : 4;
	u_int8_t hdr_length : 4;
#endif
	u_int8_t tos_tc;
	u_int16_t length;
	u_int16_t ip_id;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t frag_offset1 : 5;
	u_int8_t mf : 1;
	u_int8_t df : 1;
	u_int8_t rf : 1;
	u_int8_t frag_offset2;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t rf : 1;
	u_int16_t df : 1;
	u_int16_t mf : 1;
	u_int16_t frag_offset : 13;
#endif
	u_int8_t ttl_hopl;
	u_int8_t protocol;
	u_int16_t checksum;
	u_int32_t src_addr;
	u_int32_t dest_addr;
	// extension_headers
} base_header_ipv4_t;

/**
 * @brief Define the IP v4 static part
 *
 * See RFC5225 page 61
 */

typedef struct __attribute__((packed)) ipv4_static
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reserved : 6;
	u_int8_t innermost_ip : 1;
	u_int8_t version_flag : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t version_flag : 1;
	u_int8_t innermost_ip : 1;
	u_int8_t reserved : 6;
#endif
	u_int8_t protocol;
	u_int32_t src_addr;
	u_int32_t dst_addr;
} ipv4_static_t;

/**
 * @brief Define the IP v4 endpoint innermost dynamic
 *
 * See RFC5225 page 61
 */

typedef struct __attribute__((packed)) ipv4_endpoint_innermost_dynamic1
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id_behavior_innermost : 2;
	u_int8_t df : 1;
	u_int8_t reorder_ratio : 2;
	u_int8_t reserved : 3;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 3;
	u_int8_t reorder_ratio : 2;
	u_int8_t df : 1;
	u_int8_t ip_id_behavior_innermost : 2;
#endif
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
	u_int16_t msn;
} ipv4_endpoint_innermost_dynamic1_t;

// with IP-ID

typedef struct __attribute__((packed)) ipv4_endpoint_innermost_dynamic2
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id_behavior_innermost : 2;
	u_int8_t df : 1;
	u_int8_t reorder_ratio : 2;
	u_int8_t reserved : 3;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 3;
	u_int8_t reorder_ratio : 2;
	u_int8_t df : 1;
	u_int8_t ip_id_behavior_innermost : 2;
#endif
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
	u_int16_t ip_id;
	u_int16_t msn;
} ipv4_endpoint_innermost_dynamic2_t;

/**
 * @brief Define the IP v4 regular innermost dynamic
 *
 * See RFC5225 page 62
 */

typedef struct __attribute__((packed)) ipv4_regular_innermost_dynamic1
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id_behavior_innermost : 2;
	u_int8_t df : 1;
	u_int8_t reserved : 5;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 5;
	u_int8_t df : 1;
	u_int8_t ip_id_behavior_innermost : 2;
#endif
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
} ipv4_regular_innermost_dynamic1_t;

// with IP-ID

typedef struct __attribute__((packed)) ipv4_regular_innermost_dynamic2
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id_behavior_innermost : 2;
	u_int8_t df : 1;
	u_int8_t reserved : 5;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 5;
	u_int8_t df : 1;
	u_int8_t ip_id_behavior_innermost : 2;
#endif
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
	u_int16_t ip_id;
} ipv4_regular_innermost_dynamic2_t;

/**
 * @brief Define the IP v4 outer dynamic
 *
 * See RFC5225 page 62
 */

typedef struct __attribute__((packed)) ipv4_outer_dynamic1
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id_behavior_outer : 2;
	u_int8_t df : 1;
	u_int8_t reserved : 5;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 5;
	u_int8_t df : 1;
	u_int8_t ip_id_behavior_outer : 2;
#endif
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
} ipv4_outer_dynamic1_t;

// with IP-ID

typedef struct __attribute__((packed)) ipv4_outer_dynamic2
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id_behavior_outer : 2;
	u_int8_t df : 1;
	u_int8_t reserved : 5;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 5;
	u_int8_t df : 1;
	u_int8_t ip_id_behavior_outer : 2;
#endif
	u_int8_t tos_tc;
	u_int8_t ttl_hopl;
	u_int16_t ip_id;
} ipv4_outer_dynamic2_t;

/**
 * @brief Define the IP v4 outer irregular
 *
 * See RFC5225 page 62
 */

typedef struct __attribute__((packed)) ipv4_outer_irregular
{
	u_int16_t ip_id;              // [ 0, 16 ];
	u_int8_t tos_tc;              // [ 0, 8 ];
	u_int8_t ttl_hopl;            // [ 0, 8 ];
} ipv4_outer_irregular_t;

/**
 * @brief Define the IP v4 innermost irregular
 *
 * See RFC5225 page 62
 */

typedef struct __attribute__((packed)) ipv4_innermost_irregular
{
	u_int16_t ip_id;              // [ 0, 16 ];
} ipv4_innermost_irregular_t;

/*
 * @brief Define the IP common compressed format
 *
 * See RFC5225 page 90
 */

typedef struct __attribute__((packed)) ip_co_common
{
	u_int8_t discriminator;             // =:= '11111010'                     [ 8 ];
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE,THIS.LENGTH)  [ 7 ];
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];

	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
	u_int8_t reorder_ratio : 2;         // =:= irregular(2)                   [ 2 ];
	u_int8_t tos_tc_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;    // =:= irregular(1)                   [ 1 ];
	u_int8_t flags_indicator : 1;       // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE,THIS.LENGTH)  [ 7 ];

	u_int8_t flags_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;    // =:= irregular(1)                   [ 1 ];
	u_int8_t tos_tc_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t reorder_ratio : 2;         // =:= irregular(2)                   [ 2 ];
	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
#endif
	// outer_ip_indicator : df : ip_id_behavior_innermost
	//                =:= profile_2_3_4_flags_enc(flags_indicator.CVALUE, ip_version.UVALUE) [ 0, 8 ];
	// tos_tc =:= static_or_irreg(tos_tc_indicator.CVALUE, 8)                                [ 0, 8 ];
	// ttl_hopl =:= static_or_irreg(ttl_hopl_indicator.CVALUE, ttl_hopl.ULENGTH)             [ 0, 8 ];
	// msn =:= msn_lsb(8)                                                                    [ VARIABLE ];
	// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,
	//                                     ip_id_indicator.CVALUE)                           [ 0, 8, 16 ];
} ip_co_common_t;

/*
 * @brief Define the IP UO-0 compressed format
 *
 * See RFC5225 page 91
 */

typedef struct __attribute__((packed)) ip_pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                                [ 0 ];
} ip_pt_0_crc3_t;

/*
 * @brief Define the new IP format, type 0 with strong CRC and more SN bits
 *
 * See RFC5225 page 91
 */

typedef struct __attribute__((packed)) ip_pt_0_crc7
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 5;                  // =:= msn_lsb(6)                     [ 6 ];
	u_int8_t discriminator : 3;         // =:= '100'                          [ 3 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
	u_int8_t msn2 : 1;                  // =:= msn_lsb(6)                     [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '100'                          [ 3 ];
	u_int16_t msn : 6;                  // =:= msn_lsb(6)                     [ 6 ];
	u_int16_t header_crc : 7;           // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                                [ 0 ];
} ip_pt_0_crc7_t;

/*
 * @brief Define the IP UO-1-ID format, (PT-1 only used for sequential)
 *
 * See RFC5225 page 91
 */

typedef struct __attribute__((packed)) ip_pt_1_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 2;                  // =:= msn_lsb(6)                     [ 6 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
	u_int8_t discriminator : 3;         // =:= '101'                          [ 3 ];

	u_int8_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
	u_int8_t msn2 : 4;                  // =:= msn_lsb(6)                     [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '101'                          [ 3 ];
	u_int16_t header_crc : 3;           // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
	u_int16_t msn : 6;                  // =:= msn_lsb(6)                     [ 6 ];
	u_int16_t ip_id : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
#endif
} ip_pt_1_seq_id_t;

/*
 * @brief Define the IP UOR-2-ID replacement compressed format
 *
 * See RFC5225 page 91
 */

typedef struct __attribute__((packed)) ip_pt_2_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id1 : 5;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
	u_int8_t discriminator : 3;         // =:= '110'                          [ 3 ];

	u_int8_t header_crc : 7;           // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '110'                          [ 3 ];
	u_int16_t ip_id : 6;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
	u_int16_t header_crc : 7;           // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
#endif
	u_int8_t msn : 8;                   // =:= msn_lsb(8)                     [ 8 ];
} ip_pt_2_seq_id_t;

/***********************************************************/
/*                UDP                                      */
/***********************************************************/

/**
 * @brief Define the UDP header
 *
 * See RFC5225 page 63
 */

typedef struct __attribute__((packed)) base_header_udp
{
	u_int16_t src_port;
	u_int16_t dst_port;
	u_int16_t udp_length;
	u_int16_t checksum;
} base_header_udp_t;

/**
 * @brief Define the UDP static part
 *
 * See RFC5225 page 63
 */

typedef struct __attribute__((packed)) udp_static
{
	u_int16_t src_port;
	u_int16_t dst_port;
} udp_static_t;

/*
 * @brief Define the UDP endpoint dynamic part
 *
 * See RFC5225 page 63
 */

typedef struct __attribute__((packed)) udp_endpoint_dynamic
{
	u_int16_t checksum;
	u_int16_t msn;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reorder_ratio : 2;
	u_int8_t reserved : 6;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 6;
	u_int8_t reorder_ratio : 2;
#endif
} udp_endpoint_dynamic_t;

/*
 * @brief Define the UDP regular dynamic part
 *
 * See RFC5225 page 63
 */

typedef struct __attribute__((packed)) udp_regular_dynamic
{
	u_int16_t checksum;
} udp_regular_dynamic_t;

/*
 * @brief Define the UDP irregular with checksum part
 *
 * See RFC5225 page 64
 */

typedef struct __attribute__((packed)) udp_with_checksum_irregular
{
	u_int16_t checksum;
} udp_with_checksum_irregular_t;

/*
 * @brief Define the UDP common compressed format
 *
 * See RFC5225 page 85
 */

typedef struct __attribute__((packed)) udp_co_common
{
	u_int8_t discriminator;             // =:= '11111010'                     [ 8 ];

#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;            // =:= crc7(THIS.VALUE,THIS.ULENGTH)  [ 7 ];
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];

	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
	u_int8_t reorder_ratio : 2;         // =:= irregular(2)                   [ 2 ];
	u_int8_t tos_tc_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;    // =:= irregular(1)                   [ 1 ];
	u_int8_t flags_indicator : 1;       // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.VALUE,THIS.ULENGTH)  [ 7 ];

	u_int8_t flags_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;    // =:= irregular(1)                   [ 1 ];
	u_int8_t tos_tc_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t reorder_ratio : 2;         // =:= irregular(2)                   [ 2 ];
	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
#endif

// outer_ip_indicator : df : ip_id_behavior_innermost : tos_tc
//       =:= static_or_irreg(ttl_hopl_indicator.CVALUE,ttl_hopl.ULENGTH) [ 0, 8 ];
//	msn =:= msn_lsb(8)                                                    [ 8 ];
// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,ip_id_indicator.CVALUE) [ 0, 8, 16 ];
//
} udp_co_common_t;

/*
 * @brief Define the profile_2_3_4_flags encoding if present
 *
 * See RFC5225 page 76
 */

typedef union __attribute__((packed)) profile_2_3_4_flags
{
	struct __attribute__((packed))
	{
#if __BYTE_ORDER == __LITTLE_ENDIAN
		u_int8_t reserved : 4;             // =:= compressed_value(4, 0 )        [ 4 ];
		u_int8_t ip_id_behavior : 2;       // =:= irregular(2)                   [ 2 ];
		u_int8_t df : 1;                   // =:= dont_fragment(ip_version)      [ 1 ];
		u_int8_t ip_outer_indicator : 1;   // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
		u_int8_t ip_outer_indicator : 1;   // =:= irregular(1)                   [ 1 ];
		u_int8_t df : 1;                   // =:= dont_fragment(ip_version)      [ 1 ];
		u_int8_t ip_id_behavior : 2;       // =:= irregular(2)                   [ 2 ];
		u_int8_t reserved : 4;             // =:= compressed_value(4, 0 )        [ 4 ];
#endif
	} name;
	u_int8_t flags;
} profile_2_3_4_flags_t;

/*
 * @brief Define the UDP UO-0 compressed format
 *
 * See RFC5225 page 85
 */

typedef struct __attribute__((packed)) udp_pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;            // =:= crc3(THIS.VALUE,THIS.ULENGTH)  [ 3 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.VALUE,THIS.ULENGTH)  [ 3 ];
#endif
	//	ip_id =:= inferred_sequential_ip_id                                   [ 0 ];
} udp_pt_0_crc3_t;

/*
 * @brief Define the UDP New compressed format, type 0 with strong CRC and more SN bits
 *
 * See RFC5225 page 85
 */

typedef struct __attribute__((packed)) udp_pt_0_crc7
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 5;                  // =:= msn_lsb(6)                     [ 6 ];
	u_int8_t discriminator : 3;         // =:= '100'                          [ 3 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.VALUE,THIS.ULENGTH)  [ 7 ];
	u_int8_t msn2 : 1;                  // =:= msn_lsb(6)                     [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '100'                          [ 3 ];
	u_int16_t msn : 6;                  // =:= msn_lsb(6)                     [ 6 ];
	u_int16_t header_crc : 7;           // =:= crc7(THIS.VALUE,THIS.ULENGTH)  [ 7 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                    [ 0 ];
} udp_pt_0_crc7_t;

/*
 * @brief Define the UDP UO-1-ID compressed format
 *
 * See RFC5225 page 86
 */

typedef struct __attribute__((packed)) udp_pt_1_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 2;                  // =:= msn_lsb(6)                                     [ 6 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.VALUE,THIS.ULENGTH)                  [ 3 ];
	u_int8_t discriminator : 3;         // =:= '101'                                          [ 3 ];

	u_int8_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4) [ 4 ];
	u_int8_t msn2 : 4;                  // =:= msn_lsb(6)                                     [ 6 ];

#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '101'                                          [ 3 ];
	u_int16_t header_crc : 3;           // =:= crc3(THIS.VALUE,THIS.ULENGTH)                  [ 3 ];
	u_int16_t msn : 6;                  // =:= msn_lsb(6)                                     [ 6 ];
	u_int16_t ip_id : 4;                // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4) [ 4 ];
#endif
} udp_pt_1_seq_id_t;

/*
 * @brief Define the UDP UO-2-ID compressed format
 *
 * See RFC5225 page 86
 */

typedef struct __attribute__((packed)) udp_pt_2_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id1 : 5;                // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 6) [ 6 ];
	u_int8_t discriminator : 3;         // =:= '101'                                          [ 3 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.VALUE,THIS.ULENGTH)                  [ 7 ];
	u_int8_t ip_id2 : 1;                 // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4) [ 4 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '110'                                          [ 3 ];
	u_int16_t ip_id : 6;                // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 6) [ 6 ];
	u_int16_t header_crc : 7;           // =:= crc7(THIS.VALUE,THIS.ULENGTH)                  [ 7 ];
#endif
	u_int8_t msn;                       // =:= msn_lsb(8)                                     [ 8 ];
} udp_pt_2_seq_id_t;

/***********************************************************/
/*                RTP                                      */
/***********************************************************/

/**
 * @brief Define the RTP header without CSRC identifiers
 *
 * See RFC5225 page 64
 */

typedef struct __attribute__((packed)) base_header_rtp
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t cc : 4;
	u_int8_t extension : 1;
	u_int8_t pad_bit : 1;
	u_int8_t rtp_version : 2;

	u_int8_t payload_type : 7;
	u_int8_t marker : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t rtp_version : 2;
	u_int8_t pad_bit : 1;
	u_int8_t extension : 1;
	u_int8_t cc : 4;

	u_int8_t marker : 1;
	u_int8_t payload_type : 7;
#endif
	u_int16_t sequence_number;
	u_int32_t timestamp;
	u_int32_t ssrc;
} base_header_rtp_t;

/**
 * @brief Define the RTP header with CSRC identifiers
 *
 * See RFC5225 page 64
 */

typedef struct __attribute__((packed)) base_header_rtp_with_csrc
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t cc : 4;
	u_int8_t extension : 1;
	u_int8_t pad_bit : 1;
	u_int8_t rtp_version : 2;

	u_int8_t payload_type : 7;
	u_int8_t marker : 1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t rtp_version : 2;
	u_int8_t pad_bit : 1;
	u_int8_t extension : 1;
	u_int8_t cc : 4;

	u_int8_t marker : 1;
	u_int8_t payload_type : 7;
#endif
	u_int16_t sequence_number;
	u_int32_t timestamp;
	u_int32_t ssrc;
	u_int32_t csrc_list[1];
} base_header_rtp_with_csrc_t;

#define size_of_base_header_rtp_with_csrc(ptr)  (int)(sizeof(base_header_rtp_t) + \
                                                      (ptr->cc * sizeof(u_int32_t)))

/*
 * @brief Define the RTP static part
 *
 * See RFC5225 page 64
 */

typedef struct __attribute__((packed)) rtp_static
{
	u_int32_t ssrc;
} rtp_static_t;

/*
 * @brief Define the RTP dynamic part
 *
 * See RFC5225 page 64
 */

typedef struct __attribute__((packed)) rtp_dynamic
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t extension : 1;             // =:= irregular(1)                   [ 1 ];
	u_int8_t pad_bit : 1;               // =:= irregular(1)                   [ 1 ];
	u_int8_t tis_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t tss_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t list_present : 1;          // =:= irregular(1)                   [ 1 ];
	u_int8_t reorder_ratio : 2;         // =:= irregular(2)                   [ 2 ];
	u_int8_t reserved : 1;              // =:= compressed_value(1, 0)         [ 1 ];

	u_int8_t payload_type : 7;          // =:= irregular(7)                   [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 1;              // =:= compressed_value(1, 0)         [ 1 ];
	u_int8_t reorder_ratio : 2;         // =:= irregular(2)                   [ 2 ];
	u_int8_t list_present : 1;          // =:= irregular(1)                   [ 1 ];
	u_int8_t tss_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t tis_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t pad_bit : 1;               // =:= irregular(1)                   [ 1 ];
	u_int8_t extension : 1;             // =:= irregular(1)                   [ 1 ];

	u_int8_t marker : 1;                // =:= irregular(1)                   [ 1 ];
	u_int8_t payload_type : 7;          // =:= irregular(7)                   [ 7 ];
#endif
	u_int16_t sequence_number;          // =:= irregular(16)                  [ 16 ];
	u_int32_t timestamp;                // =:= irregular(32)                  [ 32 ];
// ts_stride =:= sdvl_or_default(tss_indicator.CVALUE, TS_STRIDE_DEFAULT)    [ VARIABLE ];
// time_stride =:= sdvl_or_default(tis_indicator.CVALUE, TIME_STRIDE_DEFAULT ) [ VARIABLE ];
// csrc_list =:= csrc_list_dynchain(list_present.CVALUE, cc.UVALUE )         [ VARIABLE ];
} rtp_dynamic_t;

/*
 * @brief Define the profile_1_7_flags1 encoding if present
 *
 * See RFC5225 page 75
 */
typedef union __attribute__((packed)) profile_1_7_flags1
{
	struct __attribute__((packed)) {
#if __BYTE_ORDER == __LITTLE_ENDIAN
		u_int8_t reorder_ratio : 2;        // =:= irregular(2)                   [ 2 ];
		u_int8_t ip_id_behavior : 2;       // =:= irregular(2)                   [ 2 ];
		u_int8_t df : 1;                   // =:= dont_fragment(ip_version)      [ 1 ];
		u_int8_t tos_tc_indicator : 1;     // =:= irregular(1)                   [ 1 ];
		u_int8_t ttl_hopl_indicator : 1;   // =:= irregular(1)                   [ 1 ];
		u_int8_t ip_outer_indicator : 1;   // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
		u_int8_t ip_outer_indicator : 1;   // =:= irregular(1)                   [ 1 ];
		u_int8_t ttl_hopl_indicator : 1;   // =:= irregular(1)                   [ 1 ];
		u_int8_t tos_tc_indicator : 1;     // =:= irregular(1)                   [ 1 ];
		u_int8_t df : 1;                   // =:= dont_fragment(ip_version)      [ 1 ];
		u_int8_t ip_id_behavior : 2;       // =:= irregular(2)                   [ 2 ];
		u_int8_t reorder_ratio : 2;        // =:= irregular(2)                   [ 2 ];
#endif
	} name;
	u_int8_t flags;
} profile_1_7_flags1_t;

/*
 * @brief Define the profile_1_flags2 encoding if present
 *
 * See RFC5225 page 75/76
 */
typedef union __attribute__((packed)) profile_1_flags2
{
	struct __attribute__((packed)) {
#if __BYTE_ORDER == __LITTLE_ENDIAN
		u_int8_t reserved : 3;             // =:= compressed_value(3, 0)         [ 3 ];
		u_int8_t extension : 1;            // =:= irregular(1)                   [ 1 ];
		u_int8_t pad_bit : 1;              // =:= irregular(1)                   [ 1 ];
		u_int8_t time_stride_indicator : 1; // =:= irregular(1)                   [ 1 ]; // Also referenced as tis_indicator
		u_int8_t pt_indicator : 1;         // =:= irregular(1)                   [ 1 ];
		u_int8_t list_indicator : 1;       // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
		u_int8_t list_indicator : 1;       // =:= irregular(1)                   [ 1 ];
		u_int8_t pt_indicator : 1;         // =:= irregular(1)                   [ 1 ];
		u_int8_t time_stride_indicator : 1; // =:= irregular(1)                   [ 1 ]; // Also referenced as tis_indicator
		u_int8_t pad_bit : 1;              // =:= irregular(1)                   [ 1 ];
		u_int8_t extension : 1;            // =:= irregular(1)                   [ 1 ];
		u_int8_t reserved : 3;             // =:= compressed_value(3, 0)         [ 3 ];
#endif
	} name;
	u_int8_t flags;
} profile_1_flags2_t;

/*
 * @brief Define the profile_7_flags2 encoding if present
 *
 * See RFC5225 page 77
 */
typedef union __attribute__((packed)) profile_7_flags2
{
	struct __attribute__((packed)) {
#if __BYTE_ORDER == __LITTLE_ENDIAN
		u_int8_t reserved : 1;             // =:= compressed_value(1, 0)         [ 1 ];
		u_int8_t coverage_behavior : 2;    // =:= irregular(2)                   [ 2 ];
		u_int8_t extension : 1;            // =:= irregular(1)                   [ 1 ];
		u_int8_t pad_bit : 1;              // =:= irregular(1)                   [ 1 ];
		u_int8_t time_stride_indicator : 1; // =:= irregular(1)                   [ 1 ]; // Also referenced as tis_indicator
		u_int8_t pt_indicator : 1;         // =:= irregular(1)                   [ 1 ];
		u_int8_t list_indicator : 1;       // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
		u_int8_t list_indicator : 1;       // =:= irregular(1)                   [ 1 ];
		u_int8_t pt_indicator : 1;         // =:= irregular(1)                   [ 1 ];
		u_int8_t time_stride_indicator : 1; // =:= irregular(1)                   [ 1 ]; // Also referenced as tis_indicator
		u_int8_t pad_bit : 1;              // =:= irregular(1)                   [ 1 ];
		u_int8_t extension : 1;            // =:= irregular(1)                   [ 1 ];
		u_int8_t coverage_behavior : 2;    // =:= irregular(2)                   [ 2 ];
		u_int8_t reserved : 1;             // =:= compressed_value(1, 0)         [ 1 ];
#endif
	} name;
	u_int8_t flags;
} profile_7_flags2_t;

/*
 * @brief Define the RTP common compressed format
 *
 * See RFC5225 page 80
 */

typedef struct __attribute__((packed)) rtp_co_common
{
	u_int8_t discriminator;             // =:= '11111010'                     [ 8 ];
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE,THIS.LENGTH)  [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                   [ 1 ];

	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t tss_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t tsc_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t flags2_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t flags1_indicator : 1;      // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t marker : 1;                // =:= irregular(1)                   [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE,THIS.LENGTH)  [ 7 ];

	u_int8_t flags1_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t flags2_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t tsc_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t tss_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
#endif
	// outer_ip_indicator : ttl_hopl_indicator : tos_tc_indicator : df :
	// ip_id_behavior_innermost : reorder_ratio
	//                =:= profile_1_7_flags1_enc(flags1_indicator.CVALUE, ip_version.UVALUE) [ 0, 8 ];
	// list_indicator : pt_indicator : tis_indicator : pad_bit : extension
	//                         =:= profile_1_flags2_enc(flags2_indicator.CVALUE)             [ 0, 8 ];
	// tos_tc =:= static_or_irreg(tos_tc_indicator.CVALUE, 8)                                [ 0, 8 ];
	// ttl_hopl =:= static_or_irreg(ttl_hopl_indicator.CVALUE, ttl_hopl.ULENGTH)             [ 0, 8 ];
	// payload_type =:= pt_irr_or_static(pt_indicator)                                       [ 0, 8 ];
	// sequence_number =:= sdvl_sn_lsb(sequence_number.ULENGTH)                              [ VARIABLE ];
	// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,
	//                                     ip_id_indicator.CVALUE)                           [ 0, 8, 16 ];
	// ts_scaled =:= variable_scaled_timestamp(tss_indicator.CVALUE, tsc_indicator.CVALUE,
	//                ts_stride.UVALUE, time_stride.UVALUE)                                  [ VARIABLE ];
	// timestamp =:= variable_unscaled_timestamp(tss_indicator.CVALUE, tsc_indicator.CVALUE) [ VARIABLE ];
	// ts_stride =:= sdvl_or_static(tss_indicator.CVALUE)                                    [ VARIABLE ];
	// time_stride =:= sdvl_or_static(tis_indicator.CVALUE)                                  [ VARIABLE ];
	// csrc_list =:= csrc_list_presence(list_indicator.CVALUE)                               [ VARIABLE ];
} rtp_co_common_t;

/*
 * @brief Define the RTP UO-0 compressed format
 *
 * See RFC5225 page 81
 */

typedef struct __attribute__((packed)) rtp_pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
#endif
	// timestamp =:= inferred_scaled_field                                    [ 0 ];
	// ip_id =:= inferred_sequential_ip_id                                    [ 0 ];
} rtp_pt_0_crc3_t;


/*
 * @brief Define the RTP Type 0 with strong CRC and more SN bits, compressed format
 *
 * See RFC5225 page 81
 */

typedef struct __attribute__((packed)) rtp_pt_0_crc7
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 4;                  // =:= msn_lsb(5)                     [ 5 ];
	u_int8_t discriminator : 4;         // =:= '1000'                         [ 4 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
	u_int8_t msn2 : 1;                  // =:= msn_lsb(5)                     [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 4;        // =:= '1000'                         [ 4 ];
	u_int16_t msn : 5;                  // =:= msn_lsb(5)                     [ 5 ];
	u_int16_t header_crc : 7;           // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
#endif
	// timestamp =:= inferred_scaled_field                                    [ 0 ];
	// ip_id =:= inferred_sequential_ip_id                                    [ 0 ];
} rtp_pt_0_crc7_t;


/*
 * @brief Define the RTP UO-1 replacement compressed format
 *
 * See RFC5225 page 81
 */

typedef struct __attribute__((packed)) rtp_pt_1_rnd
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];

	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];

	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
#endif
} rtp_pt_1_rnd_t;

/*
 * @brief Define the RTP UO-1-ID replacement compressed format
 *
 * See RFC5225 page 82
 */

typedef struct __attribute__((packed)) rtp_pt_1_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
	u_int8_t discriminator : 4;         // =:= '1001'                                        [ 4 ];

	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)               [ 3 ];
	u_int8_t msn : 5;                   // =:= msn_lsb(5)                                    [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 4;         // =:= '1001'                                        [ 4 ];
	u_int8_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];

	u_int8_t msn : 5;                   // =:= msn_lsb(5)                                    [ 5 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)               [ 3 ];
#endif
	// timestamp =:= inferred_scaled_field                                                   [ 0 ];
} rtp_pt_1_seq_id_t;

/*
 * @brief Define the RTP UO-1-TS replacement compressed format
 *
 * See RFC5225 page 82
 */

typedef struct __attribute__((packed)) rtp_pt_1_seq_ts
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];

	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];

	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                          [ 0 ];
} rtp_pt_1_seq_ts_t;

/*
 * @brief Define the RTP UOR-2 replacement compressed format
 *
 * See RFC5225 page 82
 */

typedef struct __attribute__((packed)) rtp_pt_2_rnd
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 5;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int8_t discriminator : 3;         // =:= '110'                                [ 3 ];

	u_int8_t ts_scaled : 6;             // =:= scaled_ts_lsb(time_stride.UVALUE, 6) [ 6 ];
	u_int8_t msn2 : 2;                  // =:= msn_lsb(7)                           [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '110'                                [ 3 ];
	u_int16_t msn : 7;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int16_t ts_scaled : 6;            // =:= scaled_ts_lsb(time_stride.UVALUE, 6) [ 6 ];

	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
#endif
} rtp_pt_2_rnd_t;

/*
 * @brief Define the RTP UOR-2-ID replacement compressed format
 *
 * See RFC5225 page 82/83
 */

typedef struct __attribute__((packed)) rtp_pt_2_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                    [ 7 ];
	u_int8_t discriminator : 5;         // =:= '11000'                                       [ 5 ];

	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];
	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                    [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)               [ 7 ];
	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 5;         // =:= '11000'                                       [ 5 ];
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                    [ 7 ];

	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                    [ 7 ];
	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];

	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)               [ 7 ];
#endif
	// timestamp =:= inferred_scaled_field                                                   [ 0 ];
} rtp_pt_2_seq_id_t;

/*
 * @brief Define the RTP UOR-2-ID-ext1 replacement compressed format
 *
 * See RFC5225 page 83 (both TS and IP-ID)
 */

typedef struct __attribute__((packed)) rtp_pt_2_seq_both
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                     [ 7 ];
	u_int8_t discriminator : 5;         // =:= '11001'                                        [ 5 ];

	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];
	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                     [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)                [ 7 ];
	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];

	u_int8_t marker : 1;                // =:= irregular(1)                                   [ 1 ];
	u_int8_t ts_scaled : 7;             // =:= scaled_ts_lsb(time_stride.UVALUE, 7)           [ 7 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 5;         // =:= '11001'                                        [ 5 ];
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                     [ 7 ];

	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                     [ 7 ];
	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];

	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)                [ 7 ];

	u_int8_t ts_scaled : 7;             // =:= scaled_ts_lsb(time_stride.UVALUE, 7)           [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                                   [ 1 ];
#endif
} rtp_pt_2_seq_both_t;

/*
 * @brief Define the RTP UOR-2-TS replacement compressed format
 *
 * See RFC5225 page 83
 */

typedef struct __attribute__((packed)) rtp_pt_2_seq_ts
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 4;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int8_t discriminator : 4;         // =:= '1101'                               [ 4 ];

	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
	u_int8_t msn2 : 3;                  // =:= msn_lsb(7)                           [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 4;         // =:= '1101'                               [ 4 ];
	u_int8_t msn1 : 4;                  // =:= msn_lsb(7)                           [ 7 ];

	u_int8_t msn2 : 3;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];

	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                          [ 0 ];
} rtp_pt_2_seq_ts_t;

/***********************************************************/
/*                UDP-Lite                                 */
/***********************************************************/

/**
 * @brief Define the UDP-Lite header
 *
 * See RFC5225 page 67
 */

typedef struct __attribute__((packed)) base_header_udp_lite
{
	u_int16_t src_port;
	u_int16_t dst_port;
	u_int16_t checksum_coverage;
	u_int16_t checksum;
} base_header_udp_lite_t;

/**
 * @brief Define the UDP-Lite static part
 *
 * See RFC5225 page 67
 */

typedef struct __attribute__((packed)) udp_lite_static
{
	u_int16_t src_port;
	u_int16_t dst_port;
} udp_lite_static_t;

/**
 * @brief Define the UDP-Lite endpoint dynamic part
 *
 * See RFC5225 page 67 (and Errata Verified 3248)
 */

typedef struct __attribute__((packed)) udp_lite_endpoint_dynamic1
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reorder_ratio : 2;       // =:= irregular(2)                                          [ 2 ];
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
	u_int8_t reserved : 4;            // =:= compressed_value(4, 0)                                [ 4 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 4;            // =:= compressed_value(4, 0)                                [ 4 ];
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
	u_int8_t reorder_ratio : 2;       // =:= irregular(2)                                          [ 2 ];
#endif
//	u_int16_t	checksum_coverage;    // =:= checksum_coverage_dynchain(coverage_behavior.UVALUE)  [ 0, 16 ];
	u_int16_t checksum;               // =:= irregular(16)                                         [ 16 ];
	u_int16_t msn;                    // =:= irregular(16)                                         [ 16 ];
} udp_lite_endpoint_dynamic1_t;

typedef struct __attribute__((packed)) udp_lite_endpoint_dynamic2
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reorder_ratio : 2;       // =:= irregular(2)                                          [ 2 ];
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
	u_int8_t reserved : 4;            // =:= compressed_value(4, 0)                                [ 4 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 4;            // =:= compressed_value(4, 0)                                [ 4 ];
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
	u_int8_t reorder_ratio : 2;       // =:= irregular(2)                                          [ 2 ];
#endif
	u_int16_t checksum_coverage;      // =:= checksum_coverage_dynchain(coverage_behavior.UVALUE)  [ 0, 16 ];
	u_int16_t checksum;               // =:= irregular(16)                                         [ 16 ];
	u_int16_t msn;                    // =:= irregular(16)                                         [ 16 ];
} udp_lite_endpoint_dynamic2_t;

/**
 * @brief Define the UDP-Lite regular dynamic part
 *
 * See RFC5225 page 68 (and Errata Verified 3248)
 */

typedef struct __attribute__((packed)) udp_lite_regular_dynamic1
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reserved : 6;            // =:= compressed_value(6, 0)                                [ 6 ];
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
	u_int8_t reserved : 6;            // =:= compressed_value(6, 0)                                [ 6 ];
#endif
//	u_int16_t	checksum_coverage;    // =:= checksum_coverage_dynchain(coverage_behavior.UVALUE)  [ 0, 16 ];
	u_int16_t checksum;               // =:= irregular(16)                                         [ 16 ];
} udp_lite_regular_dynamic1_t;

typedef struct __attribute__((packed)) udp_lite_regular_dynamic2
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reserved : 6;            // =:= compressed_value(6, 0)                                [ 6 ];
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t coverage_behavior : 2;   // =:= irregular(2)                                          [ 2 ];
	u_int8_t reserved : 6;            // =:= compressed_value(6, 0)                                [ 6 ];
#endif
	u_int16_t checksum_coverage;      // =:= checksum_coverage_dynchain(coverage_behavior.UVALUE)  [ 0, 16 ];
	u_int16_t checksum;               // =:= irregular(16)                                         [ 16 ];
} udp_lite_regular_dynamic2_t;

/**
 * @brief Define the UDP-Lite irregular part
 *
 * See RFC5225 page 68
 */

typedef struct __attribute__((packed)) udp_lite_irregular1
{
//	u_int16_t	checksum_coverage;    // =:= checksum_coverage_irregular(coverage_behavior.UVALUE) [ 0, 16 ];
	u_int16_t checksum;               // =:= irregular(16)                                         [ 16 ];
} udp_lite_irregular1_t;

typedef struct __attribute__((packed)) udp_lite_irregular2
{
	u_int16_t checksum_coverage;      // =:= checksum_coverage_irregular(coverage_behavior.UVALUE) [ 0, 16 ];
	u_int16_t checksum;               // =:= irregular(16)                                         [ 16 ];
} udp_lite_irregular2_t;

/*
 * @brief Define the UDP-Lite common compressed format
 *
 * See RFC5225 page 99
 */

typedef struct __attribute__((packed)) udp_lite_co_common
{
	u_int8_t discriminator;           // =:= '11111010'                               [ 8 ];

#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;          // =:= crc7(THIS.VALUE,THIS.ULENGTH)            [ 7 ];
	u_int8_t ip_id_indicator : 1;     // =:= irregular(1)                             [ 1 ];

	u_int8_t control_crc3 : 3;        // =:= control_crc3_encoding                    [ 3 ];
	u_int8_t reorder_ratio : 2;       // =:= irregular(2)                             [ 2 ];
	u_int8_t tos_tc_indicator : 1;    // =:= irregular(1)                             [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;  // =:= irregular(1)                             [ 1 ];
	u_int8_t flags_indicator : 1;     // =:= irregular(1)                             [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t ip_id_indicator : 1;     // =:= irregular(1)                             [ 1 ];
	u_int8_t header_crc : 7;          // =:= crc7(THIS.VALUE,THIS.ULENGTH)            [ 7 ];

	u_int8_t flags_indicator : 1;     // =:= irregular(1)                             [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;  // =:= irregular(1)                             [ 1 ];
	u_int8_t tos_tc_indicator : 1;    // =:= irregular(1)                             [ 1 ];
	u_int8_t reorder_ratio : 2;       // =:= irregular(2)                             [ 2 ];
	u_int8_t control_crc3 : 3;        // =:= control_crc3_encoding                    [ 3 ];
#endif

// outer_ip_indicator : df : ip_id_behavior_innermost : coverage_behavior
//            =:= profile_8_flags_enc(flags_indicator.CVALUE, ip_version.UVALUE)               [ 0, 8 ];
// tos_tc =:= static_or_irreg(tos_tc_indicator.CVALUE, 8)                                      [ 0, 8 ];
// ttl_hopl =:= static_or_irreg(ttl_hopl_indicator.CVALUE, ttl_hopl.ULENGTH)                   [ 0, 8 ];
// msn =:= msn_lsb(8)                                                                          [ 8 ];
// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,ip_id_indicator.CVALUE) [ 0, 8, 16 ];
//
} udp_lite_co_common_t;

/*
 * @brief Define the profile_8_flags encoding if present
 *
 * See RFC5225 page 77
 */

typedef struct __attribute__((packed)) profile_8_flags
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t coverage_behavior : 2;     // =:= irregular(2)                   [ 2 ];
	u_int8_t ip_id_behavior : 2;        // =:= irregular(2)                   [ 2 ];
	u_int8_t df : 1;                    // =:= dont_fragment(ip_version)      [ 1 ];
	u_int8_t ip_outer_indicator : 1;    // =:= irregular(1)                   [ 1 ];
	u_int8_t reserved : 2;              // =:= compressed_value(2, 0 )        [ 2 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 2;              // =:= compressed_value(2, 0 )        [ 2 ];
	u_int8_t ip_outer_indicator : 1;    // =:= irregular(1)                   [ 1 ];
	u_int8_t df : 1;                    // =:= dont_fragment(ip_version)      [ 1 ];
	u_int8_t ip_id_behavior : 2;        // =:= irregular(2)                   [ 2 ];
	u_int8_t coverage_behavior : 2;     // =:= irregular(2)                   [ 2 ];
#endif
} profile_8_flags_t;

/*
 * @brief Define the UDP-Lite UO-0 compressed format
 *
 * See RFC5225 page 99
 */

typedef struct __attribute__((packed)) udp_lite_pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;           // =:= crc3(THIS.VALUE,THIS.ULENGTH)  [ 3 ];
	u_int8_t msn : 4;                  // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t discriminator : 1;        // =:= '0'                            [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;        // =:= '0'                            [ 1 ];
	u_int8_t msn : 4;                  // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t header_crc : 3;           // =:= crc3(THIS.VALUE,THIS.ULENGTH)  [ 3 ];
#endif
//	ip_id =:= inferred_sequential_ip_id                                      [ 0 ];
} udp_lite_pt_0_crc3_t;

/*
 * @brief Define the UDP-Lite New compressed format, type 0 with strong CRC and more SN bits
 *
 * See RFC5225 page 85
 */

typedef struct __attribute__((packed)) udp_lite_pt_0_crc7
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 5;                 // =:= msn_lsb(6)                     [ 6 ];
	u_int8_t discriminator : 3;        // =:= '100'                          [ 3 ];

	u_int8_t header_crc : 7;           // =:= crc7(THIS.VALUE,THIS.ULENGTH)  [ 7 ];
	u_int8_t msn2 : 1;                 // =:= msn_lsb(6)                     [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;       // =:= '100'                          [ 3 ];
	u_int16_t msn : 6;                 // =:= msn_lsb(6)                     [ 6 ];
	u_int16_t header_crc : 7;          // =:= crc7(THIS.VALUE,THIS.ULENGTH)  [ 7 ];
#endif
//	ip_id =:= inferred_sequential_ip_id                                      [ 0 ];
} udp_lite_pt_0_crc7_t;

/*
 * @brief Define the UDP UO-1-ID compressed format
 *
 * See RFC5225 page 86
 */

typedef struct __attribute__((packed)) udp_lite_pt_1_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 2;                 // =:= msn_lsb(6)                                     [ 6 ];
	u_int8_t header_crc : 3;           // =:= crc3(THIS.VALUE,THIS.ULENGTH)                  [ 3 ];
	u_int8_t discriminator : 3;        // =:= '101'                                          [ 3 ];

	u_int8_t ip_id : 4;                // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4) [ 4 ];
	u_int8_t msn2 : 4;                 // =:= msn_lsb(6)                                     [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;       // =:= '101'                                          [ 3 ];
	u_int16_t header_crc : 3;          // =:= crc3(THIS.VALUE,THIS.ULENGTH)                  [ 3 ];
	u_int16_t msn : 6;                 // =:= msn_lsb(6)                                     [ 6 ];
	u_int16_t ip_id : 4;               // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4) [ 4 ];
#endif
} udp_lite_pt_1_seq_id_t;

/*
 * @brief Define the UDP UO-2-ID compressed format
 *
 * See RFC5225 page 86
 */

typedef struct __attribute__((packed)) udp_lite_pt_2_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id1 : 5;               // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 6) [ 6 ];
	u_int8_t discriminator : 3;        // =:= '101'                                          [ 3 ];

	u_int8_t header_crc : 7;           // =:= crc7(THIS.VALUE,THIS.ULENGTH)                  [ 7 ];
	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4) [ 4 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;       // =:= '110'                                          [ 3 ];
	u_int16_t ip_id : 6;               // =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 6) [ 6 ];
	u_int16_t header_crc : 7;          // =:= crc7(THIS.VALUE,THIS.ULENGTH)                  [ 7 ];
#endif
	u_int8_t msn;                      // =:= msn_lsb(8)                                     [ 8 ];
} udp_lite_pt_2_seq_id_t;

/***********************************************************/
/*                UDP-Lite / RTP                           */
/***********************************************************/

/**
 * @brief Define the UDP-Lite/RTP header
 *
 * See RFC5225 page 92
 */

/*
 * @brief Define the UDP-Lite / RTP common compressed format
 *
 * See RFC5225 page 94
 */

typedef struct __attribute__((packed)) udp_lite_rtp_co_common
{
	u_int8_t discriminator;             // =:= '11111010'                     [ 8 ];
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE,THIS.LENGTH)  [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                   [ 1 ];

	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t tss_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t tsc_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t flags2_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t flags1_indicator : 1;      // =:= irregular(1)                   [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t marker : 1;                // =:= irregular(1)                   [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE,THIS.LENGTH)  [ 7 ];

	u_int8_t flags1_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t flags2_indicator : 1;      // =:= irregular(1)                   [ 1 ];
	u_int8_t tsc_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t tss_indicator : 1;         // =:= irregular(1)                   [ 1 ];
	u_int8_t ip_id_indicator : 1;       // =:= irregular(1)                   [ 1 ];
	u_int8_t control_crc3 : 3;          // =:= control_crc3_encoding          [ 3 ];
#endif
	// outer_ip_indicator : ttl_hopl_indicator : tos_tc_indicator : df :
	// ip_id_behavior_innermost : reorder_ratio
	//                =:= profile_1_7_flags1_enc(flags1_indicator.CVALUE, ip_version.UVALUE) [ 0, 8 ];
	// list_indicator : pt_indicator : tis_indicator : pad_bit : extension
	//                         =:= profile_1_flags2_enc(flags2_indicator.CVALUE)             [ 0, 8 ];
	// tos_tc =:= static_or_irreg(tos_tc_indicator.CVALUE, 8)                                [ 0, 8 ];
	// ttl_hopl =:= static_or_irreg(ttl_hopl_indicator.CVALUE, ttl_hopl.ULENGTH)             [ 0, 8 ];
	// payload_type =:= pt_irr_or_static(pt_indicator)                                       [ 0, 8 ];
	// sequence_number =:= sdvl_sn_lsb(sequence_number.ULENGTH)                              [ VARIABLE ];
	// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,
	//                                     ip_id_indicator.CVALUE)                           [ 0, 8, 16 ];
	// ts_scaled =:= variable_scaled_timestamp(tss_indicator.CVALUE, tsc_indicator.CVALUE,
	//                ts_stride.UVALUE, time_stride.UVALUE)                                  [ VARIABLE ];
	// timestamp =:= variable_unscaled_timestamp(tss_indicator.CVALUE, tsc_indicator.CVALUE) [ VARIABLE ];
	// ts_stride =:= sdvl_or_static(tss_indicator.CVALUE)                                    [ VARIABLE ];
	// time_stride =:= sdvl_or_static(tis_indicator.CVALUE)                                  [ VARIABLE ];
	// csrc_list =:= csrc_list_presence(list_indicator.CVALUE)                               [ VARIABLE ];
} udp_lite_rtp_co_common_t;

/*
 * @brief Define the UDP-Lite / RTP UO-0 compressed format
 *
 * See RFC5225 page 95
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
#endif
	// timestamp =:= inferred_scaled_field                                    [ 0 ];
	// ip_id =:= inferred_sequential_ip_id                                    [ 0 ];
} udp_lite_rtp_pt_0_crc3_t;


/*
 * @brief Define the UDP-Lite / RTP Type 0
 * with strong CRC and more SN bits, compressed format
 *
 * See RFC5225 page 95
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_0_crc7
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 4;                  // =:= msn_lsb(5)                     [ 5 ];
	u_int8_t discriminator : 4;         // =:= '1000'                         [ 4 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
	u_int8_t msn2 : 1;                  // =:= msn_lsb(5)                     [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 4;        // =:= '1000'                         [ 4 ];
	u_int16_t msn : 5;                  // =:= msn_lsb(5)                     [ 5 ];
	u_int16_t header_crc : 7;           // =:= crc7(THIS.UVALUE, THIS.ULENGTH)[ 7 ];
#endif
	// timestamp =:= inferred_scaled_field                                    [ 0 ];
	// ip_id =:= inferred_sequential_ip_id                                    [ 0 ];
} udp_lite_rtp_pt_0_crc7_t;


/*
 * @brief Define the UDP-Lite / RTP UO-1 replacement compressed format
 *
 * See RFC5225 page 95
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_1_rnd
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];

	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];

	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
#endif
} udp_lite_rtp_pt_1_rnd_t;


/*
 * @brief Define the UDP-Lite / RTP UO-1-ID replacement compressed format
 *
 * See RFC5225 page 96
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_1_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
	u_int8_t discriminator : 4;         // =:= '1001'                                        [ 4 ];

	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)               [ 3 ];
	u_int8_t msn : 5;                   // =:= msn_lsb(5)                                    [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 4;         // =:= '1001'                                        [ 4 ];
	u_int8_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];

	u_int8_t msn : 5;                   // =:= msn_lsb(5)                                    [ 5 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)               [ 3 ];
#endif
	// timestamp =:= inferred_scaled_field                                                   [ 0 ];
} udp_lite_rtp_pt_1_seq_id_t;

/*
 * @brief Define the UDP-Lite / RTP UO-1-TS replacement compressed format
 *
 * See RFC5225 page 96
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_1_seq_ts
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];

	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 3;         // =:= '101'                                [ 3 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                           [ 4 ];

	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)      [ 3 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                          [ 0 ];
} udp_lite_rtp_pt_1_seq_ts_t;

/*
 * @brief Define the UDP-Lite / RTP UOR-2 replacement compressed format
 *
 * See RFC5225 page 96
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_2_rnd
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 5;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int8_t discriminator : 3;         // =:= '110'                                [ 3 ];

	u_int8_t ts_scaled : 6;             // =:= scaled_ts_lsb(time_stride.UVALUE, 6) [ 6 ];
	u_int8_t msn2 : 2;                  // =:= msn_lsb(7)                           [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;        // =:= '110'                                [ 3 ];
	u_int16_t msn : 7;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int16_t ts_scaled : 6;            // =:= scaled_ts_lsb(time_stride.UVALUE, 6) [ 6 ];

	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
#endif
} udp_lite_rtp_pt_2_rnd_t;

/*
 * @brief Define the UDP-Lite / RTP UOR-2-ID replacement compressed format
 *
 * See RFC5225 page 96
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_2_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                    [ 7 ];
	u_int8_t discriminator : 5;         // =:= '11000'                                       [ 5 ];

	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];
	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                    [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)               [ 7 ];
	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 5;         // =:= '11000'                                       [ 5 ];
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                    [ 7 ];

	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                    [ 7 ];
	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];

	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5) [ 5 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)               [ 7 ];
#endif
	// timestamp =:= inferred_scaled_field                                                   [ 0 ];
} udp_lite_rtp_pt_2_seq_id_t;

/*
 * @brief Define the UDP-Lite / RTP UOR-2-ID-ext1 replacement compressed format
 *
 * See RFC5225 page 97 (both TS and IP-ID)
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_2_seq_both
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                     [ 7 ];
	u_int8_t discriminator : 5;         // =:= '11001'                                        [ 5 ];

	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];
	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                     [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)                [ 7 ];
	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];

	u_int8_t marker : 1;                // =:= irregular(1)                                   [ 1 ];
	u_int8_t ts_scaled : 7;             // =:= scaled_ts_lsb(time_stride.UVALUE, 7)           [ 7 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 5;         // =:= '11001'                                        [ 5 ];
	u_int8_t msn1 : 3;                  // =:= msn_lsb(7)                                     [ 7 ];

	u_int8_t msn2 : 4;                  // =:= msn_lsb(7)                                     [ 7 ];
	u_int8_t ip_id1 : 4;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];

	u_int8_t ip_id2 : 1;                // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 5)  [ 5 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)                [ 7 ];

	u_int8_t ts_scaled : 7;             // =:= scaled_ts_lsb(time_stride.UVALUE, 7)           [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                                   [ 1 ];
#endif
} udp_lite_rtp_pt_2_seq_both_t;

/*
 * @brief Define the UDP-Lite / RTP UOR-2-TS replacement compressed format
 *
 * See RFC5225 page 97
 */

typedef struct __attribute__((packed)) udp_lite_rtp_pt_2_seq_ts
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t msn1 : 4;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int8_t discriminator : 4;         // =:= '1101'                               [ 4 ];

	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];
	u_int8_t msn2 : 3;                  // =:= msn_lsb(7)                           [ 7 ];

	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 4;         // =:= '1101'                               [ 4 ];
	u_int8_t msn1 : 4;                  // =:= msn_lsb(7)                           [ 7 ];

	u_int8_t msn2 : 3;                  // =:= msn_lsb(7)                           [ 7 ];
	u_int8_t ts_scaled : 5;             // =:= scaled_ts_lsb(time_stride.UVALUE, 5) [ 5 ];

	u_int8_t marker : 1;                // =:= irregular(1)                         [ 1 ];
	u_int8_t header_crc : 7;            // =:= crc7(THIS.UVALUE, THIS.ULENGTH)      [ 7 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                          [ 0 ];
} udp_lite_rtp_pt_2_seq_ts_t;


/***********************************************************/
/*                ESP                                      */
/***********************************************************/

/**
 * @brief Define the ESP header
 *
 * See RFC5225 page 68
 */

typedef struct __attribute__((packed)) base_header_esp
{
	u_int32_t spi;
	u_int32_t sequence_number;
} base_header_esp_t;

/**
 * @brief Define the ESP static part
 *
 * See RFC5225 page 68
 */

typedef struct __attribute__((packed)) esp_static
{
	u_int32_t spi;
} esp_static_t;

/**
 * @brief Define the ESP dynamic part
 *
 * See RFC5225 page 68
 */

typedef struct __attribute__((packed)) esp_dynamic
{
	u_int32_t sequence_number;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t reorder_ratio : 2;
	u_int8_t reserved : 6;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t reserved : 6;
	u_int8_t reorder_ratio : 2;
#endif
} esp_dynamic_t;

/*
 * @brief Define the ESP common compressed format
 *
 * See RFC5225 page 88
 */

typedef struct __attribute__((packed)) esp_co_common
{
	u_int8_t discriminator;               // =:= '11111010'                        [ 8 ];
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;              // =:= crc7(THIS.UVALUE,THIS.ULENGTH)    [ 7 ];
	u_int8_t ip_id_indicator : 1;         // =:= irregular(1)                      [ 1 ];

	u_int8_t control_crc3 : 3;            // =:= irregular(3)                      [ 3 ];
	u_int8_t reorder_ratio : 2;           // =:= irregular(2)                      [ 2 ];
	u_int8_t tos_tc_indicator : 1;        // =:= irregular(1)                      [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;      // =:= irregular(1)                      [ 1 ];
	u_int8_t flags_indicator : 1;         // =:= irregular(1)                      [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t ip_id_indicator : 1;         // =:= irregular(1)                      [ 1 ];
	u_int8_t header_crc : 7;              // =:= crc7(THIS.UVALUE,THIS.ULENGTH)    [ 7 ];

	u_int8_t flags_indicator : 1;         // =:= irregular(1)                      [ 1 ];
	u_int8_t ttl_hopl_indicator : 1;      // =:= irregular(1)                      [ 1 ];
	u_int8_t tos_tc_indicator : 1;        // =:= irregular(1)                      [ 1 ];
	u_int8_t reorder_ratio : 2;           // =:= irregular(2)                      [ 2 ];
	u_int8_t control_crc3 : 3;            // =:= irregular(3)                      [ 3 ];
#endif
	// outer_ip_indicator : df : ip_id_behavior_innermost
	//	     =:= profile_2_3_4_flags_enc(flags_indicator.CVALUE, ip_version.UVALUE)           [ 0, 8 ];
	// tos_tc =:= static_or_irreg(tos_tc_indicator.CVALUE, 8)                                [ 0 , 8 ];
	// ttl_hopl =:= static_or_irreg(ttl_hopl_indicator.CVALUE, ttl_hopl.ULENGTH)             [ 0, 8 ];
	// sequence_number =:= sdvl_sn_lsb(sequence_number.ULENGTH)                              [ VARIABLE ];
	// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,
	//                                     ip_id_indicator.CVALUE)                           [ 0, 8, 16 ];
} esp_co_common_t;

/*
 * @brief Define the ESP pt_0_crc3 compressed format
 *
 * See RFC5225 page 88
 */

typedef struct __attribute__((packed)) esp_pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;            // =:= crc3((THIS.UVALUE,THIS.ULENGTH)     [ 3 ];
	u_int8_t sequence_number : 4;       // =:= msn_lsb(4)                          [ 4 ];
	u_int8_t discriminator : 1;         // =:= '0'                                 [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;         // =:= '0'                                 [ 1 ];
	u_int8_t sequence_number : 4;       // =:= msn_lsb(4)                          [ 4 ];
	u_int8_t header_crc : 3;            // =:= crc3((THIS.UVALUE,THIS.ULENGTH)     [ 3 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                         [ 0 ];
} esp_pt_0_crc3_t;

/*
 * @brief Define the ESP pt_0_crc7 compressed format
 *
 * See RFC5225 page 88
 */
typedef struct __attribute__((packed)) esp_pt_0_crc7
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t sequence_number1 : 5;       // =:= msn_lsb(6)                          [ 6 ];
	u_int8_t discriminator : 3;          // =:= '100'                               [ 3 ];

	u_int8_t header_crc : 7;             // =:= crc7((THIS.UVALUE,THIS.ULENGTH)     [ 7 ];
	u_int8_t sequence_number2 : 1;       // =:= msn_lsb(6)                          [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;         // =:= '100'                               [ 3 ];
	u_int16_t sequence_number : 6;       // =:= msn_lsb(6)                          [ 6 ];
	u_int16_t header_crc : 7;            // =:= crc7((THIS.UVALUE,THIS.ULENGTH)     [ 7 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                          [ 0 ];
} esp_pt_0_crc7_t;

/*
 * @brief Define the ESP pt_1_seq_id compressed format
 *
 * See RFC5225 page 88/89
 */

typedef struct __attribute__((packed)) esp_pt_1_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t sequence_number1 : 2;         // =:= msn_lsb(6)                                    [ 6 ];
	u_int8_t header_crc : 3;               // =:= crc3((THIS.UVALUE,THIS.ULENGTH)               [ 3 ];
	u_int8_t discriminator : 3;            // =:= '101'                                         [ 3 ];

	u_int8_t ip_id : 4;                    // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
	u_int8_t sequence_number2 : 4;         // =:= msn_lsb(6)                                    [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;         // =:= '101'                                         [ 3 ];
	u_int16_t header_crc : 3;            // =:= crc3((THIS.UVALUE,THIS.ULENGTH)               [ 3 ];
	u_int16_t sequence_number : 6;       // =:= msn_lsb(6)                                    [ 6 ];
	u_int16_t ip_id : 4;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE, 4) [ 4 ];
#endif
} esp_pt_1_seq_id_t;

/*
 * @brief Define the ESP pt_2_seq_id compressed format
 *
 * See RFC5225 page 89
 */

typedef struct __attribute__((packed)) esp_pt_2_seq_id
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t ip_id1 : 5;                   // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE,6)  [ 6 ];
	u_int8_t discriminator : 3;            // =:= '110'                                         [ 3 ];

	u_int8_t header_crc : 7;               // crc7((THIS.UVALUE,THIS.ULENGTH)                   [ 7 ];
	u_int8_t ip_id2 : 1;                   // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE,6)  [ 6 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int16_t discriminator : 3;         // =:= '110'                                         [ 3 ];
	u_int16_t ip_id : 6;                 // =:= ip_id_lsb(ip_id_behavior_innermost.UVALUE,6)  [ 6 ];
	u_int16_t header_crc : 7;            // crc7((THIS.UVALUE,THIS.ULENGTH)                   [ 7 ];
#endif
	u_int8_t sequence_number;            // =:= msn_lsb(8)                                    [ 8 ];
} esp_pt_2_seq_id_t;

/*
 * @brief Define the common co_common compressed format
 *
 * See ip_co_common, esp_co_common, udp_co_common, udp_lite_co_common, ...
 */

typedef struct __attribute__((packed)) xxx_co_common
{
	u_int8_t discriminator;           // =:= '11111010'                               [ 8 ];

#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 7;          // =:= crc7(THIS.VALUE,THIS.ULENGTH)            [ 7 ];
	u_int8_t indicator : 1;           // =:= irregular(1)                             [ 1 ];

	u_int8_t control_crc3 : 3;        // =:= control_crc3_encoding                    [ 3 ];
	u_int8_t indicators : 5;          // =:= irregular(5)                             [ 5 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t indicator : 1;           // =:= irregular(1)                             [ 1 ];
	u_int8_t header_crc : 7;          // =:= crc7(THIS.VALUE,THIS.ULENGTH)            [ 7 ];

	u_int8_t indicators : 5;          // =:= irregular(5)                             [ 5 ];
	u_int8_t control_crc3 : 3;        // =:= control_crc3_encoding                    [ 3 ];
#endif

} xxx_co_common_t;

/**
 * @brief Define union of different header pointers
 */

typedef union
{
	unsigned int *uint;
	u_int8_t *uint8;
	base_header_ipvx_t *ipvx;
	base_header_ipv6_t *ipv6;
	base_header_ipv4_t *ipv4;
	ipv6_opt_t *ipv6_opt;
	ip_dest_opt_t *ip_dest_opt;
	ip_hop_opt_t *ip_hop_opt;
	ip_rout_opt_t *ip_rout_opt;
	ip_gre_opt_t *ip_gre_opt;
	ip_mime_opt_t *ip_mime_opt;
	ip_ah_opt_t *ip_ah_opt;
	base_header_udp_t *udp;
	base_header_rtp_t *rtp;
	base_header_rtp_with_csrc_t *rtp_with_csrc;
	base_header_udp_lite_t *udp_lite;
	base_header_esp_t *esp;
} base_header_ip_t;

/*
 * @brief Define the UO-0 compressed format
 *
 * See RFC5225 page 91,
 */

typedef struct __attribute__((packed)) pt_0_crc3
{
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t discriminator : 1;         // =:= '0'                            [ 1 ];
	u_int8_t msn : 4;                   // =:= msn_lsb(4)                     [ 4 ];
	u_int8_t header_crc : 3;            // =:= crc3(THIS.UVALUE, THIS.ULENGTH)[ 3 ];
#endif
	// ip_id =:= inferred_sequential_ip_id                                                [ 0 ];
} pt_0_crc3_t;



// TEST
#define UNALIGNED

// TEST
#if defined(__arm__) || defined(__ARM_EABI__)

typedef struct __attribute__((packed))
{
	u_int16_t UNALIGNED uint16;
} unaligned_uint16_t;

typedef struct __attribute__((packed))
{
	u_int32_t UNALIGNED uint32;
} unaligned_uint32_t;

#endif

/**
 * @brief Define union of for 16 bits data
 *
 */

typedef union __attribute__((packed))
{
	u_int8_t uint8[2];
	u_int16_t uint16 UNALIGNED;
} WB_t;


/**
 * @brief Define union of for 32 bits data
 *
 */

typedef union __attribute__((packed))
{
	u_int8_t uint8[4];
	u_int16_t UNALIGNED uint16[2];
	u_int32_t UNALIGNED uint32;
} LWB_t;

/**
 * @brief Define union of different compressed datas pointers
 *
 */

typedef union
{
	unsigned int *uint;
	u_int8_t *uint8;
	u_int16_t UNALIGNED *uint16;
	u_int32_t UNALIGNED *uint32;

#if defined(__arm__) || defined(__ARM_EABI__)
	unaligned_uint16_t UNALIGNED *ua16;
	unaligned_uint32_t UNALIGNED *ua32;
#endif

	ip_opt_static_t *ip_opt_static;
	ip_opt_dynamic_t *ip_opt_dynamic;

	ip_dest_opt_static_t *ip_dest_opt_static;
	ip_dest_opt_dynamic_t *ip_dest_opt_dynamic;
	ip_hop_opt_static_t *ip_hop_opt_static;
	ip_hop_opt_dynamic_t *ip_hop_opt_dynamic;
	ip_rout_opt_static_t *ip_rout_opt_static;
	ip_gre_opt_static_t *ip_gre_opt_static;
	ip_mime_opt_static_t *ip_mime_opt_static;
	ip_ah_opt_static_t *ip_ah_opt_static;
	ip_ah_opt_dynamic_t *ip_ah_opt_dynamic;

	ipv6_static1_t *ipv6_static1;
	ipv6_static2_t *ipv6_static2;
	ipv6_endpoint_dynamic_t *ipv6_endpoint_dynamic;
	ipv6_regular_dynamic_t *ipv6_regular_dynamic;
	ipv6_outer_irregular_t *ipv6_outer_irregular;

	ipv4_static_t *ipv4_static;
	ipv4_endpoint_innermost_dynamic1_t *ipv4_endpoint_innermost_dynamic1;
	ipv4_endpoint_innermost_dynamic2_t *ipv4_endpoint_innermost_dynamic2;
	ipv4_regular_innermost_dynamic1_t *ipv4_regular_innermost_dynamic1;
	ipv4_regular_innermost_dynamic2_t *ipv4_regular_innermost_dynamic2;
	ipv4_outer_dynamic1_t *ipv4_outer_dynamic1;
	ipv4_outer_dynamic2_t *ipv4_outer_dynamic2;
	ipv4_outer_irregular_t *ipv4_outer_irregular;
	ipv4_innermost_irregular_t *ipv4_innermost_irregular;

	xxx_co_common_t *xxx_co_common;
	pt_0_crc3_t *pt_0_crc3;

	ip_co_common_t *ip_co_common;
	ip_pt_0_crc3_t *ip_pt_0_crc3;
	ip_pt_0_crc7_t *ip_pt_0_crc7;
	ip_pt_1_seq_id_t *ip_pt_1_seq_id;
	ip_pt_2_seq_id_t *ip_pt_2_seq_id;

	profile_1_7_flags1_t *profile_1_7_flags1;
	profile_1_flags2_t *profile_1_flags2;
	profile_7_flags2_t *profile_7_flags2;
	profile_2_3_4_flags_t *profile_2_3_4_flags;
	profile_8_flags_t *profile_8_flags;

	udp_static_t *udp_static;
	udp_endpoint_dynamic_t *udp_endpoint_dynamic;
	udp_regular_dynamic_t *udp_regular_dynamic;
	udp_with_checksum_irregular_t *udp_with_checksum_irregular;
	udp_co_common_t *udp_co_common;
	udp_pt_0_crc3_t *udp_pt_0_crc3;
	udp_pt_0_crc7_t *udp_pt_0_crc7;
	udp_pt_1_seq_id_t *udp_pt_1_seq_id;
	udp_pt_2_seq_id_t *udp_pt_2_seq_id;

	rtp_static_t *rtp_static;
	rtp_dynamic_t *rtp_dynamic;
	rtp_co_common_t *rtp_co_common;
	rtp_pt_0_crc3_t *rtp_pt_0_crc3;
	rtp_pt_0_crc7_t *rtp_pt_0_crc7;
	rtp_pt_1_rnd_t *rtp_pt_1_rnd;
	rtp_pt_1_seq_id_t *rtp_pt_1_seq_id;
	rtp_pt_1_seq_ts_t *rtp_pt_1_seq_ts;
	rtp_pt_2_rnd_t *rtp_pt_2_rnd;
	rtp_pt_2_seq_id_t *rtp_pt_2_seq_id;
	rtp_pt_2_seq_both_t *rtp_pt_2_seq_both;
	rtp_pt_2_seq_ts_t *rtp_pt_2_seq_ts;

	udp_lite_static_t *udp_lite_static;
	udp_lite_endpoint_dynamic1_t *udp_lite_endpoint_dynamic1;
	udp_lite_endpoint_dynamic2_t *udp_lite_endpoint_dynamic2;
	udp_lite_regular_dynamic1_t *udp_lite_regular_dynamic1;
	udp_lite_regular_dynamic2_t *udp_lite_regular_dynamic2;
	udp_lite_irregular1_t *udp_lite_irregular1;
	udp_lite_irregular2_t *udp_lite_irregular2;
	udp_lite_co_common_t *udp_lite_co_common;
	udp_lite_pt_0_crc3_t *udp_lite_pt_0_crc3;
	udp_lite_pt_0_crc7_t *udp_lite_pt_0_crc7;
	udp_lite_pt_1_seq_id_t *udp_lite_pt_1_seq_id;
	udp_lite_pt_2_seq_id_t *udp_lite_pt_2_seq_id;

	udp_lite_rtp_co_common_t *udp_lite_rtp_co_common;
	udp_lite_rtp_pt_0_crc3_t *udp_lite_rtp_pt_0_crc3;
	udp_lite_rtp_pt_0_crc7_t *udp_lite_rtp_pt_0_crc7;
	udp_lite_rtp_pt_1_rnd_t *udp_lite_rtp_pt_1_rnd;
	udp_lite_rtp_pt_1_seq_id_t *udp_lite_rtp_pt_1_seq_id;
	udp_lite_rtp_pt_1_seq_ts_t *udp_lite_rtp_pt_1_seq_ts;
	udp_lite_rtp_pt_2_rnd_t *udp_lite_rtp_pt_2_rnd;
	udp_lite_rtp_pt_2_seq_id_t *udp_lite_rtp_pt_2_seq_id;
	udp_lite_rtp_pt_2_seq_both_t *udp_lite_rtp_pt_2_seq_both;
	udp_lite_rtp_pt_2_seq_ts_t *udp_lite_rtp_pt_2_seq_ts;

	esp_static_t *esp_static;
	esp_dynamic_t *esp_dynamic;
	esp_co_common_t *esp_co_common;
	esp_pt_0_crc3_t *esp_pt_0_crc3;
	esp_pt_0_crc7_t *esp_pt_0_crc7;
	esp_pt_1_seq_id_t *esp_pt_1_seq_id;
	esp_pt_2_seq_id_t *esp_pt_2_seq_id;

} multi_ptr_t;

#if defined(__arm__) || defined(__ARM_EABI__)

#define READNI16_FROM_MPTR(mptr)    (mptr.ua16)->uint16
#define READ16_FROM_MPTR(mptr)      (mptr.ua16++)->uint16
#define READNI32_FROM_MPTR(mptr)    (mptr.ua32)->uint32
#define READ32_FROM_MPTR(mptr)      (mptr.ua32++)->uint32

#define WRITE16_TO_MPTR(mptr,a)     (mptr.ua16++)->uint16 = a
#define WRITE16NI_TO_MPTR(mptr,a)   (mptr.ua16)->uint16 = a
#define WRITE32_TO_MPTR(mptr,a)     (mptr.ua32++)->uint32 = a
#define WRITE32NI_TO_MPTR(mptr,a)   (mptr.ua32)->uint32 = a

#define READ16_FROM_PMPTR(pmptr)    (((pmptr)->ua16)++)->uint16
#define READ32_FROM_PMPTR(pmptr)    (((pmptr)->ua32)++)->uint32

#define WRITE16_TO_PMPTR(pmptr,a)   { (((pmptr)->ua16)++)->uint16 = a; }
#define WRITE32_TO_PMPTR(pmptr,a)   { (((pmptr)->ua32)++)->uint32 = a; }

#else

#define READNI16_FROM_MPTR(mptr)    *(mptr.uint16)
#define READ16_FROM_MPTR(mptr)      *(mptr.uint16++)
#define READNI32_FROM_MPTR(mptr)    *(mptr.uint32)
#define READ32_FROM_MPTR(mptr)      *(mptr.uint32++)

#define WRITE16_TO_MPTR(mptr,a)     *(mptr.uint16++) = a
#define WRITE16NI_TO_MPTR(mptr,a)   *(mptr.uint16) = a
#define WRITE32_TO_MPTR(mptr,a)     *(mptr.uint32++) = a
#define WRITE32NI_TO_MPTR(mptr,a)   *(mptr.uint32) = a

#define READNI16_FROM_PMPTR(pmptr)  *(pmptr->uint16)
#define READ16_FROM_PMPTR(pmptr)    *(pmptr->uint16)++
#define READNI32_FROM_PMPTR(pmptr)  *(pmptr->uint32)
#define READ32_FROM_PMPTR(pmptr)    *(pmptr->uint32)++

#define WRITE16_TO_PMPTR(pmptr,a)   *(pmptr->uint16)++ = a
#define WRITE32_TO_PMPTR(pmptr,a)   *(pmptr->uint32)++ = a

#endif


/*
 * Control datas with CRC3 encoding
 * See RFC5225 section 6.6.11 page 32
 */

typedef struct __attribute__((packed))
{
	u_int8_t reorder_ratio;
	u_int32_t ts_stride;
	u_int32_t time_stride;
	u_int8_t ip_id_behavior[MAX_IP_HEADERS];
} rtp_ctrl_datas_t;

typedef struct __attribute__((packed))
{
	u_int8_t reorder_ratio;
	u_int16_t msn;
	u_int8_t ip_id_behavior[MAX_IP_HEADERS];
} udp_ctrl_datas_t;

typedef struct __attribute__((packed))
{
	u_int8_t reorder_ratio;
	u_int8_t ip_id_behavior[MAX_IP_HEADERS];
} esp_ctrl_datas_t;

typedef struct
{
	u_int8_t reorder_ratio;
	u_int16_t msn;
	u_int8_t ip_id_behavior[MAX_IP_HEADERS];
} ip_ctrl_datas_t;

typedef struct __attribute__((packed))
{
	u_int8_t reorder_ratio;
	u_int32_t ts_stride;
	u_int32_t time_stride;
	u_int8_t coverage_behavior;
	u_int8_t ip_id_behavior[MAX_IP_HEADERS];
} udplite_rtp_ctrl_datas_t;

typedef struct __attribute__((packed))
{
	u_int8_t reorder_ratio;
	u_int16_t msn;
	u_int8_t coverage_behavior;
	u_int8_t ip_id_behavior[MAX_IP_HEADERS];
} udplite_ctrl_datas_t;

typedef union __attribute__((packed))
{
	rtp_ctrl_datas_t rtp;
	udp_ctrl_datas_t udp;
	esp_ctrl_datas_t esp;
	ip_ctrl_datas_t ip;
	udplite_rtp_ctrl_datas_t udplite_rtp;
	udplite_ctrl_datas_t udplite;
} ctrl_datas_t;

#endif // _ROHC_V2_H

