/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @file rfc5225_encoding.h
 * @brief Library of encoding methods from RFC4997, RFC4996 and RFC5225
 * @author FWX <rohc_team@dialine.fr>
 */

extern unsigned int lsb_masks[];
extern unsigned int lsb_xor_masks[];


// RFC4997 page 27
u_int32_t c_lsb( int num_lsbs_param, unsigned int offset_param, unsigned int context_value,
                 unsigned int original_value );

// RFC4996 page 46
int c2_static_or_irreg8( multi_ptr_t *pmptr, u_int8_t *context_value, u_int8_t value );
#ifdef CODE_UNUSED
int c_static_or_irreg16( multi_ptr_t *pmptr, u_int16_t *context_value, u_int16_t value );
u_int8_t c_zero_or_irreg8( multi_ptr_t *pmptr, u_int8_t value );
u_int16_t c_zero_or_irreg16( multi_ptr_t *pmptr, u_int16_t value );
#endif
// RFC4996 page 46
unsigned int variable_length_32_enc( multi_ptr_t *pmptr, u_int32_t *puint32 );
// RFC4996 page 47
unsigned int c_optional32( multi_ptr_t *pmptr, u_int32_t context_value, u_int32_t value );
// RFC4996 page 47
void c_lsb_7_31( multi_ptr_t *pmptr, u_int32_t value );

#ifdef USE_ROHC_V2_MACROS
// RFC4996 page 49
#define c_field_scaling(scaled_value,residue_field,scaling_factor,unscaled_value) \
   if(scaling_factor == 0) \
	{ \
		residue_field = unscaled_value; \
		scaled_value = 0; \
	} \
   else \
	{ \
		residue_field = unscaled_value % scaling_factor; \
		scaled_value = unscaled_value / scaling_factor; \
		assert( unscaled_value == ( ( scaled_value * scaling_factor ) + residue_field ) ); \
	}
#else
void c_field_scaling( u_int32_t *scaled_value, u_int32_t *residue_field, u_int32_t scaling_factor,
                      u_int32_t unscaled_value );
#endif

// RFC4996 page 71
unsigned int rsf_index_enc( unsigned int rsf_flags );
// RFC4996 page 75
unsigned int c_ip_id_lsb( int behavior, unsigned int k, unsigned int p, WB_t context_ip_id,
                          WB_t ip_id,
                          u_int16_t msn );
// RFC4996 page 76
unsigned int c_optional_ip_id_lsb( multi_ptr_t *pmptr, int behavior, WB_t context_ip_id, WB_t ip_id,
                                   u_int16_t msn );
// RFC5225 page 69
//int c_ip_id_lsb( multi_ptr_t * pmptr , int behavior , WB_t context_ip_id , WB_t ip_id , u_int16_t msn );
// See RFC5225 page 70
int c_ip_id_sequential_variable( multi_ptr_t *pmptr, int behavior, WB_t context_ip_id, WB_t ip_id,
                                 u_int16_t msn );

#ifdef USE_ROHC_V2_MACROS
#define c_ip_id_enc_irreg(mptr,behavior,ip_id) \
	{ \
		if( (behavior) == IP_ID_BEHAVIOR_RANDOM) \
		{ \
			WRITE16_TO_MPTR(mptr,ip_id); \
			rohc_debugf(3, "Add ip_id %4.4Xh\n",ntohs(ip_id)); \
		} \
	}
#else
u_int8_t * c_ip_id_enc_irreg( multi_ptr_t mptr, int behavior, u_int16_t ip_id );
#endif

// RFC4996 page 75
unsigned int dscp_encode( multi_ptr_t *pmptr, u_int8_t context_value, u_int8_t value );
// RFC5225 page 69
#ifdef C_ROHC_V2_H
unsigned int c_msn_lsb( int k, struct sc_v2_context *v2_context );
#endif

void put_rtp_frame_in_window( rtp_context_t *rtp_context, base_header_rtp_t *rtp );

// See RFC5225 page 48/49
int c_sdvl_or_static( multi_ptr_t *mptr, u_int32_t static_value, u_int32_t value );

// See RFC5225 page 49
int c_sdvl_or_default( multi_ptr_t *mptr, u_int32_t default_value, u_int32_t value );

// See RFC5225 page 72
u_int8_t * c_sdvl_sn_lsb( multi_ptr_t mptr, u_int32_t value );
// RFC5225 page 74
u_int8_t * c_variable_scaled_timestamp( multi_ptr_t mptr, rtp_context_t *rtp_context, int tss_flag,
                                        int tsc_flag,
                                        u_int32_t timestamp );
// RFC5225 page 74
u_int8_t * c_variable_unscaled_timestamp( multi_ptr_t mptr, int tss_flag, int tsc_flag,
                                          u_int32_t context_timestamp,
                                          u_int32_t timestamp );

