/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @file rfc5225_decoding.h
 * @brief Library of decoding methods from RFC4997, RFC4996 and RFC5225
 * @author FWX <rohc_team@dialine.fr>
 */

extern unsigned int lsb_masks[];
extern unsigned int lsb_xor_masks[];


// RFC4997 page 27
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int32_t d_lsb( int num_lsbs_param, int offset_param, unsigned int context_value,
                 unsigned int value );
#endif

// RFC4996 page 46
u_int8_t d_static_or_irreg8( multi_ptr_t *pmptr, u_int8_t context_value, int indicator );
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int16_t d_static_or_irreg16( multi_ptr_t *pmptr, u_int16_t context_value, int indicator );
#endif
// RFC4996 page 46
extern unsigned int variable_length_32_size[];
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int32_t variable_length_32_dec( multi_ptr_t *pmptr, int indicator );
#endif
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int32_t d_optional32( multi_ptr_t *pmptr, int flag, u_int32_t context_value );
#endif
// RFC4996 page 47
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int32_t d_lsb_7_31( multi_ptr_t *pmptr );
#endif
#ifdef USE_ROHCV2_MACROS
// RFC4996 page 49
#define d_field_scaling(scaling_factor,scaled_value,residue_field) \
   ( (scaled_value) * (scaling_factor) ) + (residue_field);
#else
u_int32_t d_field_scaling( u_int32_t stride_value, u_int32_t scaled_value, u_int32_t residue_field );
#endif
// RFC4996 page 71
unsigned int rsf_index_dec( unsigned int rsf_index );
// RFC4996 page 75
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int16_t d_ip_id_lsb( int behavior, unsigned int k, unsigned int p, WB_t context_ip_id,
                       u_int16_t value,
                       u_int16_t msn );
#endif
// RFC5225 page 69
u_int16_t d_msn_lsb( struct d_v2_context *v2_context, int k, unsigned int value );
// RFC4996 page 76
#ifdef ALREADY_DEFINED_IN_RFC4996_C
void d_optional_ip_id_lsb( multi_ptr_t *pmptr, int behavior, int indicator, WB_t context_ip_id,
                           u_int16_t *ip_id,
                           u_int16_t msn );
#endif
void d_ip_id_sequential_variable( multi_ptr_t *pmptr, int behavior, int indicator,
                                  WB_t context_ip_id, u_int16_t *ip_id,
                                  u_int16_t msn );
int d_ip_id_sequential_variable_size( int behavior, int indicator );
#ifdef ALREADY_DEFINED_IN_RFC4996_C
u_int8_t dscp_decode( multi_ptr_t *pmptr, u_int8_t context_value, int indicator );
#endif

// REVOIR avec d_optional32() ci-dessus
void d_optional_32( multi_ptr_t *pmptr, u_int32_t *value, int flag );
// See RFC5225 page 48
void d_sdvl_or_static( multi_ptr_t *pmptr, int flag, u_int32_t context_value, u_int32_t *value );
// See RFC5225 page 49
u_int32_t d_sdvl_or_default( multi_ptr_t *pmptr, int flag, u_int32_t default_value );
int d_sdvl_or_default_size( multi_ptr_t *pmptr, int flag );
// See RFC5225 page 72
u_int32_t d_sdvl_sn_lsb( multi_ptr_t *pmptr );
int d_sdvl_size( u_int8_t discriminator );

// See RFC5225 section 6.6.9 page 30 and appendix B.3
u_int32_t d_timer_based_lsb( multi_ptr_t *pmptr, u_int16_t time_stride_param, int k, int p );
// See RFC5225 page 71
u_int32_t d_scaled_ts_lsb( multi_ptr_t *pmptr, u_int16_t time_stride_value, int k );
// RFC5225 page 73
u_int32_t d_sdvl_lsb( multi_ptr_t *pmptr, int field_width, u_int32_t context_value );
// RFC5225 page 73
u_int32_t d_sdvl_scaled_ts_lsb( multi_ptr_t *pmptr, u_int32_t time_stride, u_int32_t value );
// RFC5225 page 74
void d_variable_scaled_timestamp( multi_ptr_t *pmptr, rtp_context_t *rtp_context, int tss_flag,
                                  int tsc_flag,
                                  u_int32_t *scaled_timestamp );
int d_variable_scaled_timestamp_size( multi_ptr_t mptr, rtp_context_t *rtp_context, int tss_flag,
                                      int tsc_flag );
// RFC5225 page 74
void d_variable_unscaled_timestamp( multi_ptr_t *pmptr, int tss_flag, int tsc_flag,
                                    u_int32_t context_timestamp,
                                    u_int32_t *timestamp );
int d_variable_unscaled_timestamp_size( multi_ptr_t mptr, int tss_flag, int tsc_flag );

