/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @file d_rohc_v2.c
 * @author FWX <rohc_team@dialine.fr>
 */

#include "config.h"

#if SUPPORT_ROHC_V2

#include "d_rohc_v2.h"
#include "rohc_bit_ops.h"
#include "rohc_traces.h"
#include "rohc_debug.h"
#include "crc.h"
#include "trace.h"
#include "rfc4996_decoding.h"
#include "rfc5225_decoding.h"
#include "wlsb.h"
#include "rohc_time.h"
#include "protocols/ipproto.h"

#include <netinet/ip.h>


void synchronize(struct d_generic_context *context);
void update_inter_packet(struct d_generic_context *context);
int d_decode_dynamic_ip(const unsigned char *packet,
                        unsigned int length,
                        struct d_generic_changes *info,
                        struct list_decomp *decomp);

/*
 * Private function prototypes.
 */

int tcp_decode_static_ipv6_option(struct d_v2_context *v2_context,
                                  ip_context_ptr_t ip_context,
                                  u_int8_t protocol,
                                  multi_ptr_t c_base_header,
                                  unsigned int length,
                                  base_header_ip_t base_header);
unsigned int tcp_copy_static_ipv6_option(u_int8_t protocol,
                                         ip_context_ptr_t ip_context,
                                         base_header_ip_t base_header);
int tcp_decode_dynamic_ipv6_option(struct d_v2_context *v2_context,
                                   ip_context_ptr_t ip_context,
                                   u_int8_t protocol,
                                   multi_ptr_t c_base_header,
                                   unsigned int length,
                                   base_header_ip_t base_header);
u_int8_t * tcp_decode_irregular_ipv6_option(struct d_v2_context *v2_context,
                                            ip_context_ptr_t ip_context,
                                            u_int8_t protocol,
                                            multi_ptr_t mptr,
                                            base_header_ip_t base_header);
int tcp_decode_static_ip(struct d_v2_context *v2_context,
                         ip_context_ptr_t ip_context,
                         multi_ptr_t c_base_header,
                         unsigned int length,
                         base_header_ip_t base_header);
unsigned int tcp_copy_static_ip(ip_context_ptr_t ip_context,
                                base_header_ip_t base_header);
int tcp_decode_dynamic_ip(struct d_v2_context *v2_context,
                          ip_context_ptr_t ip_context,
                          multi_ptr_t c_base_header,
                          unsigned int length,
                          base_header_ip_t base_header,
                          int is_innermost,
                          int profile_value);
u_int8_t * tcp_decode_irregular_ip(struct d_v2_context *v2_context,
                                   ip_context_ptr_t ip_context,
                                   base_header_ip_t base_header,
                                   multi_ptr_t mptr,
                                   int is_innermost,
                                   int outer_ip_flag);
int tcp_decode_ip_baseheader(struct rohc_decomp *decomp,
                             struct d_v2_context *v2_context,
                             ip_context_ptr_t ip_inner_context,
                             base_header_ip_t base_header_inner,
                             multi_ptr_t c_base_header,
                             int length,
                             int *outer_ip_flag);
int tcp_decode_static_udp(udp_context_t *udp_context,
                          udp_static_t *udp_static,
                          int length,
                          base_header_udp_t *udp);
unsigned int tcp_copy_static_udp(udp_context_t *udp_context,
                                 base_header_udp_t *udp);
int tcp_decode_dynamic_udp(struct d_v2_context *v2_context,
                           udp_context_t *udp_context,
                           int profile_value,
                           multi_ptr_t c_base_header,
                           int length,
                           base_header_udp_t *udp);
u_int8_t * tcp_decode_irregular_udp(udp_context_t *udp_context,
                                    multi_ptr_t c_base_header,
                                    int length,
                                    base_header_udp_t *udp);
int tcp_decode_udp_baseheader(struct rohc_decomp *decomp,
                              struct d_v2_context *v2_context,
                              ip_context_ptr_t ip_inner_context,
                              udp_context_t *udp_context,
                              base_header_ip_t base_header_inner,
                              int profile_value,
                              multi_ptr_t c_base_header,
                              int length,
                              base_header_udp_t *udp,
                              int *outer_ip_flag);
int tcp_decode_static_esp(esp_context_t *esp_context,
                          esp_static_t *esp_static,
                          int length,
                          base_header_esp_t *esp);
unsigned int tcp_copy_static_esp(esp_context_t *esp_context,
                                 base_header_esp_t *esp);
int tcp_decode_dynamic_esp(struct d_v2_context *v2_context,
                           esp_context_t *esp_context,
                           esp_dynamic_t *esp_dynamic,
                           int length,
                           base_header_esp_t *esp);
int tcp_decode_esp_baseheader(struct rohc_decomp *decomp,
                              struct d_v2_context *v2_context,
                              ip_context_ptr_t ip_inner_context,
                              esp_context_t *esp_context,
                              base_header_ip_t base_header_inner,
                              int profile_value,
                              multi_ptr_t c_base_header,
                              int length,
                              base_header_esp_t *esp,
                              int *outer_ip_flag);
int tcp_decode_static_udp_lite(udp_lite_context_t *udp_lite_context,
                               udp_lite_static_t *udp_lite_static,
                               int length,
                               base_header_udp_lite_t *udp_lite);
unsigned int tcp_copy_static_udp_lite(udp_lite_context_t *udp_lite_context,
                                      base_header_udp_lite_t *udp_lite);
int tcp_decode_dynamic_udp_lite(struct d_v2_context *v2_context,
                                udp_lite_context_t *udp_lite_context,
                                int profile_value,
                                multi_ptr_t c_base_header,
                                int length,
                                base_header_udp_lite_t *udp_lite);
u_int8_t * tcp_decode_irregular_udp_lite(udp_lite_context_t *udp_lite_context,
                                         multi_ptr_t c_base_header,
                                         int length,
                                         base_header_udp_lite_t *udp_lite);
int tcp_decode_udp_lite_baseheader(struct rohc_decomp *decomp,
                                   struct d_v2_context *v2_context,
                                   ip_context_ptr_t ip_inner_context,
                                   udp_lite_context_t *udp_lite_context,
                                   base_header_ip_t base_header_inner,
                                   int profile_value,
                                   multi_ptr_t c_base_header,
                                   int length,
                                   base_header_udp_lite_t *udp_lite,
                                   int *outer_ip_flag);
int tcp_decode_static_rtp(rtp_context_t *rtp_context,
                          rtp_static_t *rtp_static,
                          int length,
                          base_header_rtp_t *rtp);
unsigned int tcp_copy_static_rtp(rtp_context_t *rtp_context,
                                 base_header_rtp_t *rtp);
u_int8_t * d_csrc_list_dynchain(rtp_context_t *rtp_context,
                                multi_ptr_t mptr,
                                base_header_rtp_with_csrc_t *rtp);
int d_csrc_list_dynchain_size(multi_ptr_t mptr);
int tcp_decode_dynamic_rtp(struct d_v2_context *v2_context,
                           rtp_context_t *rtp_context,
                           multi_ptr_t c_base_header,
                           int length,
                           base_header_rtp_with_csrc_t *rtp);
int tcp_decode_rtp_baseheader(struct rohc_decomp *decomp,
                              struct d_v2_context *v2_context,
                              ip_context_ptr_t ip_inner_context,
                              rtp_context_t *rtp_context,
                              base_header_ip_t base_header_inner,
                              int profile_value,
                              multi_ptr_t c_base_header,
                              int length,
                              base_header_rtp_with_csrc_t *rtp,
                              int *outer_ip_flag);


/**
 * @brief This is a version of ip_compute_csum() optimized for IP headers,
 *        which always checksum on 4 octet boundaries.
 *
 * @param iph The IPv4 header
 * @param ihl The length of the IPv4 header
 * @return    The IPv4 checksum
 */
static uint16_t my_ip_fast_csum(const void *iph, unsigned int ihl)
{
	u_int32_t checksum = 0;
	multi_ptr_t mptr;
	WB_t wb;
	int i;

	rohc_debugf(3, "iph %p ihl %d \n",iph,ihl * 4);

	mptr.uint8 = (u_int8_t *) iph;

	i = ihl << 1;
	while(i-- != 0)
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		wb.uint8[1] = *(mptr.uint8++);
		wb.uint8[0] = *(mptr.uint8++);
		#elif __BYTE_ORDER == __BIG_ENDIAN
		wb.uint16 = READ16_FROM_MPTR(mptr);
		#endif
		checksum += wb.uint16;
		// rohc_debugf(3, "checksum %Xh value %4.4X\n",checksum,wb.uint16);
	}

	while( ( checksum & 0xFFFF0000 ) != 0)
	{
		checksum = ( checksum & 0xFFFF ) + ( checksum >> 16 );
	}

	wb.uint16 = ~checksum;
	rohc_debugf(3, "checksum %Xh\n",wb.uint16);
	return htons( wb.uint16 );
}


/**
 * @brief Create the RoHCv2 decompression context.
 *
 * This function is one of the functions that must exist in one profile for the
 * framework to work.
 *
 * @return The newly-created RoHCv2 decompression context
 */
void * d_v2_create(void)
{
	struct d_generic_context *context;
	struct d_v2_context *v2_context;

	/* create the generic context */
	context = d_generic_create();
	if(context == NULL)
	{
		goto quit;
	}

	/* create the RoHCv2-specific part of the context */
	v2_context = malloc(sizeof(struct d_v2_context));
	if(v2_context == NULL)
	{
		rohc_debugf(0, "cannot allocate memory for the RoHCv2-specific context\n");
		goto destroy_context;
	}
	bzero(v2_context, sizeof(struct d_v2_context));
	context->specific = v2_context;

	/* some RoHCv2-specific values and functions */
// A REVOIR	context->next_header_len = sizeof(tcphdr_t);
	context->build_next_header = NULL;
//	context->decode_static_next_header = tcp_decode_static_tcp;
//	context->decode_dynamic_next_header = tcp_decode_dynamic_tcp;  // A REVOIR
//	context->decode_uo_tail = NULL;
// A REVOIR	context->compute_crc_static = tcp_compute_crc_static;
// A REVOIR	context->compute_crc_dynamic = tcp_compute_crc_dynamic;


	// DBX ???

#ifdef LMJQMSLJ
	/* create the RoHCv2-specific part of the header changes */
	context->outer_ip_changes->next_header_len = sizeof(tcphdr_t);
	context->outer_ip_changes->next_header = malloc(sizeof(struct d_generic_changes));
	if(context->outer_ip_changes->next_header == NULL)
	{
		rohc_debugf(0, "cannot allocate memory for the RoHCv2-specific "
		            "part of the header changes outer_ip_changes\n");
		goto free_v2_context;
	}
	bzero(context->outer_ip_changes->next_header, sizeof(struct d_generic_changes));

	context->inner_ip_changes->next_header_len = sizeof(tcphdr_t);
	context->inner_ip_changes->next_header = malloc(sizeof(struct d_generic_changes));
	if(context->inner_ip_changes->next_header == NULL)
	{
		rohc_debugf(0, "cannot allocate memory for the RoHCv2-specific "
		            "part of the header changes inner_ip_changes\n");
		goto free_outer_ip_changes_next_header;
	}
	bzero(context->inner_ip_changes->next_header, sizeof(struct d_generic_changes));

	/* set next header to RoHCv2 */
	context->next_header_proto = IPPROTO_TCP;
#endif
	rohc_debugf(3, "RoHCv2 context created (%p)\n",context);

	return context;

#ifdef LMJQMSLJ
free_inner_ip_changes_next_header:
	zfree(context->inner_ip_changes->next_header);
free_outer_ip_changes_next_header:
	zfree(context->outer_ip_changes->next_header);
free_v2_context:
	zfree(v2_context);
#endif
destroy_context:
	d_generic_destroy(context);
quit:
	return NULL;
}


/**
 * @brief Destroy the context.
 *
 * This function is one of the functions that must exist in one profile for the
 * framework to work.
 *
 * @param context The compression context
 */
void d_v2_destroy(void *context)
{
	struct d_generic_context *c = context;

	if(c != NULL)
	{
		/* clean RoHCv2-specific memory */
		if(c->outer_ip_changes != NULL && c->outer_ip_changes->next_header != NULL)
		{
			rohc_debugf(3, "zfree outer_ip_changes (%p)\n",c->outer_ip_changes->next_header);
			zfree(c->outer_ip_changes->next_header);
			fflush(stdout);
		}
		if(c->inner_ip_changes != NULL && c->inner_ip_changes->next_header != NULL)
		{
			rohc_debugf(3, "zfree inner_ip_changes (%p)\n",c->inner_ip_changes->next_header);
			zfree(c->inner_ip_changes->next_header);
			fflush(stdout);
		}

		rohc_debugf(3, "RoHCv2 context destroyed (%p)\n",context);

		/* destroy the generic decompression context (c->specific is
		 * destroyed by d_generic_destroy) */
		d_generic_destroy(c);
	}
}


/**
 * @brief Get the size of the static part of an IR packet
 * @return the size
 */
int tcp_get_static_size(void)
{
	rohc_debugf(3, "?????\n");
	fflush(stdout);
	/* Size of Source and Destination ports */
// A REVOIR	return sizeof(tcp_static_t);i
	return 20;
}


/**
 * @brief Decode one IR-CO packet, but not IR packet.
 *
 * This function is one of the functions that must exist in one profile for the
 * framework to work.
 *
 * @param decomp         The ROHC decompressor
 * @param context        The decompression context
 * @param rohc_packet    The ROHC packet to decode
 * @param rohc_length    The length of the ROHC packet
 * @param add_cid_len    The length of the optional Add-CID field
 * @param large_cid_len  The length of the optional large CID field
 * @param dest           OUT: The decoded IP packet
 * @return               The length of the uncompressed IP packet
 *                       or ROHC_ERROR if an error occurs
 *                       or ROHC_ERROR_CRC if a CRC error occurs
 */
int d_v2_decode(struct rohc_decomp *decomp,
                struct d_context *context,
                const unsigned char *const rohc_packet,
                const unsigned int rohc_length,
                const size_t add_cid_len,
                const size_t large_cid_len,
                unsigned char *dest)
{
	struct d_generic_context *g_context = context->specific;
	struct d_v2_context *v2_context = g_context->specific;
//	const struct d_profile * profile = context->profile;
	const d_v2_profile_t *profile_v2 = (const d_v2_profile_t *) context->profile;
	ip_context_ptr_t ip_inner_context;
	ip_context_ptr_t ip_context;
	multi_ptr_t c_base_header;
	base_header_ip_t base_header_inner;
	base_header_ip_t base_header;
//	unsigned int payload_size;
	int length = ROHC_ERROR;
	u_int8_t packet_type;
	u_int8_t protocol;
	int outer_ip_flag;
	int size;
//	int read;

// @param second_byte The offset for the second byte of the ROHC packet
//                    (depends on the CID encoding and the packet type,
//                    may not exist in packet)
	int second_byte = add_cid_len; // A VERIFIER

//	u_int8_t header_crc;

	u_int8_t ip_id_behaviors[MAX_IP_HEADERS];
	u_int8_t ip_id_behaviors_count = 0;

	u_int8_t ctrl_data_value[11 + MAX_IP_HEADERS];
	multi_ptr_t ctrl_data_ptr;

	/* lengths of ROHC and uncompressed headers to be computed during parsing */
	unsigned int rohc_header_len;
	unsigned int uncomp_header_len = 0;

	/* remaining ROHC data not parsed yet */
	unsigned char *rohc_remain_data = (unsigned char *) rohc_packet;

	/* ROHC and uncompressed payloads (they are the same) */
	unsigned int payload_len;

	rohc_debugf(3, "decomp %p context %p rohc_packet %p rohc_length %d second_byte %d dest %p\n",
	            decomp,context,rohc_packet,rohc_length,second_byte,dest);
	rohc_debugf(3, "profile id %4.4Xh\n",profile_v2->profile.id);

//	synchronize(g_context); // DBX
//	g_context->current_packet_time = get_microseconds();

	TraceData((unsigned char*)rohc_packet,rohc_length);

	packet_type = *rohc_packet;
	rohc_debugf(3, "Packet id %2.2Xh\n",packet_type);

	rohc_debugf(3, "old msn %Xh\n",v2_context->msn);

	ip_context.uint8 = v2_context->ip_context;

	rohc_debugf(3, "copy octet %2.2Xh to offset %d\n",*rohc_packet,second_byte - 1);
	c_base_header.uint8 = (u_int8_t*) &rohc_packet[second_byte - 1];
	*c_base_header.uint8 = *rohc_packet;
	TraceData((unsigned char*)rohc_packet,rohc_length > 0x40 ? 0x40 : rohc_length);


	// Init control datas for control_crc3_encoding (see 6.6.11 page 32)
	ctrl_data_ptr.uint8 = ctrl_data_value;

	ip_context.uint8 = v2_context->ip_context;

	do
	{

		protocol = ip_context.vx->next_header;

		// Copy ip_id_behavior of this header for control_crc3_encoding (see 6.6.11 page 32)
		ip_id_behaviors[ ip_id_behaviors_count++ ] = ip_context.vx->ip_id_behavior;
		rohc_debugf(3, "Add ip_id_behavior %d to control_data_crc\n",ip_context.vx->ip_id_behavior);

		ip_inner_context.uint8 = ip_context.uint8;

		if(ip_context.vx->version == IPV4)
		{
			++base_header.ipv4;
			++ip_context.v4;
		}
		else
		{
			++base_header.ipv6;
			++ip_context.v6;
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				protocol = ip_context.v6_option->next_header;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}

	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	rohc_debugf(3, "profile id %4.4Xh\n",profile_v2->profile.id);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
			++ip_context.udp;
			++base_header.udp;
			break;
		case ROHCv2_PROFILE_UDP:
			break;
		case ROHCv2_PROFILE_ESP:
			break;
		case ROHCv2_PROFILE_IP:
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
			++ip_context.udp_lite;
			++base_header.udp_lite;
			break;
		case ROHCv2_PROFILE_UDPLITE:
			break;
		default:
			break;
	}



	// Verify the header received with CRC

	/*
	            co_common pt_0_crc3 pt_0_crc7 pt_1_rnd pt_1_seq_id pt_1_seq_ts pt_2_rnd pt_2_seq_id pt_2_seq_both pt_2_seq_ts
	RTP           0xFA       '0'      '1000'   '101'     '1001'      '101'      '110'      '11000'     '11001'       '1101'
	UDP           0xFA       '0'      '100'              '101'                             '110'
	ESP           0xFA       '0'      '100'              '101'                             '110'
	IP-Only       0xFA       '0'      '100'              '101'                             '110'
	UDP-lite RTP  0xFA       '0'      '1000'   '101'     '1001'      '101'      '110'      '11000'     '11001'       '1101'
	UDP-lite      0xFA       '0'      '100'              '101'                             '110'
	*/

	{
		multi_ptr_t mptr;
		u_int8_t discriminator;
		u_int8_t header_crc;
		u_int8_t control_crc3;

		mptr.uint8 = c_base_header.uint8;
		discriminator = *mptr.uint8;

		if(discriminator == 0xFA)
		{
			header_crc = mptr.xxx_co_common->header_crc;
			mptr.xxx_co_common->header_crc = 0;

			rohc_debugf(3, "ip_inner_context %p\n",ip_inner_context.uint8);

			switch(profile_v2->profile.id)
			{
				case ROHCv2_PROFILE_RTP:
				{
					profile_1_7_flags1_t profile_1_7_flags1;
					profile_1_flags2_t profile_1_flags2;
					u_int32_t time_stride;
					u_int32_t ts_stride;
					int i;

					size = sizeof(rtp_co_common_t);
					++mptr.rtp_co_common;
					rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					rohc_debugf(
					   3,
					   "RTP flags1_indicator %d flags2_indicator %d tsc_indicator %d ttl_hopl_indicator %d\n",
					   c_base_header.rtp_co_common->flags1_indicator,
					   c_base_header.rtp_co_common->flags2_indicator,
					   c_base_header.rtp_co_common->tsc_indicator,
					   c_base_header.rtp_co_common->tss_indicator);
					if(c_base_header.rtp_co_common->flags1_indicator != 0)
					{
						profile_1_7_flags1 = *(mptr.profile_1_7_flags1++);
						size += sizeof(profile_1_7_flags1_t);
						*(ctrl_data_ptr.uint8++) = profile_1_7_flags1.name.reorder_ratio;
						rohc_debugf(3, "Add reorder_ratio %d to control_data_crc\n",
						            profile_1_7_flags1.name.reorder_ratio);
						ip_id_behaviors[ ip_id_behaviors_count -
						                 1 ] = profile_1_7_flags1.name.ip_id_behavior;
						rohc_debugf(3, "Add ip_id_behavior %d to control_data_crc\n",
						            profile_1_7_flags1.name.ip_id_behavior);
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					else
					{
						profile_1_7_flags1.flags = 0;
						*(ctrl_data_ptr.uint8++) = v2_context->reorder_ratio;
						rohc_debugf(3, "Add reorder_ratio %d to control_data_crc\n",
						            v2_context->reorder_ratio);
					}
					if(c_base_header.rtp_co_common->flags2_indicator != 0)
					{
						profile_1_flags2 = *(mptr.profile_1_flags2++);
						size += sizeof(profile_1_flags2_t);
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					else
					{
						profile_1_flags2.flags = 0;
					}
					if(profile_1_7_flags1.name.tos_tc_indicator != 0)
					{
						++size;
						++mptr.uint8;
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(profile_1_7_flags1.name.ttl_hopl_indicator != 0)
					{
						++size;
						++mptr.uint8;
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(profile_1_flags2.name.pt_indicator != 0)
					{
						++size;
						++mptr.uint8;
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					// For sequence_number
					i = d_sdvl_size(*mptr.uint8);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					if(ip_inner_context.vx->version == IPV4)
					{
						rohc_debugf(3, "RTP ip_id_behavior %d ip_id_indicator %d\n",
						            profile_1_7_flags1.name.ip_id_behavior,
						            c_base_header.rtp_co_common->ip_id_indicator);
						i = d_ip_id_sequential_variable_size(profile_1_7_flags1.name.ip_id_behavior,
						                                     c_base_header.rtp_co_common->ip_id_indicator);
						size += i;
						mptr.uint8 += i;
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					rohc_debugf(3, "RTP tss_indicator %d tsc_indicator %d\n",
					            c_base_header.rtp_co_common->tss_indicator,
					            c_base_header.rtp_co_common->tsc_indicator);
					i =
					   d_variable_scaled_timestamp_size(mptr,ip_context.rtp,
					                                    c_base_header.rtp_co_common->tss_indicator,
					                                    c_base_header.rtp_co_common->tsc_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					fflush(stdout);
					i =
					   d_variable_unscaled_timestamp_size(mptr,
					                                      c_base_header.rtp_co_common->tss_indicator,
					                                      c_base_header.rtp_co_common->tsc_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					fflush(stdout);
					if(c_base_header.rtp_co_common->tss_indicator != 0)
					{
						i = d_sdvl_size(*mptr.uint8);
						d_sdvl_or_static(&mptr,c_base_header.rtp_co_common->tss_indicator,
						                 ip_context.rtp->ts_stride,
						                 &ts_stride);
						size += i;
						//	mptr.uint8 += i;
						rohc_debugf(3, "RTP ts_stride %d %Xh\n",ts_stride,ts_stride);
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
						fflush(stdout);
					}
					else
					{
						ts_stride = ip_context.rtp->ts_stride;
					}
					WRITE32_TO_MPTR(ctrl_data_ptr,htonl(ts_stride));
					rohc_debugf(3, "Add ts_stride %d to control_data_crc\n",ts_stride);
					if(profile_1_flags2.name.time_stride_indicator != 0)
					{
						i = d_sdvl_size(*mptr.uint8);
						d_sdvl_or_static(&mptr,profile_1_flags2.name.time_stride_indicator,
						                 ip_context.rtp->time_stride,
						                 &time_stride);
						size += i;
						//	mptr.uint8 += i;
						rohc_debugf(3, "RTP time_stride %d %Xh\n",time_stride,time_stride);
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
						fflush(stdout);
					}
					else
					{
						time_stride = ip_context.rtp->time_stride;
					}
					WRITE32_TO_MPTR(ctrl_data_ptr,htonl(time_stride));
					rohc_debugf(3, "Add time_stride %d (%Xh) to control_data_crc\n",time_stride,
					            time_stride);
					if(profile_1_flags2.name.list_indicator != 0)
					{
						// Calculate m
						i = *mptr.uint8 & 0x0F;
						// If 4-bits XI field
						if( ( *mptr.uint8 & 0x10 ) == 0)
						{
							i = ( i + 1 ) >> 1;
						}
						++i;
						// A REVOIR ...
						size += i;
						rohc_debugf(3, "RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
						fflush(stdout);
					}
					rohc_debugf(3, "RTP co_common size %d\n",size);
					fflush(stdout);
				}
				break;
				case ROHCv2_PROFILE_UDP:
				{
					profile_2_3_4_flags_t profile_2_3_4_flags;
					u_int8_t msn;
					int i;

					size = sizeof(udp_co_common_t);
					++mptr.udp_co_common;
					rohc_debugf(3, "UDP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					*(ctrl_data_ptr.uint8++) = c_base_header.udp_co_common->reorder_ratio;
					rohc_debugf(3, "Add reorder_ratio %d to control_data_crc\n",
					            c_base_header.udp_co_common->reorder_ratio);
					if(c_base_header.udp_co_common->flags_indicator != 0)
					{
						profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
						size += sizeof(profile_2_3_4_flags_t);
						rohc_debugf(3, "UDP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					else
					{
						profile_2_3_4_flags.flags = 0;
						profile_2_3_4_flags.name.ip_id_behavior = ip_inner_context.vx->ip_id_behavior;
					}
					if(c_base_header.udp_co_common->tos_tc_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "UDP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(c_base_header.udp_co_common->ttl_hopl_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "UDP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					msn = *(mptr.uint8++);
					++size;
					*(ctrl_data_ptr.uint8++) = v2_context->msn >> 8;
					*(ctrl_data_ptr.uint8++) = msn;
					rohc_debugf(3, "Add msn %Xh to control_data_crc\n",(v2_context->msn & 0xFF00) | msn);
					i = d_ip_id_sequential_variable_size(profile_2_3_4_flags.name.ip_id_behavior,
					                                     c_base_header.udp_co_common->ip_id_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "UDP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
				}
				break;
				case ROHCv2_PROFILE_ESP:
				{
					profile_2_3_4_flags_t profile_2_3_4_flags;
					int i;

					size = sizeof(esp_co_common_t);
					++mptr.esp_co_common;
					rohc_debugf(3, "ESP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					*(ctrl_data_ptr.uint8++) = c_base_header.esp_co_common->reorder_ratio;
					rohc_debugf(3, "Add reorder_ratio %d to control_data_crc\n",
					            c_base_header.esp_co_common->reorder_ratio);
					if(c_base_header.esp_co_common->flags_indicator != 0)
					{
						profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
						size += sizeof(profile_2_3_4_flags_t);
						rohc_debugf(3, "ESP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					else
					{
						profile_2_3_4_flags.flags = 0;
						profile_2_3_4_flags.name.ip_id_behavior = ip_inner_context.vx->ip_id_behavior;
					}
					if(c_base_header.esp_co_common->tos_tc_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "ESP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(c_base_header.esp_co_common->ttl_hopl_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "ESP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					i = d_sdvl_size(*mptr.uint8);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "ESP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
					i = d_ip_id_sequential_variable_size(profile_2_3_4_flags.name.ip_id_behavior,
					                                     c_base_header.esp_co_common->ip_id_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "ESP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),
					            *(mptr.uint8 + 1));
				}
				break;
				case ROHCv2_PROFILE_IP:
				{
					profile_2_3_4_flags_t profile_2_3_4_flags;
					u_int8_t msn;
					int i;

					size = sizeof(ip_co_common_t);
					++mptr.ip_co_common;
					rohc_debugf(3, "IP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),*(mptr.uint8 + 1));
					*(ctrl_data_ptr.uint8++) = c_base_header.ip_co_common->reorder_ratio;
					rohc_debugf(3, "Add reorder_ratio %d to control_data_crc\n",
					            c_base_header.ip_co_common->reorder_ratio);
					if(c_base_header.ip_co_common->flags_indicator != 0)
					{
						profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
						size += sizeof(profile_2_3_4_flags_t);
						rohc_debugf(3, "IP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					else
					{
						profile_2_3_4_flags.flags = 0;
						profile_2_3_4_flags.name.ip_id_behavior = ip_inner_context.vx->ip_id_behavior;
					}
					if(c_base_header.ip_co_common->tos_tc_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "IP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(c_base_header.ip_co_common->ttl_hopl_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "IP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					// msn
					msn = *(mptr.uint8++);
					++size;
					*(ctrl_data_ptr.uint8++) = v2_context->msn >> 8;
					*(ctrl_data_ptr.uint8++) = msn;
					rohc_debugf(3, "Add msn %Xh to control_data_crc\n",(v2_context->msn & 0xFF00) | msn);
					rohc_debugf(3, "IP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),*(mptr.uint8 + 1));
					i = d_ip_id_sequential_variable_size(profile_2_3_4_flags.name.ip_id_behavior,
					                                     c_base_header.ip_co_common->ip_id_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "IP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,mptr.uint8,
					            *(mptr.uint8),*(mptr.uint8 + 1));
				}
				break;
				case ROHCv2_PROFILE_UDPLITE_RTP:
				{
					profile_1_7_flags1_t profile_1_7_flags1;
					profile_7_flags2_t profile_7_flags2;
					u_int32_t time_stride;
					u_int32_t ts_stride;
					int i;

					size = sizeof(udp_lite_rtp_co_common_t);
					++mptr.udp_lite_rtp_co_common;
					rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
					if(c_base_header.udp_lite_rtp_co_common->flags1_indicator != 0)
					{
						profile_1_7_flags1 = *(mptr.profile_1_7_flags1++);
						size += sizeof(profile_1_7_flags1_t);
					}
					if(c_base_header.udp_lite_rtp_co_common->flags2_indicator != 0)
					{
						profile_7_flags2 = *(mptr.profile_7_flags2++);
						size += sizeof(profile_7_flags2_t);
					}
					if(profile_1_7_flags1.name.tos_tc_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(profile_1_7_flags1.name.ttl_hopl_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(profile_7_flags2.name.pt_indicator != 0)
					{
						++size;
						++mptr.uint8;
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					// For sequence_number
					i = d_sdvl_size(*mptr.uint8);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
					if(ip_inner_context.vx->version == IPV4)
					{
						rohc_debugf(3, "UDP-Lite RTP ip_id_behavior %d ip_id_indicator %d\n",
						            profile_1_7_flags1.name.ip_id_behavior,
						            c_base_header.udp_lite_rtp_co_common->ip_id_indicator);
						i = d_ip_id_sequential_variable_size(
						   profile_1_7_flags1.name.ip_id_behavior,
						   c_base_header.udp_lite_rtp_co_common->ip_id_indicator);
						size += i;
						mptr.uint8 += i;
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					rohc_debugf(3, "UDP-Lite RTP tss_indicator %d tsc_indicator %d\n",
					            c_base_header.udp_lite_rtp_co_common->tss_indicator,
					            c_base_header.udp_lite_rtp_co_common->tsc_indicator);
					i = d_variable_scaled_timestamp_size(
					   mptr,ip_context.rtp,c_base_header.udp_lite_rtp_co_common->tss_indicator,
					   c_base_header.udp_lite_rtp_co_common->tsc_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
					fflush(stdout);
					i = d_variable_unscaled_timestamp_size(
					   mptr,c_base_header.udp_lite_rtp_co_common->tss_indicator,
					   c_base_header.udp_lite_rtp_co_common->tsc_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
					fflush(stdout);
					if(c_base_header.udp_lite_rtp_co_common->tss_indicator != 0)
					{
						i = d_sdvl_size(*mptr.uint8);
						d_sdvl_or_static(&mptr,c_base_header.udp_lite_rtp_co_common->tss_indicator,
						                 ip_context.rtp->ts_stride,
						                 &ts_stride);
						size += i;
						//	mptr.uint8 += i;
						rohc_debugf(3, "UDP-Lite RTP ts_stride %d %Xh\n",ts_stride,ts_stride);
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
						fflush(stdout);
					}
					else
					{
						ts_stride = ip_context.rtp->ts_stride;
					}
					WRITE32_TO_MPTR(ctrl_data_ptr,htonl(ts_stride));
					rohc_debugf(3, "Add ts_stride %d to control_data_crc\n",ts_stride);
					if(profile_7_flags2.name.time_stride_indicator != 0)
					{
						i = d_sdvl_size(*mptr.uint8);
						d_sdvl_or_static(&mptr,profile_7_flags2.name.time_stride_indicator,
						                 ip_context.rtp->time_stride,
						                 &time_stride);
						size += i;
						//	mptr.uint8 += i;
						rohc_debugf(3, "UDP-Lite RTP time_stride %d %Xh\n",time_stride,time_stride);
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
						fflush(stdout);
					}
					else
					{
						time_stride = ip_context.rtp->time_stride;
					}
					WRITE32_TO_MPTR(ctrl_data_ptr,htonl(time_stride));
					rohc_debugf(3, "Add time_stride %d (%Xh) to control_data_crc\n",time_stride,
					            time_stride);
					if(profile_7_flags2.name.list_indicator != 0)
					{
						// Calculate m
						i = *mptr.uint8 & 0x0F;
						// If 4-bits XI field
						if( ( *mptr.uint8 & 0x10 ) == 0)
						{
							i = ( i + 1 ) >> 1;
						}
						++i;
						// A REVOIR ...
						size += i;
						rohc_debugf(3, "UDP-Lite RTP co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
						fflush(stdout);
					}
					rohc_debugf(3, "UDP-Lite RTP co_common size %d\n",size);
					fflush(stdout);
				}
				break;
				case ROHCv2_PROFILE_UDPLITE:
				{
					profile_8_flags_t profile_8_flags;
					u_int8_t msn;
					int i;

					size = sizeof(udp_lite_co_common_t);
					++mptr.udp_lite_co_common;
					rohc_debugf(3, "UDP-Lite co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
					*(ctrl_data_ptr.uint8++) = c_base_header.udp_lite_co_common->reorder_ratio;
					rohc_debugf(3, "Add reorder_ratio %d to control_data_crc\n",
					            c_base_header.udp_lite_co_common->reorder_ratio);
					if(c_base_header.udp_lite_co_common->flags_indicator != 0)
					{
						profile_8_flags = *(mptr.profile_8_flags++);
						size += sizeof(profile_8_flags_t);
						rohc_debugf(3, "UDP-Lite co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					else
					{
						profile_8_flags.ip_id_behavior = ip_inner_context.vx->ip_id_behavior;
					}
					if(c_base_header.udp_lite_co_common->tos_tc_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "UDP-Lite co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					if(c_base_header.udp_lite_co_common->ttl_hopl_indicator != 0)
					{
						++mptr.uint8;
						++size;
						rohc_debugf(3, "UDP-Lite co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
						            mptr.uint8,*(mptr.uint8),
						            *(mptr.uint8 + 1));
					}
					// msn
					msn = *(mptr.uint8++);
					++size;
					*(ctrl_data_ptr.uint8++) = v2_context->msn >> 8;
					*(ctrl_data_ptr.uint8++) = msn;
					rohc_debugf(3, "Add msn %Xh to control_data_crc\n",(v2_context->msn & 0xFF00) | msn);
					rohc_debugf(3, "UDP-Lite co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
					i = d_ip_id_sequential_variable_size(
					   profile_8_flags.ip_id_behavior,c_base_header.udp_lite_co_common->ip_id_indicator);
					size += i;
					mptr.uint8 += i;
					rohc_debugf(3, "UDP-Lite co_common size %d mptr %p then %2.2Xh %2.2Xh\n",size,
					            mptr.uint8,*(mptr.uint8),
					            *(mptr.uint8 + 1));
				}
				break;
			}
			c_base_header.xxx_co_common->header_crc =
			   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
			                 decomp->crc_table_7);
			if(c_base_header.xxx_co_common->header_crc != header_crc)
			{
				rohc_debugf(3, "header_crc received %Xh != calculated %Xh size %d\n",header_crc,
				            c_base_header.xxx_co_common->header_crc,
				            size);
//				return ROHC_ERROR_CRC;
			}
			else
			{
				rohc_debugf(3, "header_crc received ok!\n");
			}
			// Add ip_id_behavior of all headers
			memcpy(ctrl_data_ptr.uint8,ip_id_behaviors,ip_id_behaviors_count);
			ctrl_data_ptr.uint8 += ip_id_behaviors_count;
			rohc_debugf(3, "ctrl_datas length %d :\n",(int)(ctrl_data_ptr.uint8 - ctrl_data_value));
			TraceData(ctrl_data_value,ctrl_data_ptr.uint8 - ctrl_data_value);
			control_crc3 =
			   crc_calculate(CRC_TYPE_3,ctrl_data_value,ctrl_data_ptr.uint8 - ctrl_data_value,
			                 CRC_INIT_3,
			                 decomp->crc_table_3);
			if(control_crc3 != c_base_header.xxx_co_common->control_crc3)
			{
				rohc_debugf(3, "control_crc3 received %Xh != calculated %Xh size %d\n",
				            c_base_header.xxx_co_common->control_crc3,control_crc3,
				            (int)(ctrl_data_ptr.uint8 - ctrl_data_value));
//				return ROHC_ERROR_CRC;
			}
			else
			{
				rohc_debugf(3, "control_crc3 received ok!\n");
			}

		}
		else
		{
			rohc_debugf(3, "Discriminator %Xh\n",discriminator);
			if( ( discriminator & 0x80 ) == 0)
			{
				// pt_0_crc3
				size = sizeof(pt_0_crc3_t);
				header_crc = c_base_header.pt_0_crc3->header_crc;
				c_base_header.pt_0_crc3->header_crc = 0;
				c_base_header.pt_0_crc3->header_crc =
				   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
				                 decomp->crc_table_3);
				if(header_crc != c_base_header.pt_0_crc3->header_crc)
				{
					rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
					            c_base_header.pt_0_crc3->header_crc,
					            size);
					return ROHC_ERROR;
				}
			}
			else
			{
				// If profile with RTP
				if(profile_v2->profile.id == ROHCv2_PROFILE_RTP ||
				   profile_v2->profile.id == ROHCv2_PROFILE_UDPLITE_RTP)
				{
					//      '101' pt_1_rnd
					//	'101' pt_1_seq_ts
					//	'110' pt_2_rnd
					//	'1000' pt_0_crc7
					//	'1001' pt_1_seq_id
					//	'1101' pt_2_seq_ts
					//	'11000' pt_2_seq_id
					//	'11001' pt_2_seq_both
					switch(discriminator & 0xE0)
					{
						case 0x80:  // '100'
						case 0xA0:  // '101'
						case 0xC0:  // '110'
						case 0xE0:  // '111'
						case 0xD0:  // '1101'  // A REVOIR
							break;
					}

				}
				else
				{
					switch(discriminator & 0xE0)
					{
						case 0x80:  // '100' pt_0_crc7
							break;
						case 0xA0:  // '101' pt_1_seq_id
							break;
						case 0xC0:  // '110' pt_2_seq_id
							break;
						default:
							rohc_debugf(3, "Discriminator %Xh unknown\n",discriminator & 0xE0);
							goto error;
					}
				}
			}
		}

	}





	/* skip the first bytes:
	 *  IR-DYN type + Profile ID + CRC (+ eventually CID bytes) */
	rohc_remain_data += second_byte;
//	rohc_header_len = second_byte;
//	rohc_header_len = 0;

	rohc_debugf(3, "rohc_packet %p compressed base header %p\n",rohc_packet,c_base_header.uint8);





	/* Init static part */

	base_header.uint8 = dest;
	ip_context.uint8 = v2_context->ip_context;

	do
	{
		base_header_inner.uint8 = base_header.uint8;

		// Copy static part of IP header
		uncomp_header_len += tcp_copy_static_ip(ip_context,base_header);

		protocol = ip_context.vx->next_header;

		if(ip_context.vx->version == IPV4)
		{
			*(ctrl_data_ptr.uint8++) = ip_context.v4->ip_id_behavior;
			++base_header.ipv4;
			++ip_context.v4;
		}
		else
		{
			*(ctrl_data_ptr.uint8++) = ip_context.v6->ip_id_behavior;
			++base_header.ipv6;
			++ip_context.v6;
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				uncomp_header_len += tcp_copy_static_ipv6_option(protocol,ip_context,base_header);
				protocol = ip_context.v6_option->next_header;
				base_header.uint8 += ip_context.v6_option->option_length;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}

	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);


	rohc_debugf(3, "uncomp_header_len %d\n",uncomp_header_len);

	outer_ip_flag = 0;

	rohc_debugf(3, "profile id %4.4Xh\n",profile_v2->profile.id);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
			// Copy static UDP part
			uncomp_header_len += tcp_copy_static_udp(ip_context.udp,base_header.udp);
			++ip_context.udp;
			++base_header.udp;
			// Copy static RTP part
			uncomp_header_len += tcp_copy_static_rtp(ip_context.rtp,base_header.rtp);
			fflush(stdout);
			size =
			   tcp_decode_rtp_baseheader(decomp,v2_context,ip_inner_context,ip_context.rtp,
			                             base_header_inner,
			                             profile_v2->profile.id,c_base_header,rohc_length,
			                             base_header.rtp_with_csrc,
			                             &outer_ip_flag);
			fflush(stdout);
			break;
		case ROHCv2_PROFILE_UDP:
			// Copy static UDP part
			uncomp_header_len += tcp_copy_static_udp(ip_context.udp,base_header.udp);
			size =
			   tcp_decode_udp_baseheader(decomp,v2_context,ip_inner_context,ip_context.udp,
			                             base_header_inner,
			                             profile_v2->profile.id,c_base_header,rohc_length,
			                             base_header.udp,
			                             &outer_ip_flag);
			break;
		case ROHCv2_PROFILE_ESP:
			// Copy static ESP part
			uncomp_header_len += tcp_copy_static_esp(ip_context.esp,base_header.esp);
			size =
			   tcp_decode_esp_baseheader(decomp,v2_context,ip_inner_context,ip_context.esp,
			                             base_header_inner,
			                             profile_v2->profile.id,c_base_header,rohc_length,
			                             base_header.esp,
			                             &outer_ip_flag);
			break;
		case ROHCv2_PROFILE_IP:
			size =
			   tcp_decode_ip_baseheader(decomp,v2_context,ip_inner_context,base_header_inner,
			                            c_base_header,
			                            rohc_length,
			                            &outer_ip_flag);
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
			break;
		case ROHCv2_PROFILE_UDPLITE:
			// Copy static UDP-Lite part
			uncomp_header_len += tcp_copy_static_udp_lite(ip_context.udp_lite,base_header.udp_lite);
			size =
			   tcp_decode_udp_lite_baseheader(decomp,v2_context,ip_inner_context,ip_context.udp_lite,
			                                  base_header_inner,profile_v2->profile.id,c_base_header,
			                                  rohc_length,base_header.udp_lite,
			                                  &outer_ip_flag);
			break;
		default:
			break;
	}

	if(size < 0)
	{
		goto error;
	}

	rohc_debugf(3, "uncomp_header_len %d size %d\n",uncomp_header_len,size);

//	rohc_header_len += size;
	c_base_header.uint8 += size;

	/* Uncompress irregular chain */

	base_header.uint8 = dest;
	ip_context.uint8 = v2_context->ip_context;

	do
	{
		protocol = ip_context.vx->next_header;

		c_base_header.uint8 =
		   tcp_decode_irregular_ip(v2_context,ip_context,base_header,c_base_header,
		                           base_header.ipvx ==
		                           base_header_inner.ipvx,
		                           outer_ip_flag);

		if(ip_context.vx->version == IPV4)
		{
			++base_header.ipv4;
			++ip_context.v4;
		}
		else
		{
			++base_header.ipv6;
			++ip_context.v6;
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				c_base_header.uint8 =
				   tcp_decode_irregular_ipv6_option(v2_context,ip_context,protocol,c_base_header,
				                                    base_header);
				protocol = ip_context.v6_option->next_header;
				base_header.uint8 += ip_context.v6_option->option_length;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}

	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
		case ROHCv2_PROFILE_UDP:
			// length not initialized!
			c_base_header.uint8 = tcp_decode_irregular_udp(ip_context.udp,c_base_header,length,
			                                               base_header.udp);
			++base_header.udp;
			break;
		case ROHCv2_PROFILE_ESP:
			++base_header.esp;
			break;
		case ROHCv2_PROFILE_IP:
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
		case ROHCv2_PROFILE_UDPLITE:
			// length not initialized!
			c_base_header.uint8 =
			   tcp_decode_irregular_udp_lite(ip_context.udp_lite,c_base_header,length,
			                                 base_header.udp_lite);
			++base_header.udp_lite;
			break;
	}

	fflush(stdout);
	rohc_header_len = c_base_header.uint8 - rohc_packet;
	payload_len = rohc_length - rohc_header_len;
	rohc_debugf(3, "rohc_len %d rohc_header_len %d uncomp_header_len %d\n",rohc_length,
	            rohc_header_len,
	            uncomp_header_len);
	size = uncomp_header_len + payload_len;
	rohc_debugf(3, "payload_len %d\n",payload_len);

	// Update uncompress packet

	base_header.uint8 = dest;
	ip_context.uint8 = v2_context->ip_context;

	length = size;

	do
	{
		protocol = ip_context.vx->next_header;

		if(ip_context.vx->version == IPV4)
		{
			base_header.ipv4->hdr_length = sizeof(base_header_ipv4_t) >> 2;
			base_header.ipv4->length = htons(length);
			base_header.ipv4->checksum = 0;
			base_header.ipv4->checksum = my_ip_fast_csum(base_header.uint8,
			                                             base_header.ipv4->hdr_length);
//			base_header.ipv4->checksum = ip_fast_csum(base_header.uint8,base_header.ipv4->hdr_length);
			rohc_debugf(3, "IP checksum = 0x%04x for %d\n", ntohs(
			               base_header.ipv4->checksum), base_header.ipv4->hdr_length);
			TraceIpV4(base_header.ipv4);
			length -= sizeof(base_header_ipv4_t);
			++base_header.ipv4;
			++ip_context.v4;
		}
		else
		{
			length -= sizeof(base_header_ipv6_t);
			base_header.ipv6->payload_length = htons(length);
			TraceIpV6(base_header.ipv6);
			++base_header.ipv6;
			++ip_context.v6;
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				protocol = ip_context.v6_option->next_header;
				length -= ip_context.v6_option->option_length;
				base_header.uint8 += ip_context.v6_option->option_length;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}

	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
		case ROHCv2_PROFILE_UDP:
			base_header.udp->udp_length = htons(length);
			TraceUdp(base_header.udp);
			++base_header.udp;
			if(profile_v2->profile.id == ROHCv2_PROFILE_RTP)
			{
				++base_header.rtp;
			}
			break;
		case ROHCv2_PROFILE_ESP:
			TraceEsp(base_header.esp);
			++base_header.esp;
			break;
		case ROHCv2_PROFILE_IP:
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
		case ROHCv2_PROFILE_UDPLITE:
			if(ip_context.udp_lite->coverage_behavior == UDP_LITE_COVERAGE_INFERRED)
			{
				base_header.udp_lite->checksum_coverage = htons(sizeof(base_header_udp_t) + payload_len);
			}
			TraceUdpLite(base_header.udp_lite);
			++base_header.udp_lite;
			if(profile_v2->profile.id == ROHCv2_PROFILE_UDPLITE_RTP)
			{
				++base_header.rtp;
			}
			break;
	}

	if(payload_len != 0)
	{
		memcpy(base_header.uint8,c_base_header.uint8,payload_len);
	}

	if(base_header_inner.ipvx->version == IPV4)
	{
		rohc_debugf(3, "CONTEXT %p msn %4.4Xh IP-ID %4.4Xh\n",context,v2_context->msn,
		            ntohs(base_header_inner.ipv4->ip_id));
	}

	/* statistics */
	context->header_compressed_size += rohc_header_len;
	c_add_wlsb(context->header_16_compressed, 0, rohc_header_len);
	context->header_uncompressed_size += uncomp_header_len;
	c_add_wlsb(context->header_16_uncompressed, 0, uncomp_header_len);

	rohc_debugf(3, "msn %4.4Xh\n", v2_context->msn );

	rohc_debugf(3, "return %d\n", size );
	return size;

error:
	return ROHC_ERROR;
}


/**
 * @brief Decode one IR packet for the RoHCv2 profile.
 *
 * This function is one of the functions that must exist in one profile for the
 * framework to work.
 *
 * @param decomp          The ROHC decompressor
 * @param context         The decompression context
 * @param rohc_packet     The ROHC packet to decode
 * @param rohc_length     The length of the ROHC packet to decode
 * @param large_cid_len   The length of the large CID field
 * @param is_addcid_used  Whether the add-CID field is present or not
 * @param dest            The decoded IP packet
 * @return                The length of the uncompressed IP packet
 *                        or ROHC_OK_NO_DATA if packet is feedback only
 *                        or ROHC_ERROR if an error occurs
 */
int d_v2_decode_ir(struct rohc_decomp *decomp,
                   struct d_context *context,
                   const unsigned char *rohc_packet,
                   const unsigned int rohc_length,
                   int large_cid_len,
                   int is_addcid_used,
                   unsigned char *dest)
{
	struct d_generic_context *g_context = context->specific;
	struct d_v2_context *v2_context = g_context->specific;
	const d_v2_profile_t *profile_v2 = (const d_v2_profile_t *) context->profile;
	ip_context_ptr_t ip_inner_context;
	ip_context_ptr_t ip_context;
//	multi_ptr_t c_base_header_inner;
//	multi_ptr_t c_base_header2;
	multi_ptr_t c_base_header;
	base_header_ip_t base_header;
	unsigned int payload_size;
	unsigned int length;
	u_int8_t protocol;
	u_int16_t size;
//	int read2;
	int read;

	rohc_debugf(
	   3,
	   "decomp %p context %p rohc_packet %p rohc_length %d large_cid_len %d is_addcid_used %d dest %p\n",
	   decomp, context, rohc_packet, rohc_length, large_cid_len, is_addcid_used, dest);
	rohc_debugf(3, "profile id %4.4Xh\n",profile_v2->profile.id);

	TraceData((unsigned char*)rohc_packet,rohc_length);
	fflush(stdout);

	/* skip:
	 *  - the first byte of the ROHC packet (field 2)
	 *  - the Profile byte (field 4)
	 *  - the CRC byte (field 5) */
	length = 3;

	c_base_header.uint8 = (u_int8_t*)( rohc_packet + large_cid_len + length );


	/* static chain */

	base_header.uint8 = dest;
	ip_context.uint8 = v2_context->ip_context;

	size = 0;
	do
	{

		ip_inner_context.uint8 = ip_context.uint8;
		ip_context.vx->inner = 0;
		read = tcp_decode_static_ip(v2_context,ip_context,c_base_header,rohc_length - length,
		                            base_header);
		length += read;
		c_base_header.uint8 += read;
		protocol = ip_context.vx->next_header;
		ip_context.uint8 += ip_context.vx->context_length;
		if(base_header.ipvx->version == IPV4)
		{
			++base_header.ipv4;
			size += sizeof(base_header_ipv4_t);
		}
		else
		{
			++base_header.ipv6;
			size += sizeof(base_header_ipv6_t);
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				read =
				   tcp_decode_static_ipv6_option(v2_context,ip_context,protocol,c_base_header,length,
				                                 base_header);
				rohc_debugf(3, "read %d ip_context protocol %d context_length %d option_length %d\n",
				            read,protocol,ip_context.v6_option->option_length,
				            ip_context.v6_option->context_length);
				length += read;
				c_base_header.uint8 += read;
//rohc_debugf(3, "context %p protocol %d context_length %d option_length %d\n",ip_context.uint8,protocol,ip_context.v6_option->context_length,ip_context.v6_option->option_length);
//TraceData(ip_context.uint8,ip_context.v6_option->context_length);
				size += ip_context.v6_option->option_length;
				protocol = ip_context.v6_option->next_header;
				base_header.uint8 += ip_context.v6_option->option_length;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}
		rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
		assert( ip_context.uint8 < &v2_context->ip_context[MAX_IP_CONTEXT_SIZE] );
		rohc_debugf(3, "Current IP packet\n");
		TraceData(dest,size);
		fflush(stdout);

	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	ip_inner_context.vx->inner = 1;

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
			read = tcp_decode_static_udp(ip_context.udp,c_base_header.udp_static,rohc_length - length,
			                             base_header.udp);
			++base_header.udp;
			size += sizeof(base_header_udp_t);

			length += read;
			c_base_header.uint8 += read;
			rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
			++ip_context.udp;
			/*
			if( ip_context.rtp->ts_decomp_initialized == 0 )
			{
			      ++ip_context.rtp->ts_decomp_initialized;
			   // Create the ts_sc_decomp object
			   d_create_sc(&ip_context.rtp->ts_decomp);
			}
			*/
			read = tcp_decode_static_rtp(ip_context.rtp,c_base_header.rtp_static,rohc_length - length,
			                             base_header.rtp);
			++base_header.rtp;
			size += sizeof(base_header_rtp_t);
			break;
		case ROHCv2_PROFILE_UDP:
			read = tcp_decode_static_udp(ip_context.udp,c_base_header.udp_static,rohc_length - length,
			                             base_header.udp);
			++base_header.udp;
			size += sizeof(base_header_udp_t);
			break;
		case ROHCv2_PROFILE_ESP:
			read = tcp_decode_static_esp(ip_context.esp,c_base_header.esp_static,rohc_length - length,
			                             base_header.esp);
			++base_header.esp;
			size += sizeof(base_header_esp_t);
			break;
		case ROHCv2_PROFILE_IP:
			read = 0;
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
			read =
			   tcp_decode_static_udp_lite(ip_context.udp_lite,c_base_header.udp_lite_static,
			                              rohc_length -
			                              length,
			                              base_header.udp_lite);
			++base_header.udp_lite;
			size += sizeof(base_header_udp_lite_t);

			length += read;
			c_base_header.uint8 += read;
			rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
			++ip_context.udp_lite;
			/*
			if( ip_context.rtp->ts_decomp_initialized == 0 )
			{
			      ++ip_context.rtp->ts_decomp_initialized;
			   // Create the ts_sc_decomp object
			   d_create_sc(&ip_context.rtp->ts_decomp);
			}
			*/
			read = tcp_decode_static_rtp(ip_context.rtp,c_base_header.rtp_static,rohc_length - length,
			                             base_header.rtp);
			++base_header.rtp;
			size += sizeof(base_header_rtp_t);
			break;
		case ROHCv2_PROFILE_UDPLITE:
			read =
			   tcp_decode_static_udp_lite(ip_context.udp_lite,c_base_header.udp_lite_static,
			                              rohc_length -
			                              length,
			                              base_header.udp_lite);
			++base_header.udp_lite;
			size += sizeof(base_header_udp_lite_t);
			break;
	}

	if(read != 0)
	{
		rohc_debugf(3, "Current IP packet\n");
		TraceData(dest,size);
		fflush(stdout);

		length += read;
		c_base_header.uint8 += read;
		rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
		fflush(stdout);
	}

	/* dynamic chain */

	base_header.uint8 = dest;
	ip_context.uint8 = v2_context->ip_context;

	do
	{
		read =
		   tcp_decode_dynamic_ip(v2_context,ip_context,c_base_header,rohc_length - length,base_header,
		                         ip_context.vx->inner,
		                         context->profile->id);
		length += read;
		c_base_header.uint8 += read;
		rohc_debugf(3, "length %d read %d\n",length,read);
		protocol = ip_context.vx->next_header;
		ip_context.uint8 += ip_context.vx->context_length;
		if(base_header.ipvx->version == IPV4)
		{
			++base_header.ipv4;
		}
		else
		{
			++base_header.ipv6;
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				read =
				   tcp_decode_dynamic_ipv6_option(v2_context,ip_context,protocol,c_base_header,length,
				                                  base_header);
				length += read;
				c_base_header.uint8 += read;
//rohc_debugf(3, "context %p protocol %d context_length %d option_length %d\n",ip_context.uint8,protocol,ip_context.v6_option->context_length,ip_context.v6_option->option_length);
//TraceData(ip_context.uint8,ip_context.v6_option->context_length);
				protocol = ip_context.v6_option->next_header;
				base_header.uint8 += ip_context.v6_option->option_length;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}
		assert( ip_context.uint8 < &v2_context->ip_context[MAX_IP_CONTEXT_SIZE] );
		rohc_debugf(3, "Current IP packet\n");
		TraceData(dest,size);
		fflush(stdout);
	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
	fflush(stdout);

	rohc_debugf(3, "Current IP packet\n");
	TraceData(dest,size);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
			read =
			   tcp_decode_dynamic_udp(v2_context,ip_context.udp,context->profile->id,c_base_header,
			                          rohc_length - length,
			                          base_header.udp);
			++base_header.udp;

			length += read;
			c_base_header.uint8 += read;
			rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
			++ip_context.udp;
			read = tcp_decode_dynamic_rtp(v2_context,ip_context.rtp,c_base_header,rohc_length - length,
			                              base_header.rtp_with_csrc);
			++base_header.rtp;
			break;
		case ROHCv2_PROFILE_UDP:
			read =
			   tcp_decode_dynamic_udp(v2_context,ip_context.udp,context->profile->id,c_base_header,
			                          rohc_length - length,
			                          base_header.udp);
			++base_header.udp;
			break;
		case ROHCv2_PROFILE_ESP:
			read =
			   tcp_decode_dynamic_esp(v2_context,ip_context.esp,c_base_header.esp_dynamic,
			                          rohc_length -
			                          length,
			                          base_header.esp);
			++base_header.esp;
			break;
		case ROHCv2_PROFILE_IP:
			read = 0;
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
			read =
			   tcp_decode_dynamic_udp_lite(v2_context,ip_context.udp_lite,context->profile->id,
			                               c_base_header,
			                               rohc_length - length,
			                               base_header.udp_lite);
			++base_header.udp_lite;

			length += read;
			c_base_header.uint8 += read;
			rohc_debugf(3, "length %d read %d size %d\n",length,read,size);
			++ip_context.udp_lite;
			read = tcp_decode_dynamic_rtp(v2_context,ip_context.rtp,c_base_header,rohc_length - length,
			                              base_header.rtp_with_csrc);
			++base_header.rtp;
			break;
		case ROHCv2_PROFILE_UDPLITE:
			read =
			   tcp_decode_dynamic_udp_lite(v2_context,ip_context.udp_lite,context->profile->id,
			                               c_base_header,
			                               rohc_length - length,
			                               base_header.udp_lite);
			++base_header.udp_lite;
			break;
	}

	length += read;
	c_base_header.uint8 += read;
	rohc_debugf(3, "length %d read %d size %d\n",length,read,size);

//	memcpy(&v2_context->old_tcphdr,tcp,sizeof(tcphdr_t));

	payload_size = rohc_length - length - large_cid_len;
	rohc_debugf(3, "payload_size %d\n",payload_size);

	// Calculate scaled value and residue (see RFC4996 page 32/33)
#ifdef LMQJSQ
	if(payload_size != 0)
	{
		v2_context->seq_number_scaled = ntohl(tcp->seq_number) / payload_size;
		v2_context->seq_number_residue = ntohl(tcp->seq_number) % payload_size;
	}
#endif
	// copy payload
	memcpy(base_header.uint8,c_base_header.uint8,payload_size);
	rohc_debugf(3, "copy %d bytes of payload\n",payload_size);
	fflush(stdout);
	// Add payload size
	size += payload_size;
	rohc_debugf(3, "payload_size %d size %d\n",payload_size,size);

	rohc_debugf(3, "Total length %d\n",size);
	fflush(stdout);
	base_header.uint8 = dest;
	ip_context.uint8 = v2_context->ip_context;

	length = size;

	do
	{
		if(base_header.ipvx->version == IPV4)
		{
			protocol = base_header.ipv4->protocol;
			base_header.ipv4->length = htons(length);
			base_header.ipv4->checksum = 0;
			base_header.ipv4->checksum = my_ip_fast_csum(base_header.uint8,
			                                             base_header.ipv4->hdr_length);
//			base_header.ipv4->checksum = ip_fast_csum(base_header.uint8,base_header.ipv4->hdr_length);
			rohc_debugf(3, "IP checksum = 0x%04x for %d\n", ntohs(
			               base_header.ipv4->checksum), base_header.ipv4->hdr_length);
			TraceIpV4(base_header.ipv4);
			++base_header.ipv4;
			++ip_context.v4;
			length -= sizeof(base_header_ipv4_t);
		}
		else
		{
			protocol = base_header.ipv6->next_header;
			length -= sizeof(base_header_ipv6_t);
			base_header.ipv6->payload_length = htons(length);
			rohc_debugf(3, "payload_length %d\n",ntohs(base_header.ipv6->payload_length));
			TraceIpV6(base_header.ipv6);
			++base_header.ipv6;
			++ip_context.v6;
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
//rohc_debugf(3, "context %p protocol %d context_length %d option_length %d\n",ip_context.uint8,protocol,ip_context.v6_option->context_length,ip_context.v6_option->option_length);
				length -= ip_context.v6_option->option_length;
				protocol = base_header.ipv6_opt->next_header;
				base_header.uint8 += ip_context.v6_option->option_length;
				ip_context.uint8 += ip_context.v6_option->context_length;
			}
		}
		rohc_debugf(3, "protocol %d\n",protocol);
		fflush(stdout);
	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
			base_header.udp->udp_length = htons(length);
			TraceUdp(base_header.udp);
			++base_header.udp;
			TraceRtp(base_header.rtp_with_csrc);
			break;
		case ROHCv2_PROFILE_UDP:
			base_header.udp->udp_length = htons(length);
			TraceUdp(base_header.udp);
			break;
		case ROHCv2_PROFILE_ESP:
			TraceEsp(base_header.esp);
			break;
		case ROHCv2_PROFILE_IP:
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
			rohc_debugf(3, "length %d size %d coverage_behavior %d\n",length,size,
			            ip_context.udp_lite->coverage_behavior);
			if(ip_context.udp_lite->coverage_behavior == UDP_LITE_COVERAGE_INFERRED)
			{
				base_header.udp_lite->checksum_coverage = htons(length);
			}
			TraceUdpLite(base_header.udp_lite);
			++base_header.udp_lite;
			TraceRtp(base_header.rtp_with_csrc);
			break;
		case ROHCv2_PROFILE_UDPLITE:
			rohc_debugf(3, "length %d size %d coverage_behavior %d\n",length,size,
			            ip_context.udp_lite->coverage_behavior);
			if(ip_context.udp_lite->coverage_behavior == UDP_LITE_COVERAGE_INFERRED)
			{
				base_header.udp_lite->checksum_coverage = htons(length);
			}
			TraceUdpLite(base_header.udp_lite);
			break;
	}

	rohc_debugf(3, "msn %4.4Xh\n", v2_context->msn );

	rohc_debugf(3, "return %d\n", size );
	fflush(stdout);
	return size;

//error:
//	return ROHC_ERROR;
}


/**
 * @brief Find the length of the IR header.
 *
 * This function is one of the functions that must exist in one profile for the
 * framework to work.
 *
 * \verbatim

 Basic structure of the IR packet (5.7.7.1):

      0   1   2   3   4   5   6   7
     --- --- --- --- --- --- --- ---
 1  |         Add-CID octet         |  if for small CIDs and CID != 0
    +---+---+---+---+---+---+---+---+
 2  | 1   1   1   1   1   1   0 | D |
    +---+---+---+---+---+---+---+---+
    |                               |
 3  /    0-2 octets of CID info     /  1-2 octets if for large CIDs
    |                               |
    +---+---+---+---+---+---+---+---+
 4  |            Profile            |  1 octet
    +---+---+---+---+---+---+---+---+
 5  |              CRC              |  1 octet
    +---+---+---+---+---+---+---+---+
    |                               |
 6  |         Static chain          |  variable length
    |                               |
    +---+---+---+---+---+---+---+---+
    |                               |
 7  |         Dynamic chain         |  present if D = 1, variable length
    |                               |
    +---+---+---+---+---+---+---+---+
    |                               |
 8  |           Payload             |  variable length
    |                               |
     - - - - - - - - - - - - - - - -

\endverbatim
 *
 * The function computes the length of the fields 2 + 4-8, ie. the first byte,
 * the Profile and CRC fields, the static and dynamic chains (outer and inner
 * IP headers + RoHCv2 header).
 *
 * @param context         The decompression context
 * @param packet          The pointer on the IR packet minus the Add-CID byte
 *                        (ie. the field 2 in the figure)
 * @param plen            The length of the IR packet minus the Add-CID byte
 * @param large_cid_len   The size of the large CID field
 *                        (ie. field 3 in the figure)
 * @return                The length of the IR header,
 *                        0 if an error occurs
 */
unsigned int tcp_detect_ir_size(struct d_context *context,
                                unsigned char *packet,
                                unsigned int plen,
                                unsigned int large_cid_len)
{
	const d_v2_profile_t *profile_v2 = (const d_v2_profile_t *) context->profile;
	multi_ptr_t bc_base_header;
	multi_ptr_t c_base_header;
	unsigned int length;
	u_int8_t protocol;
	int nb_ip_header;
	int bsize;
	int size;
	int i;

	rohc_debugf(3, "context %p packet %p plen %d large_cid_len %d\n",context,packet,plen,
	            large_cid_len);
	TraceData(packet,plen);
	fflush(stdout);
	/* Profile and CRC fields + IP static & dynamic chains */
	/* skip:
	 *  - the first byte of the ROHC packet (field 2)
	 *  - the Profile byte (field 4)
	 *  - the CRC byte (field 5) */
	length = 3;

	/* static chain */

	c_base_header.uint8 = packet + large_cid_len + length;
	bc_base_header.uint8 = c_base_header.uint8;
	nb_ip_header = 0;

	do
	{

		// If IPV4
		if(c_base_header.ipv4_static->version_flag == 0)
		{
			protocol = c_base_header.ipv4_static->protocol;
			size = sizeof(ipv4_static_t);
			rohc_debugf(3, "IP v4 static part length %d\n",size);
			TraceData(c_base_header.uint8,size);
		}
		else
		{
			// IPV6
			if(c_base_header.ipv6_static1->flow_label_enc_discriminator == 0)
			{
				protocol = c_base_header.ipv6_static1->next_header;
				size = sizeof(ipv6_static1_t);
			}
			else
			{
				protocol = c_base_header.ipv6_static2->next_header;
				size = sizeof(ipv6_static2_t);
			}
			rohc_debugf(3, "IP v6 static part length %d\n",size);
			TraceData(c_base_header.uint8,size);
			// For all IPv6 options
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				c_base_header.uint8 += size;
				length += size;
				rohc_debugf(3, "length %d protocol %d profile %Xh protocol %d\n",length,protocol,
				            profile_v2->profile.id,
				            profile_v2->protocol);
				switch(protocol)
				{
					case IPPROTO_HOPOPTS:  // IPv6 Hop-by-Hop options
						size = sizeof(ip_hop_opt_static_t);
						break;
					case IPPROTO_ROUTING:  // IPv6 routing header
						size = (c_base_header.ip_opt_static->length + 1) << 3;
						break;
					case IPPROTO_GRE:
						if(c_base_header.ip_gre_opt_static->k_flag != 0)
						{
							size = sizeof(ip_gre_opt_static_t);
							break;
						}
						size = sizeof(ip_gre_opt_static_t) - sizeof(u_int32_t);
						break;
					case IPPROTO_DSTOPTS:  // IPv6 destination options
						size = sizeof(ip_dest_opt_static_t);
						break;
					case IPPROTO_MIME:
						if(c_base_header.ip_mime_opt_static->s_bit != 0)
						{
							size = sizeof(ip_mime_opt_static_t);
							break;
						}
						size = sizeof(ip_mime_opt_static_t) - sizeof(u_int32_t);
						break;
					case IPPROTO_AH:
						size = sizeof(ip_ah_opt_static_t);
						break;
					default:
						length = 0;
						goto quit;
				}
				rohc_debugf(3, "IP v6 option %d static part length %d\n",protocol,size);
				TraceData(c_base_header.uint8,size);
				protocol = c_base_header.ip_opt_static->next_header;
			}
		}

		c_base_header.uint8 += size;
		length += size;
		rohc_debugf(3, "length %d protocol %d profile %Xh protocol %d\n",length,protocol,
		            profile_v2->profile.id,
		            profile_v2->protocol);

		// On IP header more
		++nb_ip_header;
		fflush(stdout);
	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	/* XXP static part (see RFC 4996 page 73) */

	if( ( size = profile_v2->static_part_size ) != 0)
	{
		rohc_debugf(3, "XXP static part length %d\n",size);
		TraceData(c_base_header.uint8,size);

		c_base_header.uint8 += size;
		length += size;
	}

	/* dynamic chain */

	i = 1;

	do
	{

		// If IPV4
		if(bc_base_header.ipv4_static->version_flag == 0)
		{
			protocol = bc_base_header.ipv4_static->protocol;
			bsize = sizeof(ipv4_static_t);
			// If is_innermost
			if(i == nb_ip_header)
			{
				if(context->profile->id == PROFILE_IP_0104)
				{
					if(c_base_header.ipv4_endpoint_innermost_dynamic1->ip_id_behavior_innermost ==
					   IP_ID_BEHAVIOR_ZERO)
					{
						size = sizeof(ipv4_endpoint_innermost_dynamic1_t);
					}
					else
					{
						size = sizeof(ipv4_endpoint_innermost_dynamic2_t);
					}
				}
				else
				{
					if(c_base_header.ipv4_regular_innermost_dynamic1->ip_id_behavior_innermost ==
					   IP_ID_BEHAVIOR_ZERO)
					{
						size = sizeof(ipv4_regular_innermost_dynamic1_t);
					}
					else
					{
						size = sizeof(ipv4_regular_innermost_dynamic2_t);
					}
				}
			}
			else
			{
				if(c_base_header.ipv4_outer_dynamic1->ip_id_behavior_outer == IP_ID_BEHAVIOR_ZERO)
				{
					size = sizeof(ipv4_outer_dynamic1_t);
				}
				else
				{
					size = sizeof(ipv4_outer_dynamic2_t);
				}
			}
			rohc_debugf(3, "IP v4 dynamic part length %d\n",size);
			TraceData(c_base_header.uint8,size);
		}
		else
		{
			// IPV6
			if(bc_base_header.ipv6_static1->flow_label_enc_discriminator == 0)
			{
				protocol = bc_base_header.ipv6_static1->next_header;
				bsize = sizeof(ipv6_static1_t);
			}
			else
			{
				protocol = bc_base_header.ipv6_static2->next_header;
				bsize = sizeof(ipv6_static2_t);
			}
			// If is_innermost and profile 0x0104
			if(i == nb_ip_header && context->profile->id == PROFILE_IP_0104)
			{
				size = sizeof(ipv6_endpoint_dynamic_t);
			}
			else
			{
				size = sizeof(ipv6_regular_dynamic_t);
			}
			rohc_debugf(3, "IP v6 dynamic part length %d\n",size);
			TraceData(c_base_header.uint8,size);
			// For all IPv6 options
			while( ( ipproto_specifications[protocol] & IPV6_OPTION ) != 0)
			{
				bc_base_header.uint8 += bsize;
				c_base_header.uint8 += size;
				length += size;
				rohc_debugf(3, "next_header %d length %d\n",protocol,length);
				switch(protocol)
				{
					case IPPROTO_HOPOPTS:  // IPv6 Hop-by-Hop options
						bsize = sizeof(ip_hop_opt_static_t);
						size = ( (bc_base_header.ip_hop_opt_static->length + 1) << 3 ) - 2;
						break;
					case IPPROTO_ROUTING:  // IPv6 routing header
						bsize = (bc_base_header.ip_rout_opt_static->length + 1) << 3;
						size = 0;
						break;
					case IPPROTO_GRE:
						size = sizeof(u_int16_t) * bc_base_header.ip_gre_opt_static->c_flag;
						size += sizeof(u_int32_t) * bc_base_header.ip_gre_opt_static->s_flag;
						if(bc_base_header.ip_gre_opt_static->k_flag != 0)
						{
							bsize = sizeof(ip_gre_opt_static_t);
							break;
						}
						bsize = sizeof(ip_gre_opt_static_t) - sizeof(u_int32_t);
						break;
					case IPPROTO_DSTOPTS:  // IPv6 destination options
						bsize = sizeof(ip_dest_opt_static_t);
						size = ( (bc_base_header.ip_dest_opt_static->length + 1) << 3 ) - 2;
						break;
					case IPPROTO_MIME:
						size = 0;
						if(bc_base_header.ip_mime_opt_static->s_bit != 0)
						{
							bsize = sizeof(ip_mime_opt_static_t);
							break;
						}
						bsize = sizeof(ip_mime_opt_static_t) - sizeof(u_int32_t);
						break;
					case IPPROTO_AH:
						bsize = sizeof(ip_ah_opt_static_t);
						size = bc_base_header.ip_ah_opt_static->length << 2;
						break;
				}
				rohc_debugf(3, "IP v6 option %d dynamic part length %d\n",protocol,size);
				TraceData(c_base_header.uint8,size);
				protocol = bc_base_header.ip_opt_static->next_header;
			}
		}

		// One IP header more
		++i;

		bc_base_header.uint8 += bsize;

		c_base_header.uint8 += size;
		length += size;

		rohc_debugf(3, "next_header %d length %d\n",protocol,length);

	}
	while( ( ipproto_specifications[protocol] & IP_TUNNELING ) != 0);

	switch(profile_v2->profile.id)
	{
		case ROHCv2_PROFILE_RTP:
			size = sizeof(udp_regular_dynamic_t);
			c_base_header.uint8 += size;
			size += sizeof(rtp_dynamic_t);
			{
				multi_ptr_t mptr;
				mptr.rtp_dynamic = c_base_header.rtp_dynamic + 1;
				size += d_sdvl_or_default_size(&mptr,c_base_header.rtp_dynamic->tss_indicator);
				size += d_sdvl_or_default_size(&mptr,c_base_header.rtp_dynamic->tis_indicator);
				if(c_base_header.rtp_dynamic->list_present != 0)
				{
					size += d_csrc_list_dynchain_size(mptr);
				}
			}
			break;
		case ROHCv2_PROFILE_UDP:
			size = sizeof(udp_endpoint_dynamic_t);
			break;
		case ROHCv2_PROFILE_ESP:
			size = sizeof(esp_dynamic_t);
			break;
		case ROHCv2_PROFILE_IP:
			size = 0;
			break;
		case ROHCv2_PROFILE_UDPLITE_RTP:
			if(c_base_header.udp_lite_regular_dynamic1->coverage_behavior ==
			   UDP_LITE_COVERAGE_INFERRED)
			{
				size = sizeof(udp_lite_regular_dynamic1_t);
			}
			else
			{
				size = sizeof(udp_lite_regular_dynamic2_t);
			}
			c_base_header.uint8 += size;
			size += sizeof(rtp_dynamic_t);
			break;
		case ROHCv2_PROFILE_UDPLITE:
			if(c_base_header.udp_lite_endpoint_dynamic1->coverage_behavior ==
			   UDP_LITE_COVERAGE_INFERRED)
			{
				size = sizeof(udp_lite_endpoint_dynamic1_t);
			}
			else
			{
				size = sizeof(udp_lite_endpoint_dynamic2_t);
			}
			break;
	}

	rohc_debugf(3, "XXP dynamic size %d\n",size);

	length += size;

	rohc_debugf(3, "IR size %d\n",length);

quit:
	return length;
}


/**
 * @brief Decode the static IP v6 option header of the rohc packet.
 *
 * @param v2_context     The specific RoHCv2 decompression context
 * @param ip_context     The specific IP decompression context
 * @param protocol       The IPv6 protocol option
 * @param c_base_header  The compressed IP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param base_header    The decoded IP packet
 * @return               The length of static IP header
 *                       0 if an error occurs
 */
int tcp_decode_static_ipv6_option(struct d_v2_context *v2_context,
                                  ip_context_ptr_t ip_context,
                                  u_int8_t protocol,
                                  multi_ptr_t c_base_header,
                                  unsigned int length,
                                  base_header_ip_t base_header)
{
	int size;

	rohc_debugf(
	   3, "v2_context %p ip_context %p protocol %d c_base_header %p length %d base_header %p\n",
	   v2_context,ip_context.uint8,protocol,c_base_header.uint8,length,base_header.uint8);

	ip_context.v6_option->next_header = c_base_header.ip_opt_static->next_header;
	base_header.ipv6_opt->next_header = c_base_header.ip_opt_static->next_header;

	switch(protocol)
	{
		case IPPROTO_HOPOPTS:  // IPv6 Hop-by-Hop options
			ip_context.v6_option->option_length = ( c_base_header.ip_opt_static->length + 1 ) << 3;
			ip_context.v6_option->context_length = 2 + ip_context.v6_option->option_length;
			rohc_debugf(3, "IP v6 option Hop-by-Hop length %d context_length %d option_length %d\n",
			            c_base_header.ip_opt_static->length,ip_context.v6_option->context_length,
			            ip_context.v6_option->option_length);
			ip_context.v6_option->length = c_base_header.ip_opt_static->length;
			base_header.ipv6_opt->length = ip_context.v6_option->length;
			size = sizeof(ip_hop_opt_static_t);
			break;
		case IPPROTO_ROUTING:  // IPv6 routing header
			size = ( c_base_header.ip_opt_static->length + 1 ) << 3;
			ip_context.v6_option->context_length = 2 + size;
			ip_context.v6_option->option_length = size;
			memcpy(&ip_context.v6_option->length,&c_base_header.ip_rout_opt_static->length,size - 1);
			memcpy(&base_header.ipv6_opt->length,&ip_context.v6_option->length,size - 1);
			break;
		case IPPROTO_GRE:
			ip_context.v6_option->context_length = sizeof(ipv6_gre_option_context_t);
			size = c_base_header.ip_gre_opt_static->c_flag +
			       c_base_header.ip_gre_opt_static->k_flag +
			       c_base_header.ip_gre_opt_static->s_flag + 1;
			ip_context.v6_option->option_length = size << 3;
			if( ( ip_context.v6_gre_option->protocol ==
			      c_base_header.ip_gre_opt_static->protocol ) == 0)
			{
				base_header.ip_gre_opt->protocol = htons(0x0800);
			}
			else
			{
				base_header.ip_gre_opt->protocol = htons(0x86DD);
			}
			ip_context.v6_gre_option->c_flag = c_base_header.ip_gre_opt_static->c_flag;
			base_header.ip_gre_opt->c_flag = ip_context.v6_gre_option->c_flag;
			ip_context.v6_gre_option->s_flag = c_base_header.ip_gre_opt_static->s_flag;
			base_header.ip_gre_opt->s_flag = ip_context.v6_gre_option->s_flag;
			if( ( ip_context.v6_gre_option->k_flag = c_base_header.ip_gre_opt_static->k_flag ) != 0)
			{
				base_header.ip_gre_opt->k_flag = 1;
				ip_context.v6_gre_option->key = c_base_header.ip_gre_opt_static->key;
				base_header.ip_gre_opt->datas[ip_context.v6_gre_option->c_flag] =
				   ip_context.v6_gre_option->key;
				size = sizeof(ip_gre_opt_static_t);
				break;
			}
			base_header.ip_gre_opt->k_flag = 0;
			size = sizeof(ip_gre_opt_static_t) - sizeof(u_int32_t);
			break;
		case IPPROTO_DSTOPTS:  // IPv6 destination options
			ip_context.v6_option->option_length = ( c_base_header.ip_opt_static->length + 1 ) << 3;
			ip_context.v6_option->context_length = 2 + ip_context.v6_option->option_length;
			rohc_debugf(3, "IP v6 option Destination length %d context_length %d option_length %d\n",
			            c_base_header.ip_opt_static->length,ip_context.v6_option->context_length,
			            ip_context.v6_option->option_length);
			ip_context.v6_option->length = c_base_header.ip_dest_opt_static->length;
			base_header.ipv6_opt->length = ip_context.v6_option->length;
			size = sizeof(ip_dest_opt_static_t);
			break;
		case IPPROTO_MIME:
			ip_context.v6_option->context_length = sizeof(ipv6_mime_option_context_t);
			ip_context.v6_option->option_length = ( 2 + c_base_header.ip_mime_opt_static->s_bit ) << 3;
			ip_context.v6_mime_option->s_bit = c_base_header.ip_mime_opt_static->s_bit;
			base_header.ip_mime_opt->s_bit = ip_context.v6_mime_option->s_bit;
			ip_context.v6_mime_option->res_bits = c_base_header.ip_mime_opt_static->res_bits;
			base_header.ip_mime_opt->res_bits = ip_context.v6_mime_option->res_bits;
			ip_context.v6_mime_option->orig_dest = c_base_header.ip_mime_opt_static->orig_dest;
			base_header.ip_mime_opt->orig_dest = ip_context.v6_mime_option->orig_dest;
			if(ip_context.v6_mime_option->s_bit != 0)
			{
				ip_context.v6_mime_option->orig_src = c_base_header.ip_mime_opt_static->orig_src;
				base_header.ip_mime_opt->orig_src = ip_context.v6_mime_option->orig_src;
				size = sizeof(ip_mime_opt_static_t);
				break;
			}
			size = sizeof(ip_mime_opt_static_t) - sizeof(u_int32_t);
			break;
		case IPPROTO_AH:
			ip_context.v6_option->context_length = sizeof(ipv6_ah_option_context_t);
			ip_context.v6_option->option_length = sizeof(ip_ah_opt_t) - sizeof(u_int32_t) +
			                                      ( c_base_header.ip_ah_opt_static->length <<
			                                        4 ) - sizeof(int32_t);
			ip_context.v6_ah_option->length = c_base_header.ip_ah_opt_static->length;
			base_header.ipv6_opt->length = ip_context.v6_ah_option->length;
			ip_context.v6_ah_option->spi = c_base_header.ip_ah_opt_static->spi;
			base_header.ip_ah_opt->spi = ip_context.v6_ah_option->spi;
			size = sizeof(ip_ah_opt_static_t);
			break;
		default:
			size = 0;
			break;
	}

	#if ROHCV2_DEBUG
	rohc_debugf(3, "IP v6 option static part length %d\n",size);
	TraceData(c_base_header.uint8,size);
	#endif
	fflush(stdout);
	return size;
}


/**
 * @brief Copy the static IP part to the IPv6 option header
 *
 * @param protocol       The IPv6 protocol option
 * @param ip_context     The specific IP decompression context
 * @param base_header    The IP header
 * @return               The size of the static part
 */
unsigned int tcp_copy_static_ipv6_option(u_int8_t protocol,
                                         ip_context_ptr_t ip_context,
                                         base_header_ip_t base_header)
{
	int size;

	rohc_debugf(3, "protocol %d ip_context %p base_header %p\n",protocol,ip_context.uint8,
	            base_header.ipvx);

	base_header.ipv6_opt->next_header = ip_context.v6_option->next_header;

	switch(protocol)
	{
		case IPPROTO_HOPOPTS:  // IPv6 Hop-by-Hop options
			//             base_header.ipv6_opt->length = ip_context.v6_option->length;
			size = ( ip_context.v6_option->length + 1 ) << 3;
			memcpy(&base_header.ipv6_opt->length,&ip_context.v6_option->length,size - 1);
			break;
		case IPPROTO_ROUTING:  // IPv6 routing header
			size = (ip_context.v6_option->length + 1) << 3;
			memcpy(&base_header.ipv6_opt->length,&ip_context.v6_option->length,size - 1);
			break;
		case IPPROTO_GRE:
			base_header.ip_gre_opt->r_flag = 0;
						 #if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header.ip_gre_opt->reserved1 = 0;
			base_header.ip_gre_opt->reserved2 = 0;
						 #elif __BYTE_ORDER == __BIG_ENDIAN
			base_header.ip_gre_opt->reserved0 = 0;
						 #endif
			base_header.ip_gre_opt->version = 0;
			if(ip_context.v6_gre_option->protocol == 0)
			{
				base_header.ip_gre_opt->protocol = htons(0x0800);
			}
			else
			{
				base_header.ip_gre_opt->protocol = htons(0x86DD);
			}
			base_header.ip_gre_opt->c_flag = ip_context.v6_gre_option->c_flag;
			base_header.ip_gre_opt->s_flag = ip_context.v6_gre_option->s_flag;
			if( ( base_header.ip_gre_opt->k_flag = ip_context.v6_gre_option->k_flag ) != 0)
			{
				base_header.ip_gre_opt->datas[ip_context.v6_gre_option->c_flag] =
				   ip_context.v6_gre_option->key;
			}
			size = sizeof(ip_gre_opt_t) - sizeof(u_int32_t) +
			       ( ( ip_context.v6_gre_option->c_flag + ip_context.v6_gre_option->k_flag +
			           ip_context.v6_gre_option->s_flag ) * sizeof(u_int32_t) );
			break;
		case IPPROTO_DSTOPTS:  // IPv6 destination options
			//     base_header.ipv6_opt->length = ip_context.v6_option->length;
			size = ( ip_context.v6_option->length + 1 ) << 3;
			memcpy(&base_header.ipv6_opt->length,&ip_context.v6_option->length,size - 1);
			break;
		case IPPROTO_MIME:

			base_header.ip_mime_opt->s_bit = ip_context.v6_mime_option->s_bit;
			base_header.ip_mime_opt->res_bits = ip_context.v6_mime_option->res_bits;
			base_header.ip_mime_opt->orig_dest = ip_context.v6_mime_option->orig_dest;
			if(ip_context.v6_mime_option->s_bit != 0)
			{
				base_header.ip_mime_opt->orig_src = ip_context.v6_mime_option->orig_src;
				size = sizeof(ip_mime_opt_t);
				break;
			}
			size = sizeof(ip_mime_opt_t) - sizeof(u_int32_t);
			break;
		case IPPROTO_AH:
			base_header.ip_ah_opt->length = ip_context.v6_ah_option->length;
			base_header.ip_ah_opt->res_bits = 0;
			base_header.ip_ah_opt->spi = ip_context.v6_ah_option->spi;
			size = sizeof(ip_ah_opt_t) + ( ip_context.v6_ah_option->length << 4 );
			break;
		default:
			size = 0;
			break;
	}

	return size;
}


/**
 * @brief Decode the dynamic IP v6 option header of the rohc packet.
 *
 * @param v2_context     The specific RoHCv2 decompression context
 * @param ip_context     The specific IP decompression context
 * @param protocol       The IPv6 protocol option
 * @param c_base_header  The compressed IP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param base_header    The decoded IP packet
 * @return               The length of dynamic IP header
 *                       0 if an error occurs
 */
int tcp_decode_dynamic_ipv6_option(struct d_v2_context *v2_context,
                                   ip_context_ptr_t ip_context,
                                   u_int8_t protocol,
                                   multi_ptr_t c_base_header,
                                   unsigned int length,
                                   base_header_ip_t base_header)
{
	int size;

	rohc_debugf(
	   3, "v2_context %p ip_context %p protocol %d c_base_header %p length %d base_header %p\n",
	   v2_context,ip_context.uint8,protocol,c_base_header.uint8,length,base_header.uint8);

	switch(protocol)
	{
		case IPPROTO_HOPOPTS:  // IPv6 Hop-by-Hop options
		case IPPROTO_DSTOPTS:  // IPv6 destination options
			size = ( (ip_context.v6_option->length + 1) << 3 ) - 2;
			memcpy(ip_context.v6_option->value,c_base_header.uint8,size);
			memcpy(base_header.ipv6_opt->value,ip_context.v6_option->value,size);
			break;
		case IPPROTO_ROUTING:  // IPv6 routing header
			size = 0;
			break;
		case IPPROTO_GRE:
			size = 0;
			if(ip_context.v6_gre_option->c_flag != 0)
			{
				base_header.ip_gre_opt->datas[0] = READ32_FROM_MPTR(c_base_header);
				size += sizeof(u_int32_t);
			}
			if(ip_context.v6_gre_option->s_flag != 0)
			{
				base_header.ip_gre_opt->datas[ip_context.v6_gre_option->c_flag] = READ32_FROM_MPTR(
				   c_base_header);
				size += sizeof(u_int32_t);
			}
								#if ROHCV2_DEBUG
			c_base_header.uint8 -= size;
								#endif
			break;
		case IPPROTO_MIME:
			size = 0;
			break;
		case IPPROTO_AH:
			ip_context.v6_ah_option->sequence_number =
			   c_base_header.ip_ah_opt_dynamic->sequence_number;
			size = (ip_context.v6_ah_option->length - 1) << 2;
			memcpy(ip_context.v6_ah_option->icv,c_base_header.ip_ah_opt_dynamic->icv,size);
			size += sizeof(u_int32_t);
			break;
		default:
			size = 0;
			break;
	}

	#if ROHCV2_DEBUG
	rohc_debugf(3, "IP v6 option %d dynamic part length %d\n",protocol,size);
	TraceData(c_base_header.uint8,size);
	#endif
	fflush(stdout);
	return size;
}


/**
 * @brief Decode the irregular IP v6 option header of the rohc packet.
 *
 * @param v2_context     The specific RoHCv2 decompression context
 * @param ip_context     The specific IP decompression context
 * @param protocol       The IP v6 protocol option
 * @param mptr           The compressed IP header of the rohc packet
 * @param base_header    The decoded IP packet
 * @return               The length of dynamic IP header
 *                       0 if an error occurs
 */
u_int8_t * tcp_decode_irregular_ipv6_option(struct d_v2_context *v2_context,
                                            ip_context_ptr_t ip_context,
                                            u_int8_t protocol,
                                            multi_ptr_t mptr,
                                            base_header_ip_t base_header)
{
	#if ROHCV2_DEBUG
	u_int8_t *ptr = mptr.uint8;
	#endif
	u_int32_t sequence_number;
	int size;

	rohc_debugf(3, "v2_context %p ip_context %p protocol %d mptr %p base_header %p\n",v2_context,
	            ip_context.uint8,protocol,mptr.uint8,
	            base_header.uint8);

	switch(protocol)
	{
		case IPPROTO_GRE:
			if(ip_context.v6_gre_option->c_flag != 0)
			{
				base_header.ip_gre_opt->datas[0] = READ32_FROM_MPTR(mptr);
			}
			if(ip_context.v6_gre_option->s_flag != 0)
			{
				if( ( *mptr.uint8 & 0x80 ) == 0)
				{
					// discriminator =:= '0'
					sequence_number =
					   ( ip_context.v6_gre_option->sequence_number & 0xFFFFFF80 ) | *(mptr.uint8++);
				}
				else
				{
					// discriminator =:= '1'
					sequence_number = ( ip_context.v6_gre_option->sequence_number & 0x80000000 ) | ntohl(
					   READ32_FROM_MPTR(mptr));
				}
				base_header.ip_gre_opt->datas[ip_context.v6_gre_option->c_flag] = htonl(sequence_number);
				ip_context.v6_gre_option->sequence_number = sequence_number;
			}
			break;
		case IPPROTO_AH:
			// sequence_number =:= lsb_7_or_31
			if( ( *mptr.uint8 & 0x80 ) == 0)
			{
				// discriminator =:= '0'
				sequence_number =
				   ( ip_context.v6_ah_option->sequence_number & 0xFFFFFF80 ) | *(mptr.uint8++);
			}
			else
			{
				// discriminator =:= '1'
				sequence_number = ( ip_context.v6_ah_option->sequence_number & 0x80000000 ) | ntohl(
				   READ32_FROM_MPTR(mptr));
			}
			ip_context.v6_ah_option->sequence_number = sequence_number;
			base_header.ip_ah_opt->sequence_number = htonl(sequence_number);
			size = (ip_context.v6_ah_option->length - 1) << 2;
			memcpy(ip_context.v6_ah_option->icv,mptr.uint8,size);
			mptr.uint8 += size;
			break;
		default:
			break;
	}

	#if ROHCV2_DEBUG
	rohc_debugf(3, "IP v6 option irregular part length %d\n",(int)(mptr.uint8 - ptr));
	TraceData(ptr,mptr.uint8 - ptr);
	#endif
	fflush(stdout);
	return mptr.uint8;
}


/**
 * @brief Decode the static IP header of the rohc packet.
 *
 * @param v2_context     The specific RoHCv2 decompression context
 * @param ip_context     The specific IP decompression context
 * @param c_base_header  The compressed IP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param base_header    The decoded IP packet
 * @return               The length of static IP header
 *                       0 if an error occurs
 */
int tcp_decode_static_ip(struct d_v2_context *v2_context,
                         ip_context_ptr_t ip_context,
                         multi_ptr_t c_base_header,
                         unsigned int length,
                         base_header_ip_t base_header)
{
	int size;

	rohc_debugf(3, "v2_context %p ip_context %p c_base_header %p length %d base_header %p\n",
	            v2_context,ip_context.uint8,c_base_header.uint8,length,base_header.uint8);

	if(c_base_header.ipv4_static->version_flag == 0)
	{
		base_header.ipv4->version = IPV4;
		base_header.ipv4->hdr_length = sizeof(base_header_ipv4_t) >> 2;
		base_header.ipv4->protocol = c_base_header.ipv4_static->protocol;
		base_header.ipv4->src_addr = c_base_header.ipv4_static->src_addr;
		base_header.ipv4->dest_addr = c_base_header.ipv4_static->dst_addr;

		ip_context.v4->version = IPV4;
		ip_context.v4->context_length = sizeof(ipv4_context_t);
		ip_context.v4->protocol = c_base_header.ipv4_static->protocol;
		ip_context.v4->src_addr = c_base_header.ipv4_static->src_addr;
		ip_context.v4->dst_addr = c_base_header.ipv4_static->dst_addr;
		size = sizeof(ipv4_static_t);
	}
	else
	{
		base_header.ipv6->version = IPV6;
		ip_context.v6->version = IPV6;
		ip_context.v6->context_length = sizeof(ipv6_context_t);
		if(c_base_header.ipv6_static1->flow_label_enc_discriminator == 0)
		{
			base_header.ipv6->flow_label1 = 0;
			base_header.ipv6->flow_label2 = 0;
			base_header.ipv6->next_header = c_base_header.ipv6_static1->next_header;
			memcpy(base_header.ipv6->src_addr,c_base_header.ipv6_static1->src_addr,
			       sizeof(u_int32_t) * 4 * 2);

			ip_context.v6->flow_label1 = 0;
			ip_context.v6->flow_label2 = 0;
			ip_context.v6->next_header = c_base_header.ipv6_static1->next_header;
			memcpy(ip_context.v6->src_addr,c_base_header.ipv6_static1->src_addr,
			       sizeof(u_int32_t) * 4 * 2);
			size = sizeof(ipv6_static1_t);
		}
		else
		{
			base_header.ipv6->flow_label1 = c_base_header.ipv6_static2->flow_label1;
			base_header.ipv6->flow_label2 = c_base_header.ipv6_static2->flow_label2;
			base_header.ipv6->next_header = c_base_header.ipv6_static2->next_header;
			memcpy(base_header.ipv6->src_addr,c_base_header.ipv6_static2->src_addr,
			       sizeof(u_int32_t) * 4 * 2);

			ip_context.v6->flow_label1 = c_base_header.ipv6_static2->flow_label1;
			ip_context.v6->flow_label2 = c_base_header.ipv6_static2->flow_label2;
			ip_context.v6->next_header = c_base_header.ipv6_static2->next_header;
			memcpy(ip_context.v6->src_addr,c_base_header.ipv6_static2->src_addr,
			       sizeof(u_int32_t) * 4 * 2);
			size = sizeof(ipv6_static2_t);
		}
		ip_context.v6->ip_id_behavior = IP_ID_BEHAVIOR_RANDOM;
	}

	rohc_debugf(3, "IP v%d static part length %d\n",base_header.ipvx->version,size);
	TraceData(c_base_header.uint8,size);
	fflush(stdout);
	return size;
}


/**
 * @brief Copy the static IP part to the IP header
 *
 * @param ip_context    The specific IP decompression context
 * @param base_header   The IP header
 * @return              The size of the static part
 */
unsigned int tcp_copy_static_ip(ip_context_ptr_t ip_context,
                                base_header_ip_t base_header)
{
	rohc_debugf(3, "ip_context %p base_header %p\n",ip_context.uint8,base_header.ipvx);

	if(ip_context.vx->version == IPV4)
	{
		base_header.ipv4->version = IPV4;
		base_header.ipv4->hdr_length = sizeof(base_header_ipv4_t) >> 2;
		// mf = 0 , rf = 0 , frag_offset = 0
		// base_header.uint16[3] = 0;
		base_header.ipv4->mf = 0;
		base_header.ipv4->rf = 0;
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header.ipv4->frag_offset1 = 0;
		base_header.ipv4->frag_offset2 = 0;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header.ipv4->frag_offset = 0;
		#endif
		base_header.ipv4->protocol = ip_context.v4->protocol;
		base_header.ipv4->src_addr = ip_context.v4->src_addr;
		base_header.ipv4->dest_addr = ip_context.v4->dst_addr;
		return sizeof(base_header_ipv4_t);
	}
	else
	{
		base_header.ipv6->version = IPV6;
		base_header.ipv6->flow_label1 = ip_context.v6->flow_label1;
		base_header.ipv6->flow_label2 = ip_context.v6->flow_label2;
		base_header.ipv6->next_header = ip_context.v6->next_header;
		memcpy(base_header.ipv6->src_addr,ip_context.v6->src_addr,sizeof(u_int32_t) * 4 * 2);
		return sizeof(base_header_ipv6_t);
	}
}


/**
 * @brief Decode the dynamic IP header of the rohc packet.
 *
 * @param v2_context      The specific RoHCv2 decompression context
 * @param ip_context      The specific IP decompression context
 * @param c_base_header   The dynamic compressed IP header of the rohc packet
 * @param length          The remain length of the rohc packet
 * @param base_header     The decoded IP packet
 * @param is_innermost    True if the IP header is the innermost of the packet
 * @param profile_value   The profile value used
 * @return                The length of dynamic IP header
 *                        0 if an error occurs
 */
int tcp_decode_dynamic_ip(struct d_v2_context *v2_context,
                          ip_context_ptr_t ip_context,
                          multi_ptr_t c_base_header,
                          unsigned int length,
                          base_header_ip_t base_header,
                          int is_innermost,
                          int profile_value)
{
	int size;

	rohc_debugf(
	   3,
	   "v2_context %p ip_context %p c_base_header %p length %d base_header %p is_innermost %d profile_value %Xh\n",
	   v2_context,ip_context.uint8,c_base_header.uint8,length,base_header.uint8,
	   is_innermost,profile_value);

	// If IPV4
	if(ip_context.vx->version == IPV4)
	{
		// If is_innermost
		if(is_innermost != 0)
		{
			if(profile_value == PROFILE_IP_0104)
			{
				v2_context->reorder_ratio =
				   c_base_header.ipv4_endpoint_innermost_dynamic1->reorder_ratio;
				base_header.ipv4->df = c_base_header.ipv4_endpoint_innermost_dynamic1->df;
				rohc_debugf(3, "df = %d\n",base_header.ipv4->df);
				ip_context.v4->ip_id_behavior =
				   c_base_header.ipv4_endpoint_innermost_dynamic1->ip_id_behavior_innermost;
				rohc_debugf(3, "ip_id_behavior %d\n",ip_context.v4->ip_id_behavior);
				if(c_base_header.ipv4_endpoint_innermost_dynamic1->ip_id_behavior_innermost ==
				   IP_ID_BEHAVIOR_ZERO)
				{
					base_header.ipv4->tos_tc = c_base_header.ipv4_endpoint_innermost_dynamic1->tos_tc;
					base_header.ipv4->ttl_hopl =
					   c_base_header.ipv4_endpoint_innermost_dynamic1->ttl_hopl;
					base_header.ipv4->ip_id = 0;
					v2_context->msn = ntohs(c_base_header.ipv4_endpoint_innermost_dynamic1->msn);
					size = sizeof(ipv4_endpoint_innermost_dynamic1_t);
				}
				else
				{
					base_header.ipv4->tos_tc = c_base_header.ipv4_endpoint_innermost_dynamic2->tos_tc;
					base_header.ipv4->ttl_hopl =
					   c_base_header.ipv4_endpoint_innermost_dynamic2->ttl_hopl;
					base_header.ipv4->ip_id = c_base_header.ipv4_endpoint_innermost_dynamic2->ip_id;
					v2_context->msn = ntohs(c_base_header.ipv4_endpoint_innermost_dynamic2->msn);
					size = sizeof(ipv4_endpoint_innermost_dynamic2_t);
				}
			}
			else
			{
				base_header.ipv4->df = c_base_header.ipv4_regular_innermost_dynamic1->df;
				rohc_debugf(3, "df = %d\n",base_header.ipv4->df);
				ip_context.v4->ip_id_behavior =
				   c_base_header.ipv4_regular_innermost_dynamic1->ip_id_behavior_innermost;
				rohc_debugf(3, "ip_id_behavior %d\n",ip_context.v4->ip_id_behavior);
				base_header.ipv4->tos_tc = c_base_header.ipv4_regular_innermost_dynamic1->tos_tc;
				base_header.ipv4->ttl_hopl = c_base_header.ipv4_regular_innermost_dynamic1->ttl_hopl;
				if(c_base_header.ipv4_regular_innermost_dynamic1->ip_id_behavior_innermost ==
				   IP_ID_BEHAVIOR_ZERO)
				{
					base_header.ipv4->ip_id = 0;
					size = sizeof(ipv4_regular_innermost_dynamic1_t);
				}
				else
				{
					base_header.ipv4->ip_id = c_base_header.ipv4_regular_innermost_dynamic2->ip_id;
					size = sizeof(ipv4_regular_innermost_dynamic2_t);
				}
			}
		}
		else
		{
			base_header.ipv4->df = c_base_header.ipv4_outer_dynamic1->df;
			rohc_debugf(3, "df = %d\n",base_header.ipv4->df);
			ip_context.v4->ip_id_behavior = c_base_header.ipv4_outer_dynamic1->ip_id_behavior_outer;
			rohc_debugf(3, "ip_id_behavior %d\n",ip_context.v4->ip_id_behavior);
			base_header.ipv4->tos_tc = c_base_header.ipv4_outer_dynamic1->tos_tc;
			base_header.ipv4->ttl_hopl = c_base_header.ipv4_outer_dynamic1->ttl_hopl;
			if(c_base_header.ipv4_outer_dynamic1->ip_id_behavior_outer == IP_ID_BEHAVIOR_ZERO)
			{
				base_header.ipv4->ip_id = 0;
				size = sizeof(ipv4_outer_dynamic1_t);
			}
			else
			{
				base_header.ipv4->ip_id = c_base_header.ipv4_outer_dynamic2->ip_id;
				size = sizeof(ipv4_outer_dynamic2_t);
			}
		}
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header.ipv4->frag_offset1 = 0;
		base_header.ipv4->frag_offset2 = 0;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header.ipv4->frag_offset = 0;
		#endif
		base_header.ipv4->mf = 0;
		base_header.ipv4->rf = 0;
		// Save value to context
		ip_context.v4->df = base_header.ipv4->df;
		ip_context.v4->tos_tc = base_header.ipv4->tos_tc;
		ip_context.v4->ttl_hopl = base_header.ipv4->ttl_hopl;
		ip_context.v4->last_ip_id.uint16 = ntohs(base_header.ipv4->ip_id);
		rohc_debugf(3, "ip_id_behavior %d\n",ip_context.v4->ip_id_behavior);
	}
	else
	{
		// If is_innermost and profile 0x0104
		if(is_innermost != 0 && profile_value == PROFILE_IP_0104)
		{
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header.ipv6->tos_tc1 = c_base_header.ipv6_endpoint_dynamic->tos_tc >> 4;
			base_header.ipv6->tos_tc2 = c_base_header.ipv6_endpoint_dynamic->tos_tc & 0x0F;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header.ipv6->tos_tc = c_base_header.ipv6_endpoint_dynamic->tos_tc;
			#endif
			base_header.ipv6->ttl_hopl = c_base_header.ipv6_endpoint_dynamic->ttl_hopl;
			v2_context->reorder_ratio = c_base_header.ipv6_endpoint_dynamic->reorder_ratio;
			v2_context->msn = ntohs(c_base_header.ipv6_endpoint_dynamic->msn);
			size = sizeof(ipv6_endpoint_dynamic_t);
		}
		else
		{
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header.ipv6->tos_tc1 = c_base_header.ipv6_endpoint_dynamic->tos_tc >> 4;
			base_header.ipv6->tos_tc2 = c_base_header.ipv6_endpoint_dynamic->tos_tc & 0x0F;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header.ipv6->tos_tc = c_base_header.ipv6_regular_dynamic->tos_tc;
			#endif
			base_header.ipv6->ttl_hopl = c_base_header.ipv6_regular_dynamic->ttl_hopl;
			size = sizeof(ipv6_regular_dynamic_t);
		}
		// Save value to context
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		ip_context.v6->tos_tc1 = base_header.ipv6->tos_tc1;
		ip_context.v6->tos_tc2 = base_header.ipv6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		ip_context.v6->tos_tc = base_header.ipv6->tos_tc;
		#endif
		ip_context.v6->ttl_hopl = base_header.ipv6->ttl_hopl;
	}

	rohc_debugf(3, "IP v%d dynamic part length %d\n",ip_context.vx->version,size);
	TraceData(c_base_header.uint8,size);
	fflush(stdout);
	return size;
}


/**
 * @brief Decode the irregular IP header of the rohc packet.
 *
 * @param v2_context         The specific RoHCv2 decompression context
 * @param ip_context         The specific IP decompression context
 * @param base_header        The IP header under built
 * @param mptr               The remain datas of the rohc packet
 * @param is_innermost       True if the IP header is the innermost of the packet
 * @param outer_ip_flag      True if ttl_hopl ou tos_tc of outer IP header has changed
 * @return                   The current point of the remain rohc_datas
 */
u_int8_t * tcp_decode_irregular_ip(struct d_v2_context *v2_context,
                                   ip_context_ptr_t ip_context,
                                   base_header_ip_t base_header,
                                   multi_ptr_t mptr,
                                   int is_innermost,
                                   int outer_ip_flag)
{
	#if ROHCV2_DEBUG
	u_int8_t *ptr = mptr.uint8;
	#endif

	rohc_debugf(3, "v2_context %p ip_context %p base_header %p mptr %p\n",v2_context,
	            ip_context.uint8,base_header.uint8,
	            mptr.uint8);
	rohc_debugf(3, "is_innermost %d outer_ip_flag %d\n",is_innermost,outer_ip_flag);

	if(ip_context.vx->version == IPV4)
	{
		// ip_id =:= ip_id_enc_irreg( ip_id_behavior_outer.UVALUE )
		// ip_id =:= ip_id_enc_irreg( ip_id_behavior_innermost.UVALUE )
		rohc_debugf(3, "ip_id_behavior %d\n",ip_context.v4->ip_id_behavior);
		if(ip_context.v4->ip_id_behavior == IP_ID_BEHAVIOR_RANDOM)
		{
			base_header.ipv4->ip_id = READ16_FROM_MPTR(mptr);
			rohc_debugf(3, "Read ip_id %4.4Xh (ip_id_behavior %d)\n",base_header.ipv4->ip_id,
			            ip_context.v4->ip_id_behavior);
			ip_context.v4->last_ip_id.uint16 = ntohs(base_header.ipv4->ip_id);
			rohc_debugf(3, "new last IP-ID %4.4Xh\n",ip_context.v4->last_ip_id.uint16);
		}
		if(is_innermost == 0)
		{
			// =:= ipv4_outer_irregular
			if(outer_ip_flag != 0)
			{
				// =:= static_or_irreg(outer_ip_flag,8)
				ip_context.v4->tos_tc = *(mptr.uint8++);
				// =:= static_or_irreg(outer_ip_flag,8)
				ip_context.v4->ttl_hopl = *(mptr.uint8++);
			}
			base_header.ipv4->tos_tc = ip_context.v4->tos_tc;
			base_header.ipv4->ttl_hopl = ip_context.v4->ttl_hopl;
		}
		else
		{
			// =:= ipv4_innermost_irregular
		}
		// Save value to context
		ip_context.v4->last_ip_id.uint16 = ntohs(base_header.ipv4->ip_id);
	}
	else
	{
		// IPv6
		if(is_innermost == 0)
		{
			// ipv6_outer_irregular
			if(outer_ip_flag != 0)
			{
				// =:= static_or_irreg(outer_ip_flag,8)
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				u_int8_t tos_tc;
				tos_tc = *(mptr.uint8++);
				ip_context.v6->tos_tc1 = tos_tc >> 4;
				ip_context.v6->tos_tc2 = tos_tc;
				#elif __BYTE_ORDER == __BIG_ENDIAN
				ip_context.v6->tos_tc = *(mptr.uint8++);
				#endif
				// =:= static_or_irreg(outer_ip_flag,8)
				ip_context.v6->ttl_hopl = *(mptr.uint8++);
			}
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header.ipv6->tos_tc1 = ip_context.v6->tos_tc1;
			base_header.ipv6->tos_tc2 = ip_context.v6->tos_tc2;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header.ipv6->tos_tc = ip_context.v6->tos_tc;
			#endif
			base_header.ipv6->ttl_hopl = ip_context.v6->ttl_hopl;
		}
		#if ROHCV2_DEBUG
		else
		{
			// ipv6_innermost_irregular
		}
		#endif
	}

	#if ROHCV2_DEBUG
	rohc_debugf(3, "IP v%d irregular part length %d\n",ip_context.vx->version,(int)(mptr.uint8 - ptr));
	TraceData(ptr,mptr.uint8 - ptr);
	#endif

	return mptr.uint8;
}


/**
 * @brief Decode the IP-only profile of the rohc packet.
 *
 * @param decomp              The ROHC decompressor
 * @param v2_context          The specific RoHCv2 decompression context
 * @param ip_inner_context    The innermost IP decompression context
 * @param base_header_inner   The decoded innermost IP header
 * @param c_base_header       Pointer to the dynamic part of the UDP header of the rohc packet
 * @param length              The remain length of the rohc packet
 * @param outer_ip_flag       The outer IP flag
 * @return                    The length of dynamic part of UDP header
 *                            0 if an error occurs
 */

int tcp_decode_ip_baseheader(struct rohc_decomp *decomp,
                             struct d_v2_context *v2_context,
                             ip_context_ptr_t ip_inner_context,
                             base_header_ip_t base_header_inner,
                             multi_ptr_t c_base_header,
                             int length,
                             int *outer_ip_flag)
{
	u_int8_t discriminator;
	u_int8_t header_crc;
	multi_ptr_t mptr;
	WB_t swapped_ip_id;
	WB_t ip_id;
	int size;

	rohc_debugf(3, "decomp %p v2_context %p ip_inner_context %p\n",decomp,v2_context,
	            ip_inner_context.uint8);
	rohc_debugf(3, "c_base_header %p length %d outer_ip_flag %p\n",c_base_header.uint8,length,
	            outer_ip_flag);

	discriminator = *c_base_header.uint8;
	if(discriminator == 0xFA)
	{
		goto decode_ip_co_common;
	}

	// Init dynamic values with default value from context
	if(ip_inner_context.vx->version == IPV4)
	{
		base_header_inner.ipv4->df = ip_inner_context.v4->df;
		base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
		base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
	}
	else
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
		base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
		#endif
		base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
	}

	if( ( discriminator & 0x80 ) == 0)
	{
		goto decode_ip_pt_0_crc3;
	}
	switch(discriminator & 0xE0)
	{
		case 0x80:
			goto decode_ip_pt_0_crc7;
		case 0xA0:
			goto decode_ip_pt_1_seq_id;
		case 0xC0:
			goto decode_ip_pt_2_seq_id;
		default:
			return -1;
	}

decode_ip_co_common:
	{
		profile_2_3_4_flags_t profile_2_3_4_flags;

		rohc_debugf(3, "IP profile co_common\n");
		mptr.ip_co_common = c_base_header.ip_co_common + 1;
#ifdef LMQJMLKQ // Done before
		size = sizeof(ip_co_common_t) + 1;
		size += c_base_header.ip_co_common->tos_tc_indicator;
		size += c_base_header.ip_co_common->ttl_hopl_indicator;
		if(c_base_header.ip_co_common->flags_indicator != 0)
		{
			++size;
			if(ip_inner_context.vx->version == IPV4)
			{
				if(mptr.profile_2_3_4_flags->name.ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
				{
					size += c_base_header.ip_co_common->ip_id_indicator + 1;
				}
			}
		}
		else
		{
			if(ip_inner_context.vx->version == IPV4)
			{
				if(ip_inner_context.vx->ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
				{
					size += c_base_header.ip_co_common->ip_id_indicator + 1;
				}
			}
		}
		rohc_debugf(3, "IP profile length %d\n",size);
		TraceData(c_base_header.uint8,size);

		header_crc = c_base_header.ip_co_common->header_crc;
		c_base_header.ip_co_common->header_crc = 0;
		c_base_header.ip_co_common->header_crc =
		   crc_calculate(CRC_TYPE_7, c_base_header.uint8, size, CRC_INIT_7,
		                 decomp->crc_table_7);
		rohc_debugf(3, "header_crc %2.2Xh <=> %2.2Xh size %d\n",header_crc,
		            c_base_header.ip_co_common->header_crc,
		            size);
		if(header_crc != c_base_header.ip_co_common->header_crc)
		{
			return ROHC_ERROR;
		}
#endif
		v2_context->reorder_ratio = c_base_header.ip_co_common->reorder_ratio;
		if(c_base_header.ip_co_common->flags_indicator != 0)
		{
			profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
			*outer_ip_flag = profile_2_3_4_flags.name.ip_outer_indicator;
		}
		else
		{
			profile_2_3_4_flags.flags = 0;
		}
		if(ip_inner_context.vx->version == IPV4)
		{
			if(c_base_header.ip_co_common->flags_indicator != 0)
			{
				ip_inner_context.v4->df = profile_2_3_4_flags.name.df;
				ip_inner_context.v4->ip_id_behavior = profile_2_3_4_flags.name.ip_id_behavior;
				rohc_debugf(3, "ip_id_behavior %d\n",ip_inner_context.v4->ip_id_behavior);
			}
			base_header_inner.ipv4->df = ip_inner_context.v4->df;
			if(c_base_header.ip_co_common->tos_tc_indicator != 0)
			{
				ip_inner_context.v4->tos_tc = *(mptr.uint8++);
			}
			base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
			if(c_base_header.ip_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v4->ttl_hopl = *(mptr.uint8++);
				rohc_debugf(3, "new ttl_hopl %Xh\n",ip_inner_context.v4->ttl_hopl);
			}
			base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
			// =:= msn_lsb(8)
			v2_context->msn = d_msn_lsb(v2_context,8,*(mptr.uint8++));
			rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
			// ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,ip_id_indicator.CVALUE)
			d_ip_id_sequential_variable(&mptr,ip_inner_context.v4->ip_id_behavior,
			                            c_base_header.ip_co_common->ip_id_indicator,
			                            ip_inner_context.v4->last_ip_id,&ip_id.uint16,
			                            v2_context->msn);
			base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
			ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
			rohc_debugf(3, "new last IP-ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16);
		}
		else
		{
			if(c_base_header.ip_co_common->tos_tc_indicator != 0)
			{
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				ip_inner_context.v6->tos_tc1 = (*mptr.uint8) >> 1;
				ip_inner_context.v6->tos_tc2 = *(mptr.uint8++);
				#elif __BYTE_ORDER == __BIG_ENDIAN
				ip_inner_context.v6->tos_tc = *(mptr.uint8++);
				#endif
			}
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
			base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
			#endif
			if(c_base_header.ip_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v6->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
			// =:= msn_lsb(8)
			v2_context->msn = d_msn_lsb(v2_context,8,*(mptr.uint8++));
			rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
		}
		size = mptr.uint8 - c_base_header.uint8;
		rohc_debugf(3, "return size %d\n",size);
		return size;
	}

decode_ip_pt_0_crc3:
	rohc_debugf(3, "IP profile pt_0_crc3\n");
	size = sizeof(ip_pt_0_crc3_t);
	header_crc = c_base_header.ip_pt_0_crc3->header_crc;
	c_base_header.ip_pt_0_crc3->header_crc = 0;
	c_base_header.ip_pt_0_crc3->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.ip_pt_0_crc3->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.ip_pt_0_crc3->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(4)
	v2_context->msn = d_msn_lsb(v2_context,4,c_base_header.ip_pt_0_crc3->msn);
	goto idem_decode_ip_pt_0_crc7;

decode_ip_pt_0_crc7:
	rohc_debugf(3, "IP profile pt_0_crc7\n");
	size = sizeof(ip_pt_0_crc7_t);
	header_crc = c_base_header.ip_pt_0_crc7->header_crc;
	c_base_header.ip_pt_0_crc7->header_crc = 0;
	c_base_header.ip_pt_0_crc7->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.ip_pt_0_crc7->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.ip_pt_0_crc7->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	v2_context->msn = d_msn_lsb(
	   v2_context,6,(c_base_header.ip_pt_0_crc7->msn1 << 1) | c_base_header.ip_pt_0_crc7->msn2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.ip_pt_0_crc7->msn);
	#endif
idem_decode_ip_pt_0_crc7:
	if(ip_inner_context.vx->version == IPV4)
	{
		switch(ip_inner_context.v4->ip_id_behavior)
		{
			case IP_ID_BEHAVIOR_SEQUENTIAL:
				++ip_inner_context.v4->last_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(ip_inner_context.v4->last_ip_id.uint16);
				break;
			case IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED:
				swapped_ip_id.uint8[0] = ip_inner_context.v4->last_ip_id.uint8[1];
				swapped_ip_id.uint8[1] = ip_inner_context.v4->last_ip_id.uint8[0];
				++swapped_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(swapped_ip_id.uint16);
				ip_inner_context.v4->last_ip_id.uint8[1] = swapped_ip_id.uint8[0];
				ip_inner_context.v4->last_ip_id.uint8[0] = swapped_ip_id.uint8[1];
				break;
			default:
				break;
		}
	}
	goto end_decode_ip_baseheader;

decode_ip_pt_1_seq_id:
	rohc_debugf(3, "IP profile pt_1_seq_id\n");
	assert( ip_inner_context.vx->version == IPV4 );
	assert( ip_inner_context.v4->ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED );
	size = sizeof(ip_pt_1_seq_id_t);
	header_crc = c_base_header.ip_pt_1_seq_id->header_crc;
	c_base_header.ip_pt_1_seq_id->header_crc = 0;
	c_base_header.ip_pt_1_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.ip_pt_1_seq_id->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.ip_pt_1_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	rohc_debugf(3, "old msn %Xh msn recv %Xh\n",v2_context->msn,
	            (c_base_header.ip_pt_1_seq_id->msn1 << 4) | c_base_header.ip_pt_1_seq_id->msn2);
	v2_context->msn = d_msn_lsb(
	   v2_context,6,(c_base_header.ip_pt_1_seq_id->msn1 << 4) | c_base_header.ip_pt_1_seq_id->msn2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.ip_pt_1_seq_id->msn);
	#endif
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	if(ip_inner_context.vx->version == IPV4)
	{
		// =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4)
//		ip_id.uint16 = ( ip_inner_context.v4->last_ip_id.uint16 & 0xFFF0 ) | d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,c_base_header.udp_pt_1_seq_id->ip_id,v2_context->msn);
		ip_id.uint16 =
		   d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,
		               c_base_header.ip_pt_1_seq_id->ip_id,
		               v2_context->msn);
		goto idem_decode_ip_pt_2_seq_id;
	}
	else
	{
		rohc_debugf(3, "ip_inner_version %d\n",ip_inner_context.vx->version);
	}
	goto end_decode_ip_baseheader;

decode_ip_pt_2_seq_id:
	rohc_debugf(3, "IP profile pt_2_seq_id\n");
	assert( ip_inner_context.vx->version == IPV4 );
	assert( ip_inner_context.v4->ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED );
	size = sizeof(ip_pt_2_seq_id_t);
	header_crc = c_base_header.ip_pt_2_seq_id->header_crc;
	c_base_header.ip_pt_2_seq_id->header_crc = 0;
	c_base_header.ip_pt_2_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.ip_pt_2_seq_id->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.ip_pt_2_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(8)
	v2_context->msn = d_msn_lsb(v2_context,8,c_base_header.ip_pt_2_seq_id->msn);
	if(ip_inner_context.vx->version == IPV4)
	{
		// =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 6)
//		ip_id.uint16 = ( ip_inner_context.v4->last_ip_id.uint16 & 0xFFF0 ) | d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,c_base_header.udp_pt_1_seq_id->ip_id,v2_context->msn);
		ip_id.uint16 =
		   d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,6,6,ip_inner_context.v4->last_ip_id,
		               c_base_header.ip_pt_1_seq_id->ip_id,
		               v2_context->msn);
idem_decode_ip_pt_2_seq_id:
		rohc_debugf(3, "last_ip_id %4.4Xh new IP_ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16,
		            ip_id.uint16);
		base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
		ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
	}
	else
	{
		rohc_debugf(3, "ip_inner_version %d\n",ip_inner_context.vx->version);
	}

end_decode_ip_baseheader:
	if(ip_inner_context.vx->version == IPV4)
	{
		base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
		base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
	}
	else
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
		base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
		#endif
		base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
	}
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	return size;
}


/**
 * @brief Decode the static part of UDP header of the rohc packet.
 *
 * @param udp_context    The specific UDP decompression context
 * @param udp_static     The compressed UDP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param udp            The decoded UDP header
 * @return               The length of static part of UDP header
 *                       0 if an error occurs
 */
int tcp_decode_static_udp(udp_context_t *udp_context,
                          udp_static_t *udp_static,
                          int length,
                          base_header_udp_t *udp)
{
	// Save information to context
	udp_context->protocol = IPPROTO_UDP;
	udp_context->src_port = udp_static->src_port;
	udp_context->dst_port = udp_static->dst_port;

	udp->src_port = udp_static->src_port;
	udp->dst_port = udp_static->dst_port;

	rohc_debugf(3, "UDP static part length %d\n",(int)sizeof(udp_static_t));
	TraceData((unsigned char*)udp_static,sizeof(udp_static_t));

	return sizeof(udp_static_t);
}


/**
 * @brief Copy the static UDP part to the UDP header
 *
 * @param udp_context    The specific UDP decompression context
 * @param udp            The UDP header
 * @return               The size of the static part
 */
unsigned int tcp_copy_static_udp(udp_context_t *udp_context,
                                 base_header_udp_t *udp)
{
	rohc_debugf(3, "udp_context %p udp %p\n",udp_context,udp);

	udp->src_port = udp_context->src_port;
	udp->dst_port = udp_context->dst_port;

	return sizeof(base_header_udp_t);
}


/**
 * @brief Decode the dynamic part of UDP header of the rohc packet.
 *
 * @param v2_context     The specific RoHCv2 decompression context
 * @param udp_context    The specific UDP decompression context
 * @param profile_value  The profile value used
 * @param c_base_header  Pointer to the dynamic part of the UDP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param udp            The decoded UDP header
 * @return               The length of dynamic part of UDP header
 *                       0 if an error occurs
 */
int tcp_decode_dynamic_udp(struct d_v2_context *v2_context,
                           udp_context_t *udp_context,
                           int profile_value,
                           multi_ptr_t c_base_header,
                           int length,
                           base_header_udp_t *udp)
{
	int size;

	// If profile value 0x0102
	if(profile_value == ROHCv2_PROFILE_UDP)
	{
		udp->checksum = c_base_header.udp_endpoint_dynamic->checksum;
		v2_context->msn = ntohs(c_base_header.udp_endpoint_dynamic->msn);
		v2_context->reorder_ratio = c_base_header.udp_endpoint_dynamic->reorder_ratio;
		size = sizeof(udp_endpoint_dynamic_t);
	}
	else
	{
		udp->checksum = c_base_header.udp_regular_dynamic->checksum;
		size = sizeof(udp_regular_dynamic_t);
	}
	udp_context->checksum_used = udp->checksum != 0;

	rohc_debugf(3, "UDP dynamic part length %d\n",size);
	TraceData(c_base_header.uint8,size);

	return size;
}


/**
 * @brief Decode the irregular part of UDP header of the rohc packet.
 *
 * @param udp_context    The specific UDP decompression context
 * @param c_base_header  Pointer to the irregular part of the UDP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param udp            The decoded UDP header
 * @return               The length of dynamic part of UDP header
 *                       0 if an error occurs
 */
u_int8_t * tcp_decode_irregular_udp(udp_context_t *udp_context,
                                    multi_ptr_t c_base_header,
                                    int length,
                                    base_header_udp_t *udp)
{
	if(udp_context->checksum_used != 0)
	{
		// =:= udp_with_checksum_irregular
		rohc_debugf(3, "UDP irregular part length 2\n");
		TraceData(c_base_header.uint8,2);
		udp->checksum = READ16_FROM_MPTR(c_base_header);
	}
	else
	{
		// =:= udp_zero_checksum_irregular
		udp->checksum = 0;
	}

	return c_base_header.uint8;
}


/**
 * @brief Decode the UDP profile of the rohc packet.
 *
 * @param decomp              The ROHC decompressor
 * @param v2_context          The specific RoHCv2 decompression context
 * @param ip_inner_context    The innermost IP decompression context
 * @param udp_context         The specific UDP decompression context
 * @param base_header_inner   The decoded innermost IP header
 * @param profile_value       The profile value used
 * @param c_base_header       Pointer to the dynamic part of the UDP header of the rohc packet
 * @param length              The remain length of the rohc packet
 * @param udp                 The decoded UDP header
 * @param outer_ip_flag       The outer IP flag
 * @return                    The length of dynamic part of UDP header
 *                            0 if an error occurs
 */
int tcp_decode_udp_baseheader(struct rohc_decomp *decomp,
                              struct d_v2_context *v2_context,
                              ip_context_ptr_t ip_inner_context,
                              udp_context_t *udp_context,
                              base_header_ip_t base_header_inner,
                              int profile_value,
                              multi_ptr_t c_base_header,
                              int length,
                              base_header_udp_t *udp,
                              int *outer_ip_flag)
{
	u_int8_t discriminator;
	u_int8_t header_crc;
	multi_ptr_t mptr;
	WB_t ip_id;
	int size;

	rohc_debugf(3,
	            "decomp %p v2_context %p ip_inner_context %p udp_context %p base_header_inner %p\n",
	            decomp,
	            v2_context,ip_inner_context.uint8,udp_context,
	            base_header_inner.uint8);
	rohc_debugf(3, "profile_value %Xh c_base_header %p length %d udp %p outer_ip_flag %p\n",
	            profile_value,c_base_header.uint8,length,udp,
	            outer_ip_flag);

	discriminator = *c_base_header.uint8;
	if(discriminator == 0xFA)
	{
		goto decode_udp_co_common;
	}

	// Init dynamic values with default value from context
	if(ip_inner_context.vx->version == IPV4)
	{
		base_header_inner.ipv4->df = ip_inner_context.v4->df;
		base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
		base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
	}
	else
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
		base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
		#endif
		base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
	}

	if( ( discriminator & 0x80 ) == 0)
	{
		goto decode_udp_pt_0_crc3;
	}
	switch(discriminator & 0xE0)
	{
		case 0x80:
			goto decode_udp_pt_0_crc7;
		case 0xA0:
			goto decode_udp_pt_1_seq_id;
		case 0xC0:
			goto decode_udp_pt_2_seq_id;
		default:
			return -1;
	}

decode_udp_co_common:
	{
		profile_2_3_4_flags_t profile_2_3_4_flags;

		rohc_debugf(3, "UDP profile co_common\n");
		mptr.udp_co_common = c_base_header.udp_co_common + 1;

#ifdef LQJSMLQJS  // Done before
		size = sizeof(udp_co_common_t) + 1;
		size += c_base_header.udp_co_common->tos_tc_indicator;
		size += c_base_header.udp_co_common->ttl_hopl_indicator;
		if(c_base_header.udp_co_common->flags_indicator != 0)
		{
			++size;
			if(ip_inner_context.vx->version == IPV4)
			{
				if(mptr.profile_2_3_4_flags->name.ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
				{
					size += c_base_header.udp_co_common->ip_id_indicator + 1;
				}
			}
		}
		else
		{
			if(ip_inner_context.vx->version == IPV4)
			{
				if(ip_inner_context.vx->ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
				{
					size += c_base_header.udp_co_common->ip_id_indicator + 1;
				}
			}
		}
		rohc_debugf(3, "UDP profile length %d\n",size);
		TraceData(c_base_header.uint8,size);

		header_crc = c_base_header.udp_co_common->header_crc;
		c_base_header.udp_co_common->header_crc = 0;
		c_base_header.udp_co_common->header_crc =
		   crc_calculate(CRC_TYPE_7, c_base_header.uint8, size, CRC_INIT_7,
		                 decomp->crc_table_7);
		rohc_debugf(3, "header_crc %2.2Xh <=> %2.2Xh size %d\n",header_crc,
		            c_base_header.udp_co_common->header_crc,
		            size);
		if(header_crc != c_base_header.udp_co_common->header_crc)
		{
			return ROHC_ERROR;
		}
#endif

		if(c_base_header.udp_co_common->flags_indicator != 0)
		{
			profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
			*outer_ip_flag = profile_2_3_4_flags.name.ip_outer_indicator;
		}
		else
		{
			profile_2_3_4_flags.flags = 0;
		}
		v2_context->reorder_ratio = c_base_header.udp_co_common->reorder_ratio;
		if(ip_inner_context.vx->version == IPV4)
		{
			if(c_base_header.udp_co_common->flags_indicator != 0)
			{
				ip_inner_context.v4->df = profile_2_3_4_flags.name.df;
				ip_inner_context.v4->ip_id_behavior = profile_2_3_4_flags.name.ip_id_behavior;
				rohc_debugf(3, "ip_id_behavior %d\n",ip_inner_context.v4->ip_id_behavior);
			}
			base_header_inner.ipv4->df = ip_inner_context.v4->df;
			if(c_base_header.udp_co_common->tos_tc_indicator != 0)
			{
				ip_inner_context.v4->tos_tc = *(mptr.uint8++);
			}
			base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
			if(c_base_header.udp_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v4->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
			// =:= msn_lsb(8)
			v2_context->msn = d_msn_lsb(v2_context,8,*(mptr.uint8++));
			rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
			// ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,ip_id_indicator.CVALUE)
			d_ip_id_sequential_variable(&mptr,ip_inner_context.v4->ip_id_behavior,
			                            c_base_header.udp_co_common->ip_id_indicator,
			                            ip_inner_context.v4->last_ip_id,&ip_id.uint16,
			                            v2_context->msn);
			base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
			ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
			rohc_debugf(3, "new last IP-ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16);
		}
		else
		{
			if(c_base_header.udp_co_common->tos_tc_indicator != 0)
			{
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				ip_inner_context.v6->tos_tc1 = (*mptr.uint8) >> 1;
				ip_inner_context.v6->tos_tc2 = *(mptr.uint8++);
				#elif __BYTE_ORDER == __BIG_ENDIAN
				ip_inner_context.v6->tos_tc = *(mptr.uint8++);
				#endif
			}
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
			base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
			#endif
			if(c_base_header.udp_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v6->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
			// =:= msn_lsb(8)
			v2_context->msn = d_msn_lsb(v2_context,8,*(mptr.uint8++));
			rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
		}
		size = mptr.uint8 - c_base_header.uint8;
		rohc_debugf(3, "return size %d\n",size);
		return size;
	}

decode_udp_pt_0_crc3:
	rohc_debugf(3, "UDP profile pt_0_crc3\n");
	size = sizeof(udp_pt_0_crc3_t);
	header_crc = c_base_header.udp_pt_0_crc3->header_crc;
	c_base_header.udp_pt_0_crc3->header_crc = 0;
	c_base_header.udp_pt_0_crc3->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.udp_pt_0_crc3->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_pt_0_crc3->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(4)
	v2_context->msn = d_msn_lsb(v2_context,4,c_base_header.udp_pt_0_crc3->msn);
	goto idem_decode_udp_pt_0_crc7;

decode_udp_pt_0_crc7:
	rohc_debugf(3, "UDP profile pt_0_crc7\n");
	size = sizeof(udp_pt_0_crc7_t);
	header_crc = c_base_header.udp_pt_0_crc7->header_crc;
	c_base_header.udp_pt_0_crc7->header_crc = 0;
	c_base_header.udp_pt_0_crc7->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.udp_pt_0_crc7->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_pt_0_crc7->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	v2_context->msn = d_msn_lsb(
	   v2_context,6,(c_base_header.udp_pt_0_crc7->msn1 << 1) | c_base_header.udp_pt_0_crc7->msn2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.udp_pt_0_crc7->msn);
	#endif
idem_decode_udp_pt_0_crc7:
	if(ip_inner_context.vx->version == IPV4)
	{
		switch(ip_inner_context.v4->ip_id_behavior)
		{
			case IP_ID_BEHAVIOR_SEQUENTIAL:
				++ip_inner_context.v4->last_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(ip_inner_context.v4->last_ip_id.uint16);
				break;
			case IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED:
			{
				WB_t swapped_ip_id;
				swapped_ip_id.uint8[0] = ip_inner_context.v4->last_ip_id.uint8[1];
				swapped_ip_id.uint8[1] = ip_inner_context.v4->last_ip_id.uint8[0];
				++swapped_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(swapped_ip_id.uint16);
				ip_inner_context.v4->last_ip_id.uint8[1] = swapped_ip_id.uint8[0];
				ip_inner_context.v4->last_ip_id.uint8[0] = swapped_ip_id.uint8[1];
			}
			break;
			default:
				break;
		}
	}
	goto end_decode_udp_base_header;

decode_udp_pt_1_seq_id:
	rohc_debugf(3, "UDP profile pt_1_seq_id\n");
	size = sizeof(udp_pt_1_seq_id_t);
	header_crc = c_base_header.udp_pt_1_seq_id->header_crc;
	c_base_header.udp_pt_1_seq_id->header_crc = 0;
	c_base_header.udp_pt_1_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.udp_pt_1_seq_id->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_pt_1_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	rohc_debugf(3, "old msn %Xh msn recv %Xh\n",v2_context->msn,
	            (c_base_header.udp_pt_1_seq_id->msn1 << 4) | c_base_header.udp_pt_1_seq_id->msn2);
	v2_context->msn = d_msn_lsb(
	   v2_context,6,(c_base_header.udp_pt_1_seq_id->msn1 << 4) | c_base_header.udp_pt_1_seq_id->msn2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.udp_pt_1_seq_id->msn);
	#endif
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	if(ip_inner_context.vx->version == IPV4)
	{
		// =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4)
//		ip_id.uint16 = ( ip_inner_context.v4->last_ip_id.uint16 & 0xFFF0 ) | d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,c_base_header.udp_pt_1_seq_id->ip_id,v2_context->msn);
		ip_id.uint16 =
		   d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,
		               c_base_header.udp_pt_1_seq_id->ip_id,
		               v2_context->msn);
		goto idem_decode_udp_pt_2_seq_id;
	}
	else
	{
		rohc_debugf(3, "ip_inner_version %d\n",ip_inner_context.vx->version);
	}
	goto end_decode_udp_base_header;

decode_udp_pt_2_seq_id:
	rohc_debugf(3, "UDP profile pt_2_seq_id\n");
	size = sizeof(udp_pt_2_seq_id_t);
	header_crc = c_base_header.udp_pt_2_seq_id->header_crc;
	c_base_header.udp_pt_2_seq_id->header_crc = 0;
	c_base_header.udp_pt_2_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.udp_pt_2_seq_id->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_pt_2_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(8)
	v2_context->msn = d_msn_lsb(v2_context,8,c_base_header.udp_pt_2_seq_id->msn);
idem_decode_udp_pt_2_seq_id:
	rohc_debugf(3, "last_ip_id %4.4Xh new IP_ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16,
	            ip_id.uint16);
	base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
	ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;

end_decode_udp_base_header:
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	return size;
}


/**
 * @brief Decode the static part of ESP header of the rohc packet.
 *
 * @param esp_context    The specific ESP decompression context
 * @param esp_static     The compressed ESP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param esp            The decoded ESP header
 * @return               The length of static part of ESP header
 *                       0 if an error occurs
 */
int tcp_decode_static_esp(esp_context_t *esp_context,
                          esp_static_t *esp_static,
                          int length,
                          base_header_esp_t *esp)
{

	// Save information to context
	esp_context->protocol = IPPROTO_ESP;
	esp_context->spi = esp_static->spi;

	esp->spi = esp_static->spi;

	rohc_debugf(3, "ESP static part length %d\n",(int)sizeof(esp_static_t));
	TraceData((unsigned char*)esp_static,sizeof(esp_static_t));

	return sizeof(esp_static_t);
}


/**
 * @brief Copy the static ESP part to the ESP header
 *
 * @param esp_context    The specific ESP decompression context
 * @param esp            The ESP header
 * @return               The size of the static part
 */
unsigned int tcp_copy_static_esp(esp_context_t *esp_context,
                                 base_header_esp_t *esp)
{

	rohc_debugf(3, "esp_context %p udp %p\n",esp_context,esp);

	esp->spi = esp_context->spi;

	return sizeof(base_header_esp_t);
}


/**
 * @brief Decode the dynamic part of ESP header of the rohc packet.
 *
 * @param v2_context     The specific RoHCv2 decompression context
 * @param esp_context    The specific ESP decompression context
 * @param esp_dynamic    Pointer to the dynamic part of the ESP header of the rohc packet
 * @param length         The remain length of the rohc packet
 * @param esp            The decoded ESP header
 * @return               The length of dynamic part of ESP header
 *                       0 if an error occurs
 */
int tcp_decode_dynamic_esp(struct d_v2_context *v2_context,
                           esp_context_t *esp_context,
                           esp_dynamic_t *esp_dynamic,
                           int length,
                           base_header_esp_t *esp)
{
	esp->sequence_number = esp_dynamic->sequence_number;
	v2_context->reorder_ratio = esp_dynamic->reorder_ratio;

	esp_context->sequence_number = ntohl(esp->sequence_number);
	v2_context->msn = esp_context->sequence_number;

	rohc_debugf(3, "ESP dynamic part length %d\n",(int)sizeof(esp_dynamic_t));
	TraceData((unsigned char*)esp_dynamic,(int)sizeof(esp_dynamic_t));

	return sizeof(esp_dynamic_t);
}


/**
 * @brief Decode the ESP profile of the rohc packet.
 *
 * @param decomp             The ROHC decompressor
 * @param v2_context         The specific RoHCv2 decompression context
 * @param ip_inner_context   The specific IP inner decompression context
 * @param esp_context        The specific ESP decompression context
 * @param base_header_inner  The decoded innermost IP header
 * @param profile_value      The profile value used
 * @param c_base_header      Pointer to the dynamic part of the ESP header of the rohc packet
 * @param length             The remain length of the rohc packet
 * @param esp                The decoded ESP header
 * @param outer_ip_flag      Adress of the outer IP flag
 * @return                   The length of dynamic part of ESP header
 *                           0 if an error occurs
 */
int tcp_decode_esp_baseheader(struct rohc_decomp *decomp,
                              struct d_v2_context *v2_context,
                              ip_context_ptr_t ip_inner_context,
                              esp_context_t *esp_context,
                              base_header_ip_t base_header_inner,
                              int profile_value,
                              multi_ptr_t c_base_header,
                              int length,
                              base_header_esp_t *esp,
                              int *outer_ip_flag)
{
	u_int8_t discriminator;
	u_int8_t header_crc;
	multi_ptr_t mptr;
	WB_t ip_id;
	int size;

	rohc_debugf(3,
	            "decomp %p v2_context %p ip_inner_context %p esp_context %p base_header_inner %p\n",
	            decomp,
	            v2_context,ip_inner_context.uint8,esp_context,
	            base_header_inner.uint8);
	rohc_debugf(3, "profile_value %Xh c_base_header %p length %d esp %p outer_ip_flag %p\n",
	            profile_value,c_base_header.uint8,length,esp,
	            outer_ip_flag);

	discriminator = *c_base_header.uint8;
	if(discriminator == 0xFA)
	{
		goto decode_esp_co_common;
	}

	// Init dynamic values with default value from context
	if(ip_inner_context.vx->version == IPV4)
	{
		base_header_inner.ipv4->df = ip_inner_context.v4->df;
		base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
		base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
	}
	else
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
		base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
		#endif
		base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
	}

	if( ( discriminator & 0x80 ) == 0)
	{
		goto decode_esp_pt_0_crc3;
	}
	switch(discriminator & 0xE0)
	{
		case 0x80:
			goto decode_esp_pt_0_crc7;
		case 0xA0:
			goto decode_esp_pt_1_seq_id;
		case 0xC0:
			goto decode_esp_pt_2_seq_id;
		default:
			return -1;
	}

decode_esp_co_common:
	{
		profile_2_3_4_flags_t profile_2_3_4_flags;
//		int i;

		rohc_debugf(3, "ESP profile co_common\n");

#ifdef LKQLSJK // Done before
		size = sizeof(esp_co_common_t);
		mptr.uint8 = c_base_header.uint8 + sizeof(esp_co_common_t);

		rohc_debugf(3, "size = %d\n",size);
		if(c_base_header.esp_co_common->flags_indicator != 0)
		{
			profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
			size += sizeof(profile_2_3_4_flags_t);
			rohc_debugf(3, "size = %d\n",size);
		}
		else
		{
			profile_2_3_4_flags.flags = 0;
		}
		i = c_base_header.esp_co_common->ttl_hopl_indicator +
		    c_base_header.esp_co_common->tos_tc_indicator;
		size += i;
		rohc_debugf(3, "size = %d\n",size);
		mptr.uint8 += i;
		rohc_debugf(3, "Compressed header at %p size sup %d\n",c_base_header.uint8,size);
		i = d_sdvl_size(*mptr.uint8);
		size += i;
		rohc_debugf(3, "size = %d\n",size);
		rohc_debugf(3, "size sup %d\n",size);
		if(ip_inner_context.vx->version == IPV4)
		{
			rohc_debugf(3, "ip_inner_context ip_id_behavior %d\n",ip_inner_context.v4->ip_id_behavior);
			if(profile_2_3_4_flags.name.ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
			{
				size += c_base_header.esp_co_common->ip_id_indicator + 1;
				rohc_debugf(3, "size sup %d\n",size);
			}
		}
		else
		{
			rohc_debugf(3, "ip_inner_context != IPV4 (%d)\n",ip_inner_context.vx->version);
		}
		rohc_debugf(3, "ESP profile length %d\n",size);
		TraceData(c_base_header.uint8,size);

		header_crc = c_base_header.esp_co_common->header_crc;
		c_base_header.esp_co_common->header_crc = 0;
		c_base_header.esp_co_common->header_crc =
		   crc_calculate(CRC_TYPE_7, c_base_header.uint8, size, CRC_INIT_7,
		                 decomp->crc_table_7);
		rohc_debugf(3, "header_crc %2.2Xh <=> %2.2Xh size %d\n",header_crc,
		            c_base_header.esp_co_common->header_crc,
		            size);
		if(header_crc != c_base_header.esp_co_common->header_crc)
		{
			return ROHC_ERROR;
		}
#endif
		mptr.esp_co_common = c_base_header.esp_co_common + 1;

		v2_context->reorder_ratio = c_base_header.esp_co_common->reorder_ratio;

		if(c_base_header.esp_co_common->flags_indicator != 0)
		{
			profile_2_3_4_flags = *(mptr.profile_2_3_4_flags++);
			*outer_ip_flag = profile_2_3_4_flags.name.ip_outer_indicator;
		}
		else
		{
			profile_2_3_4_flags.flags = 0;
		}

		if(ip_inner_context.vx->version == IPV4)
		{
			if(c_base_header.esp_co_common->flags_indicator != 0)
			{
				ip_inner_context.v4->df = profile_2_3_4_flags.name.df;
				ip_inner_context.v4->ip_id_behavior = profile_2_3_4_flags.name.ip_id_behavior;
				rohc_debugf(3, "ip_id_behavior %d\n",ip_inner_context.v4->ip_id_behavior);
			}
			base_header_inner.ipv4->df = ip_inner_context.v4->df;
			if(c_base_header.esp_co_common->tos_tc_indicator != 0)
			{
				ip_inner_context.v4->tos_tc = *(mptr.uint8++);
			}
			base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
			if(c_base_header.esp_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v4->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
			// =:= sdvl_sn_lsb(sequence_number.ULENGTH)
			esp_context->sequence_number = d_sdvl_sn_lsb(&mptr);
			rohc_debugf(3, "sequence_number %Xh\n",esp_context->sequence_number);
			v2_context->msn = esp_context->sequence_number;
			// ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,ip_id_indicator.CVALUE)
			d_ip_id_sequential_variable(&mptr,ip_inner_context.v4->ip_id_behavior,
			                            c_base_header.esp_co_common->ip_id_indicator,
			                            ip_inner_context.v4->last_ip_id,&ip_id.uint16,
			                            v2_context->msn);
			base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
			ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
			rohc_debugf(3, "new last IP-ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16);
		}
		else
		{
			if(c_base_header.esp_co_common->tos_tc_indicator != 0)
			{
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				ip_inner_context.v6->tos_tc1 = (*mptr.uint8) >> 1;
				ip_inner_context.v6->tos_tc2 = *(mptr.uint8++);
				#elif __BYTE_ORDER == __BIG_ENDIAN
				ip_inner_context.v6->tos_tc = *(mptr.uint8++);
				#endif
			}
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
			base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
			#endif
			if(c_base_header.esp_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v6->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
			// =:= sdvl_sn_lsb(sequence_number.ULENGTH)
			esp_context->sequence_number = d_sdvl_sn_lsb(&mptr);
			rohc_debugf(3, "sequence_number %Xh\n",esp_context->sequence_number);
			v2_context->msn = esp_context->sequence_number;
		}
		size = mptr.uint8 - c_base_header.uint8;
		goto end_decode_esp_baseheader2;
	}

decode_esp_pt_0_crc3:
	rohc_debugf(3, "ESP profile pt_0_crc3\n");
	size = sizeof(esp_pt_0_crc3_t);
	header_crc = c_base_header.esp_pt_0_crc3->header_crc;
	c_base_header.esp_pt_0_crc3->header_crc = 0;
	c_base_header.esp_pt_0_crc3->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.esp_pt_0_crc3->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.esp_pt_0_crc3->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(4)
	v2_context->msn = d_msn_lsb(v2_context,4,c_base_header.esp_pt_0_crc3->sequence_number);
	goto idem_decode_esp_pt_0_crc7;

decode_esp_pt_0_crc7:
	rohc_debugf(3, "ESP profile pt_0_crc7\n");
	size = sizeof(esp_pt_0_crc7_t);
	header_crc = c_base_header.esp_pt_0_crc7->header_crc;
	c_base_header.esp_pt_0_crc7->header_crc = 0;
	c_base_header.esp_pt_0_crc7->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.esp_pt_0_crc7->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.esp_pt_0_crc7->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	v2_context->msn = d_msn_lsb(
	   v2_context,6,
	   (c_base_header.esp_pt_0_crc7->sequence_number1 <<
	    1) | c_base_header.esp_pt_0_crc7->sequence_number2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.esp_pt_0_crc7->sequence_number);
	#endif
idem_decode_esp_pt_0_crc7:
	if(ip_inner_context.vx->version == IPV4)
	{
		switch(ip_inner_context.v4->ip_id_behavior)
		{
			case IP_ID_BEHAVIOR_SEQUENTIAL:
				++ip_inner_context.v4->last_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(ip_inner_context.v4->last_ip_id.uint16);
				break;
			case IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED:
			{
				WB_t swapped_ip_id;
				swapped_ip_id.uint8[0] = ip_inner_context.v4->last_ip_id.uint8[1];
				swapped_ip_id.uint8[1] = ip_inner_context.v4->last_ip_id.uint8[0];
				++swapped_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(swapped_ip_id.uint16);
				ip_inner_context.v4->last_ip_id.uint8[1] = swapped_ip_id.uint8[0];
				ip_inner_context.v4->last_ip_id.uint8[0] = swapped_ip_id.uint8[1];
			}
			break;
			default:
				break;
		}
	}
	goto end_decode_esp_baseheader;

decode_esp_pt_1_seq_id:
	rohc_debugf(3, "ESP profile pt_1_seq_id\n");
	size = sizeof(esp_pt_1_seq_id_t);
	header_crc = c_base_header.esp_pt_1_seq_id->header_crc;
	c_base_header.esp_pt_1_seq_id->header_crc = 0;
	c_base_header.esp_pt_1_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.esp_pt_1_seq_id->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.esp_pt_1_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	rohc_debugf(
	   3, "old msn %Xh msn recv %Xh\n",v2_context->msn,
	   (c_base_header.esp_pt_1_seq_id->sequence_number1 <<
	    4) | c_base_header.esp_pt_1_seq_id->sequence_number2);
	v2_context->msn = d_msn_lsb(
	   v2_context,6,
	   (c_base_header.esp_pt_1_seq_id->sequence_number1 <<
	    4) | c_base_header.esp_pt_1_seq_id->sequence_number2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.esp_pt_1_seq_id->sequence_number);
	#endif
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	if(ip_inner_context.vx->version == IPV4)
	{
		// =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4)
//	ip_id.uint16 = ( ip_inner_context.v4->last_ip_id.uint16 & 0xFFF0 ) | d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,c_base_header.esp_pt_1_seq_id->ip_id,v2_context->msn);
		ip_id.uint16 =
		   d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,
		               c_base_header.esp_pt_1_seq_id->ip_id,
		               v2_context->msn);
		rohc_debugf(3, "last_ip_id %4.4Xh new IP_ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16,
		            ip_id.uint16);
		base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
		ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
	}
	else
	{
		rohc_debugf(3, "ip_inner_version %d\n",ip_inner_context.vx->version);
	}
	goto end_decode_esp_baseheader;

decode_esp_pt_2_seq_id:
	rohc_debugf(3, "ESP profile pt_2_seq_id\n");
	size = sizeof(esp_pt_2_seq_id_t);
	header_crc = c_base_header.esp_pt_2_seq_id->header_crc;
	c_base_header.esp_pt_2_seq_id->header_crc = 0;
	c_base_header.esp_pt_2_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.esp_pt_2_seq_id->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.esp_pt_2_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(8)
	v2_context->msn = d_msn_lsb(v2_context,8,c_base_header.esp_pt_2_seq_id->sequence_number);
	goto end_decode_esp_baseheader;

end_decode_esp_baseheader:

	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	esp_context->sequence_number = ( esp_context->sequence_number & 0xFFFF0000 ) | v2_context->msn;
	v2_context->msn = esp_context->sequence_number;
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);

end_decode_esp_baseheader2:

	esp->sequence_number = htonl(esp_context->sequence_number);

	rohc_debugf(3, "ESP baseheader length %d\n",size);
	TraceData(c_base_header.uint8,size);

	return size;
}


/**
 * @brief Decode the static part of UDP-Lite header of the rohc packet.
 *
 * @param udp_lite_context  The specific UDP-Lite decompression context
 * @param udp_lite_static   The compressed UDP-Lite header of the rohc packet
 * @param length            The remain length of the rohc packet
 * @param udp_lite          The decoded UDP-Lite header
 * @return                  The length of static part of UDP-Lite header
 *                          0 if an error occurs
 */
int tcp_decode_static_udp_lite(udp_lite_context_t *udp_lite_context,
                               udp_lite_static_t *udp_lite_static,
                               int length,
                               base_header_udp_lite_t *udp_lite)
{
	// Save information to context
	udp_lite_context->protocol = IPPROTO_UDPLITE;
	udp_lite_context->src_port = udp_lite_static->src_port;
	udp_lite_context->dst_port = udp_lite_static->dst_port;

	udp_lite->src_port = udp_lite_static->src_port;
	udp_lite->dst_port = udp_lite_static->dst_port;

	rohc_debugf(3, "UDP-Lite static part length %d\n",(int)sizeof(udp_lite_static_t));
	TraceData((unsigned char*)udp_lite_static,sizeof(udp_lite_static_t));
	fflush(stdout);
	return sizeof(udp_lite_static_t);
}


/**
 * @brief Copy the static UDP-Lite part to the UDP-Lite header
 *
 * @param udp_lite_context   The specific UDP-Lite decompression context
 * @param udp_lite           The UDP-Lite header
 * @return                   The size of the static part
 */
unsigned int tcp_copy_static_udp_lite(udp_lite_context_t *udp_lite_context,
                                      base_header_udp_lite_t *udp_lite)
{
	rohc_debugf(3, "udp_lite_context %p udp %p\n",udp_lite_context,udp_lite);

	udp_lite->src_port = udp_lite_context->src_port;
	udp_lite->dst_port = udp_lite_context->dst_port;

	return sizeof(base_header_udp_lite_t);
}


/**
 * @brief Decode the dynamic part of UDP-Lite header of the rohc packet.
 *
 * @param v2_context         The specific RoHCv2 decompression context
 * @param udp_lite_context   The specific UDP-Lite decompression context
 * @param profile_value      The profile value used
 * @param c_base_header      Pointer to the dynamic part of the UDP-Lite header of the rohc packet
 * @param length             The remain length of the rohc packet
 * @param udp_lite           The decoded UDP-Lite header
 * @return                   The length of dynamic part of UDP-Lite header
 *                           0 if an error occurs
 */
int tcp_decode_dynamic_udp_lite(struct d_v2_context *v2_context,
                                udp_lite_context_t *udp_lite_context,
                                int profile_value,
                                multi_ptr_t c_base_header,
                                int length,
                                base_header_udp_lite_t *udp_lite)
{
	int size;

	// If profile value 0x0108
	if(profile_value == ROHCv2_PROFILE_UDPLITE)
	{
		// =:=udp_lite_endpoint_dynamic
		v2_context->reorder_ratio = c_base_header.udp_lite_endpoint_dynamic1->reorder_ratio;
		if( ( udp_lite_context->coverage_behavior =
		         c_base_header.udp_lite_endpoint_dynamic1->coverage_behavior ) ==
		    UDP_LITE_COVERAGE_INFERRED)
		{
			// =:= checksum_coverage_dynchain(coverage_behavior)
			// udp_lite_context->checksum_coverage = ??? no change
			udp_lite->checksum = c_base_header.udp_lite_endpoint_dynamic1->checksum;
			v2_context->msn = ntohs(c_base_header.udp_lite_endpoint_dynamic1->msn);
			size = sizeof(udp_lite_endpoint_dynamic1_t);
		}
		else
		{
			// =:= checksum_coverage_dynchain(coverage_behavior)
			udp_lite_context->checksum_coverage =
			   c_base_header.udp_lite_endpoint_dynamic2->checksum_coverage;
			udp_lite->checksum = c_base_header.udp_lite_endpoint_dynamic2->checksum;
			v2_context->msn = ntohs(c_base_header.udp_lite_endpoint_dynamic2->msn);
			size = sizeof(udp_lite_endpoint_dynamic2_t);
		}
	}
	else
	{
		assert( profile_value == ROHCv2_PROFILE_UDPLITE_RTP );
		// =:= udp_lite_regular_dynamic
		if( ( udp_lite_context->coverage_behavior =
		         c_base_header.udp_lite_regular_dynamic1->coverage_behavior ) ==
		    UDP_LITE_COVERAGE_INFERRED)
		{
			udp_lite->checksum = c_base_header.udp_lite_regular_dynamic1->checksum;
			size = sizeof(udp_lite_regular_dynamic1_t);
		}
		else
		{
			udp_lite->checksum = c_base_header.udp_lite_regular_dynamic2->checksum;
			udp_lite_context->checksum_coverage =
			   c_base_header.udp_lite_regular_dynamic2->checksum_coverage;
			size = sizeof(udp_lite_regular_dynamic2_t);
		}
	}
	udp_lite->checksum_coverage = udp_lite_context->checksum_coverage;

	rohc_debugf(3, "UDP-Lite dynamic part length %d\n",size);
	TraceData(c_base_header.uint8,size);
	fflush(stdout);
	return size;
}


/**
 * @brief Decode the irregular part of UDP-Lite header of the rohc packet.
 *
 * @param udp_lite_context   The specific UDP-Lite decompression context
 * @param c_base_header      Pointer to the irregular part of the UDP header of the rohc packet
 * @param length             The remain length of the rohc packet
 * @param udp_lite           The decoded UDP-Lite header
 * @return                   The length of dynamic part of UDP header
 *                           0 if an error occurs
 */
u_int8_t * tcp_decode_irregular_udp_lite(udp_lite_context_t *udp_lite_context,
                                         multi_ptr_t c_base_header,
                                         int length,
                                         base_header_udp_lite_t *udp_lite)
{
	int size;

	if(udp_lite_context->coverage_behavior  == UDP_LITE_COVERAGE_IRREGULAR)
	{
		// =:= checksum_coverage_irregular(coverage_behavior.UVALUE) [ 0, 16 ];
		udp_lite->checksum_coverage = c_base_header.udp_lite_irregular2->checksum_coverage;
		// =:= irregular(16)
		udp_lite->checksum = c_base_header.udp_lite_irregular2->checksum;
		size = sizeof(udp_lite_irregular2_t);
	}
	else
	{
		// =:= irregular(16)
		udp_lite->checksum = c_base_header.udp_lite_irregular1->checksum;
		size = sizeof(udp_lite_irregular1_t);
	}

	rohc_debugf(3, "UDP-Lite irregular part length %d\n",size);
	TraceData(c_base_header.uint8,size);

	fflush(stdout);

	return c_base_header.uint8 + size;
}


/**
 * @brief Decode the UDP-Lite profile of the rohc packet.
 *
 * @param decomp             The ROHC decompressor
 * @param v2_context         The specific RoHCv2 decompression context
 * @param ip_inner_context   The specific IP inner decompression context
 * @param udp_lite_context   The specific UDP-Lite decompression context
 * @param base_header_inner  The decoded innermost IP header
 * @param profile_value      The profile value used
 * @param c_base_header      Pointer to the dynamic part of the UDP-Lite header of the rohc packet
 * @param length             The remain length of the rohc packet
 * @param udp_lite           The decoded UDP-Lite header
 * @param outer_ip_flag      Adresse of the outer IP flag
 * @return                   The length of dynamic part of UDP-Lite header
 *                           0 if an error occurs
 */
int tcp_decode_udp_lite_baseheader(struct rohc_decomp *decomp,
                                   struct d_v2_context *v2_context,
                                   ip_context_ptr_t ip_inner_context,
                                   udp_lite_context_t *udp_lite_context,
                                   base_header_ip_t base_header_inner,
                                   int profile_value,
                                   multi_ptr_t c_base_header,
                                   int length,
                                   base_header_udp_lite_t *udp_lite,
                                   int *outer_ip_flag)
{
	u_int8_t discriminator;
	u_int8_t header_crc;
	multi_ptr_t mptr;
	WB_t ip_id;
	int size;

	rohc_debugf(
	   3, "decomp %p v2_context %p ip_inner_context %p udp_lite_context %p base_header_inner %p\n",
	   decomp,v2_context,ip_inner_context.uint8,udp_lite_context,base_header_inner.uint8);
	rohc_debugf(3, "profile_value %Xh c_base_header %p length %d udp_lite %p outer_ip_flag %p\n",
	            profile_value,c_base_header.uint8,length,udp_lite,
	            outer_ip_flag);

	discriminator = *c_base_header.uint8;
	if(discriminator == 0xFA)
	{
		goto decode_udp_lite_co_common;
	}

	// Init dynamic values with default value from context
	if(ip_inner_context.vx->version == IPV4)
	{
		base_header_inner.ipv4->df = ip_inner_context.v4->df;
		base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
		base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
	}
	else
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
		base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
		#endif
		base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
	}

	if( ( discriminator & 0x80 ) == 0)
	{
		goto decode_udp_lite_pt_0_crc3;
	}
	switch(discriminator & 0xE0)
	{
		case 0x80:
			goto decode_udp_lite_pt_0_crc7;
		case 0xA0:
			goto decode_udp_lite_pt_1_seq_id;
		case 0xC0:
			goto decode_udp_lite_pt_2_seq_id;
		default:
			return -1;
	}

decode_udp_lite_co_common:
	{
		profile_8_flags_t profile_8_flags;

		rohc_debugf(3, "UDP-Lite profile co_common\n");
		mptr.udp_lite_co_common = c_base_header.udp_lite_co_common + 1;
		rohc_debugf(3, "co_common indicators (%2.2Xh): ip_id %d flags %d ttl_hopl %d tos_tc %d\n",
		            *(c_base_header.uint8 + 2),c_base_header.udp_lite_co_common->ip_id_indicator,
		            c_base_header.udp_lite_co_common->flags_indicator,
		            c_base_header.udp_lite_co_common->ttl_hopl_indicator,
		            c_base_header.udp_lite_co_common->tos_tc_indicator);
#ifdef MLJQMLJSLMQ  // Done before
		size = sizeof(udp_lite_co_common_t) + 1;
		size += c_base_header.udp_lite_co_common->tos_tc_indicator;
		size += c_base_header.udp_lite_co_common->ttl_hopl_indicator;
		if(c_base_header.udp_lite_co_common->flags_indicator != 0)
		{
			++size;
			if(ip_inner_context.vx->version == IPV4)
			{
				if(mptr.profile_8_flags->ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
				{
					size += c_base_header.udp_lite_co_common->ip_id_indicator = 1;
				}
			}
		}
		else
		{
			if(ip_inner_context.vx->version == IPV4)
			{
				if(ip_inner_context.vx->ip_id_behavior <= IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED)
				{
					size += c_base_header.udp_lite_co_common->ip_id_indicator + 1;
				}
			}
		}
		rohc_debugf(3, "UDP-Lite profile length %d\n",size);
		TraceData(c_base_header.uint8,size);

		header_crc = c_base_header.udp_lite_co_common->header_crc;
		c_base_header.udp_lite_co_common->header_crc = 0;
		c_base_header.udp_lite_co_common->header_crc =
		   crc_calculate(CRC_TYPE_7, c_base_header.uint8, size, CRC_INIT_7,
		                 decomp->crc_table_7);
		rohc_debugf(3, "header_crc %2.2Xh <=> %2.2Xh size %d\n",header_crc,
		            c_base_header.udp_lite_co_common->header_crc,
		            size);
		if(header_crc != c_base_header.udp_lite_co_common->header_crc)
		{
			return ROHC_ERROR;
		}
#endif
		if(c_base_header.udp_lite_co_common->flags_indicator != 0)
		{
			profile_8_flags = *(mptr.profile_8_flags++);
			*outer_ip_flag = profile_8_flags.ip_outer_indicator;
			udp_lite_context->coverage_behavior = profile_8_flags.coverage_behavior;
		}
		v2_context->reorder_ratio = c_base_header.udp_lite_co_common->reorder_ratio;
		if(ip_inner_context.vx->version == IPV4)
		{
			if(c_base_header.udp_lite_co_common->flags_indicator != 0)
			{
				ip_inner_context.v4->df = profile_8_flags.df;
				ip_inner_context.v4->ip_id_behavior = profile_8_flags.ip_id_behavior;
				rohc_debugf(3, "ip_id_behavior %d\n",ip_inner_context.v4->ip_id_behavior);
			}
			base_header_inner.ipv4->df = ip_inner_context.v4->df;
			if(c_base_header.udp_lite_co_common->tos_tc_indicator != 0)
			{
				ip_inner_context.v4->tos_tc = *(mptr.uint8++);
			}
			base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
			if(c_base_header.udp_lite_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v4->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
			// =:= msn_lsb(8)
			v2_context->msn = d_msn_lsb(v2_context,8,*(mptr.uint8++));
			rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
			// ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE,ip_id_indicator.CVALUE)
			d_ip_id_sequential_variable(&mptr,ip_inner_context.v4->ip_id_behavior,
			                            c_base_header.udp_lite_co_common->ip_id_indicator,
			                            ip_inner_context.v4->last_ip_id,
			                            &ip_id.uint16,
			                            v2_context->msn);
			base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
			ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
			rohc_debugf(3, "new last IP-ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16);
		}
		else
		{
			if(c_base_header.udp_lite_co_common->tos_tc_indicator != 0)
			{
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				ip_inner_context.v6->tos_tc1 = (*mptr.uint8) >> 1;
				ip_inner_context.v6->tos_tc2 = *(mptr.uint8++);
				#elif __BYTE_ORDER == __BIG_ENDIAN
				ip_inner_context.v6->tos_tc = *(mptr.uint8++);
				#endif
			}
			#if __BYTE_ORDER == __LITTLE_ENDIAN
			base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
			base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
			#elif __BYTE_ORDER == __BIG_ENDIAN
			base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
			#endif
			if(c_base_header.udp_lite_co_common->ttl_hopl_indicator != 0)
			{
				ip_inner_context.v6->ttl_hopl = *(mptr.uint8++);
			}
			base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
			// =:= msn_lsb(8)
			v2_context->msn = d_msn_lsb(v2_context,8,*(mptr.uint8++));
			rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
		}
		size = mptr.uint8 - c_base_header.uint8;
		rohc_debugf(3, "return size %d\n",size);
		return size;
	}

decode_udp_lite_pt_0_crc3:
	rohc_debugf(3, "UDP-Lite profile pt_0_crc3\n");
	size = sizeof(udp_lite_pt_0_crc3_t);
	header_crc = c_base_header.udp_lite_pt_0_crc3->header_crc;
	c_base_header.udp_lite_pt_0_crc3->header_crc = 0;
	c_base_header.udp_lite_pt_0_crc3->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.udp_lite_pt_0_crc3->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_lite_pt_0_crc3->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(4)
	v2_context->msn = d_msn_lsb(v2_context,4,c_base_header.udp_lite_pt_0_crc3->msn);
	goto idem_decode_udp_pt_0_crc7;

decode_udp_lite_pt_0_crc7:
	rohc_debugf(3, "UDP-Lite profile pt_0_crc7\n");
	size = sizeof(udp_lite_pt_0_crc7_t);
	header_crc = c_base_header.udp_lite_pt_0_crc7->header_crc;
	c_base_header.udp_lite_pt_0_crc7->header_crc = 0;
	c_base_header.udp_lite_pt_0_crc7->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.udp_lite_pt_0_crc7->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_lite_pt_0_crc7->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	v2_context->msn = d_msn_lsb(
	   v2_context,6,
	   (c_base_header.udp_lite_pt_0_crc7->msn1 << 1) | c_base_header.udp_lite_pt_0_crc7->msn2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.udp_lite_pt_0_crc7->msn);
	#endif
idem_decode_udp_pt_0_crc7:
	if(ip_inner_context.vx->version == IPV4)
	{
		switch(ip_inner_context.v4->ip_id_behavior)
		{
			case IP_ID_BEHAVIOR_SEQUENTIAL:
				++ip_inner_context.v4->last_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(ip_inner_context.v4->last_ip_id.uint16);
				break;
			case IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED:
			{
				WB_t swapped_ip_id;
				swapped_ip_id.uint8[0] = ip_inner_context.v4->last_ip_id.uint8[1];
				swapped_ip_id.uint8[1] = ip_inner_context.v4->last_ip_id.uint8[0];
				++swapped_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(swapped_ip_id.uint16);
				ip_inner_context.v4->last_ip_id.uint8[1] = swapped_ip_id.uint8[0];
				ip_inner_context.v4->last_ip_id.uint8[0] = swapped_ip_id.uint8[1];
			}
			break;
			default:
				break;
		}
	}
	goto end_decode_udp_lite_base_header;

decode_udp_lite_pt_1_seq_id:
	rohc_debugf(3, "UDP-Lite profile pt_1_seq_id\n");
	size = sizeof(udp_lite_pt_1_seq_id_t);
	header_crc = c_base_header.udp_lite_pt_1_seq_id->header_crc;
	c_base_header.udp_lite_pt_1_seq_id->header_crc = 0;
	c_base_header.udp_lite_pt_1_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.udp_lite_pt_1_seq_id->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_lite_pt_1_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(6)
	#if __BYTE_ORDER == __LITTLE_ENDIAN
	rohc_debugf(
	   3, "old msn %Xh msn recv %Xh\n",v2_context->msn,
	   (c_base_header.udp_lite_pt_1_seq_id->msn1 <<
	    4) | c_base_header.udp_lite_pt_1_seq_id->msn2);
	v2_context->msn = d_msn_lsb(
	   v2_context,6,
	   (c_base_header.udp_lite_pt_1_seq_id->msn1 << 4) | c_base_header.udp_lite_pt_1_seq_id->msn2);
	#elif __BYTE_ORDER == __BIG_ENDIAN
	v2_context->msn = d_msn_lsb(v2_context,6,c_base_header.udp_lite_pt_1_seq_id->msn);
	#endif
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	if(ip_inner_context.vx->version == IPV4)
	{
		// =:= ip_id_lsb(ip_id_behavior_inner_most.UVALUE, 4)
//		ip_id.uint16 = ( ip_inner_context.v4->last_ip_id.uint16 & 0xFFF0 ) | d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,c_base_header.udp_lite_pt_1_seq_id->ip_id,v2_context->msn);
		ip_id.uint16 =
		   d_ip_id_lsb(ip_inner_context.v4->ip_id_behavior,4,4,ip_inner_context.v4->last_ip_id,
		               c_base_header.udp_lite_pt_1_seq_id->ip_id,
		               v2_context->msn);
		rohc_debugf(3, "last_ip_id %4.4Xh new IP_ID %4.4Xh\n",ip_inner_context.v4->last_ip_id.uint16,
		            ip_id.uint16);
		base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
		ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
	}
	else
	{
		rohc_debugf(3, "ip_inner_version %d\n",ip_inner_context.vx->version);
	}
	goto end_decode_udp_lite_base_header;

decode_udp_lite_pt_2_seq_id:
	rohc_debugf(3, "UDP-Lite profile pt_2_seq_id\n");
	size = sizeof(udp_lite_pt_2_seq_id_t);
	header_crc = c_base_header.udp_lite_pt_2_seq_id->header_crc;
	c_base_header.udp_lite_pt_2_seq_id->header_crc = 0;
	c_base_header.udp_lite_pt_2_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.udp_lite_pt_2_seq_id->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.udp_lite_pt_2_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	// =:= msn_lsb(8)
	v2_context->msn = d_msn_lsb(v2_context,8,c_base_header.udp_lite_pt_2_seq_id->msn);

end_decode_udp_lite_base_header:
	rohc_debugf(3, "new msn %Xh\n",v2_context->msn);
	return size;
}


/**
 * @brief Decode the static part of RTP header of the rohc packet.
 *
 * @param rtp_context  The specific RTP decompression context
 * @param rtp_static   The compressed RTP header of the rohc packet
 * @param length       The remain length of the rohc packet
 * @param rtp          The decoded RTP header
 * @return             The length of static part of RTP header
 *                     0 if an error occurs
 */
int tcp_decode_static_rtp(rtp_context_t *rtp_context,
                          rtp_static_t *rtp_static,
                          int length,
                          base_header_rtp_t *rtp)
{
	// Save information to context
//	rtp_context->protocol = IPPROTO_RTP;   // A REVOIR eventuellement
	rtp_context->ssrc = rtp_static->ssrc;

	rtp->rtp_version = 2;
	rtp->ssrc = rtp_context->ssrc;

	rohc_debugf(3, "RTP static part length %d\n",(int)sizeof(rtp_static_t));
	TraceData((unsigned char*)rtp_static,sizeof(rtp_static_t));
	fflush(stdout);
	return sizeof(rtp_static_t);
}


/**
 * @brief Copy the static RTP part to the RTP header
 *
 * @param rtp_context   The specific RTP decompression context
 * @param rtp           The RTP header
 * @return              The size of the static part
 */
unsigned int tcp_copy_static_rtp(rtp_context_t *rtp_context,
                                 base_header_rtp_t *rtp)
{
	rohc_debugf(3, "rtp_context %p rtp %p\n",rtp_context,rtp);

	rtp->rtp_version = 2;
	rtp->ssrc = rtp_context->ssrc;

	return sizeof(base_header_rtp_t);
}


/**
 * @brief Uncompress the RTP CSRC identifiers list
 *
 * @param rtp_context   The specific RTP decompression context
 * @param mptr          The RTP CSRC identifiers compressed list
 * @param rtp           The RTP header
 * @return              Pointer to the remain rohc datas
 */
u_int8_t * d_csrc_list_dynchain(rtp_context_t *rtp_context,multi_ptr_t mptr,
                                base_header_rtp_with_csrc_t *rtp)
{
	u_int8_t present;
	u_int8_t PS;
	u_int8_t m;
	u_int8_t i;
	u_int8_t index;
	multi_ptr_t pValue;

	PS = *mptr.uint8 & 0x10;
	m = *(mptr.uint8++) & 0x0F;

	// Init pointer on first value present
	/* if 8-bit XI fields */
	if(PS != 0)
	{
		pValue.uint8 = mptr.uint8 + m;
	}
	else
	{
		pValue.uint8 = mptr.uint8 + ( ( m + 1 ) >> 1 );
	}

	for(i = 0; i < m; ++i)
	{
		if(PS == 0)
		{
			if(i & 1)
			{
				present = (*mptr.uint8) & 0x08;
				index = (*mptr.uint8) & 0x07;
				++mptr.uint8;
			}
			else
			{
				present = (*mptr.uint8) & 0x80;
				index = ( (*mptr.uint8) >> 4 ) & 0x07;
			}
		}
		else
		{
			present = (*mptr.uint8) & 0x80;
			index = *(mptr.uint8++) & 0x0F;
		}

		// If value present
		if(present)
		{
			// Save it in context list
			rtp_context->csrc_list[index] = READ32_FROM_MPTR(mptr);
			rtp_context->csrv_received |= 1 << index;
		}
		else
		{
			// If value not previously received
			if( ( rtp_context->csrv_received & (1 << index) ) == 0)
			{
				// Error
			}
		}
		rtp->csrc_list[i] = rtp_context->csrc_list[index];
	}
	rtp->cc = i;

	return mptr.uint8;
}


/**
 * @brief calculate the size of the RTP CSRC identifiers list
 *
 * @param mptr     The RTP CSRC identifiers compressed list
 * @return         The size of the list
 */
int d_csrc_list_dynchain_size(multi_ptr_t mptr)
{
	u_int8_t present;
	u_int8_t PS;
	u_int8_t m;
	u_int8_t i;
	int size;

	PS = *mptr.uint8 & 0x10;
	m = *(mptr.uint8++) & 0x0F;

	// Init pointer on first value present
	/* if 8-bit XI fields */
	if(PS != 0)
	{
		size = 1 + m;
	}
	else
	{
		size = 1 + ( ( m + 1 ) >> 1 );
	}

	for(i = 0; i < m; ++i)
	{
		if(PS == 0)
		{
			if(i & 1)
			{
				present = *(mptr.uint8++) & 0x08;
			}
			else
			{
				present = (*mptr.uint8) & 0x80;
			}
		}
		else
		{
			present = *(mptr.uint8++) & 0x80;
		}

		// If value present
		if(present)
		{
			// Update size
			size += 4;
		}
	}

	rohc_debugf(3, "csrc list size %d\n",size);

	return size;
}


/**
 * @brief Decode the dynamic part of RTP header of the rohc packet.
 *
 * @param v2_context       The specific RoHCv2 decompression context
 * @param rtp_context      The specific RTP decompression context
 * @param c_base_header    Pointer to the dynamic part of the RTP header of the rohc packet
 * @param length           The remain length of the rohc packet
 * @param rtp              The decoded RTP header
 * @return                 The length of dynamic part of UDP-Lite header
 *                         0 if an error occurs
 */
int tcp_decode_dynamic_rtp(struct d_v2_context *v2_context,
                           rtp_context_t *rtp_context,
                           multi_ptr_t c_base_header,
                           int length,
                           base_header_rtp_with_csrc_t *rtp)
{
	multi_ptr_t mptr;
	int size;

	size = sizeof(rtp_dynamic_t);

	mptr.rtp_dynamic = c_base_header.rtp_dynamic + 1;

	v2_context->reorder_ratio = c_base_header.rtp_dynamic->reorder_ratio;

	rtp->pad_bit = c_base_header.rtp_dynamic->pad_bit;
	rtp->extension = c_base_header.rtp_dynamic->extension;
	rtp->marker = c_base_header.rtp_dynamic->marker;
	rtp->payload_type = c_base_header.rtp_dynamic->payload_type;
	rtp->sequence_number = c_base_header.rtp_dynamic->sequence_number;
	rtp->timestamp = c_base_header.rtp_dynamic->timestamp;

	rtp_context->marker = rtp->marker;
	rtp_context->payload_type = rtp->payload_type;
	rtp_context->pad_bit = rtp->pad_bit;
	rtp_context->extension = rtp->extension;
	rtp_context->sequence_number = ntohs(rtp->sequence_number);
	rtp_context->timestamp = ntohl(rtp->timestamp);

	// ts_stride =:= sdvl_or_default(tss_indicator.CVALUE, TS_STRIDE_DEFAULT);
	rtp_context->ts_stride = d_sdvl_or_default(&mptr,c_base_header.rtp_dynamic->tss_indicator,
	                                           TS_STRIDE_DEFAULT);
	// time_stride =:= sdvl_or_default(tis_indicator.CVALUE, TIME_STRIDE_DEFAULT)
	rtp_context->time_stride = d_sdvl_or_default(&mptr,c_base_header.rtp_dynamic->tis_indicator,
	                                             TIME_STRIDE_DEFAULT);
	// csrc_list =:= csrc_list_dynchain(list_present.CVALUE, cc.UVALUE)
	if(c_base_header.rtp_dynamic->list_present == 0)
	{
		rtp->cc = 0;
	}
	else
	{
		mptr.uint8 = d_csrc_list_dynchain(rtp_context,mptr,rtp);
	}

	size = mptr.uint8 - c_base_header.uint8;

	rohc_debugf(3, "RTP dynamic part length %d\n",size);
	TraceData(c_base_header.uint8,size);
	fflush(stdout);
	return size;
}


/**
 * @brief Decode the RTP profile of the rohc packet.
 *
 * @param decomp            The ROHC decompressor
 * @param v2_context        The specific RoHCv2 decompression context
 * @param ip_inner_context  The specific IP inner decompression context
 * @param rtp_context       The specific RTP decompression context
 * @param base_header_inner The decoded innermost IP header
 * @param profile_value     The profile value used
 * @param c_base_header     Pointer to the dynamic part of the RTP header of the rohc packet
 * @param length            The remain length of the rohc packet
 * @param rtp               The decoded RTP header
 * @param outer_ip_flag     Address of the outer IP flag
 * @return                  The length of dynamic part of UDP header
 *                          0 if an error occurs
 */

int tcp_decode_rtp_baseheader(struct rohc_decomp *decomp,
                              struct d_v2_context *v2_context,
                              ip_context_ptr_t ip_inner_context,
                              rtp_context_t *rtp_context,
                              base_header_ip_t base_header_inner,
                              int profile_value,
                              multi_ptr_t c_base_header,
                              int length,
                              base_header_rtp_with_csrc_t *rtp,
                              int *outer_ip_flag)
{
//	profile_1_7_flags1_t * profile_1_7_flags1;
//	profile_1_flags2_t * profile_1_flags2;
	u_int8_t discriminator;
	u_int8_t header_crc;
	multi_ptr_t mptr;
	WB_t ip_id;
	int size;

	rohc_debugf(3,
	            "decomp %p v2_context %p ip_inner_context %p rtp_context %p base_header_inner %p\n",
	            decomp,
	            v2_context,ip_inner_context.uint8,rtp_context,
	            base_header_inner.uint8);
	rohc_debugf(3, "profile_value %Xh c_base_header %p length %d udp %p outer_ip_flag %p\n",
	            profile_value,c_base_header.uint8,length,rtp,
	            outer_ip_flag);
	fflush(stdout);
	discriminator = *c_base_header.uint8;
	if(discriminator == 0xFA)
	{
		goto decode_rtp_co_common;
	}

	// Init dynamic values with default value from context
	if(ip_inner_context.vx->version == IPV4)
	{
		base_header_inner.ipv4->df = ip_inner_context.v4->df;
		base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
		base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
	}
	else
	{
		#if __BYTE_ORDER == __LITTLE_ENDIAN
		base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
		base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
		#elif __BYTE_ORDER == __BIG_ENDIAN
		base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
		#endif
		base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
	}

	if( ( discriminator & 0x80 ) == 0)
	{
		goto decode_rtp_pt_0_crc3;
	}
	switch(discriminator & 0xE0)
	{
		case 0x80:
			goto decode_rtp_pt_0_crc7;
		case 0xA0:
			goto decode_rtp_pt_1_seq_id;
		case 0xC0:
			goto decode_rtp_pt_2_seq_id;
		default:
			return -1;
	}

#ifdef lQLSJ

	rtp_pt_0_crc3 discriminator : 1;  // =:= '0'     0x00
	rtp_pt_0_crc7 discriminator : 4;  // =:= '1000'  0x08
	rtp_pt_1_rnd discriminator : 3;   // =:= '101'   0x05
	rtp_pt_1_seq_id discriminator : 4; // =:= '1001'  0x09
	rtp_pt_1_seq_ts discriminator : 3; // =:= '101'   0x05
	rtp_pt_2_rnd discriminator : 3;   // =:= '110'   0x06
	rtp_pt_2_seq_id discriminator : 5; // =:= '11000' 0x18
	rtp_pt_2_seq_both discriminator : 5; // =:= '11001' 0x19
	rtp_pt_2_seq_ts discriminator : 4; // =:= '1101'  0x0D

	rtp_pt_0_crc3 discriminator : 1;  // =:= '0'     0x00
	rtp_pt_1_rnd discriminator : 3;   // =:= '101'   0x05
	rtp_pt_1_seq_ts discriminator : 3; // =:= '101'   0x05
	rtp_pt_2_rnd discriminator : 3;   // =:= '110'   0x06
	rtp_pt_0_crc7 discriminator : 4;  // =:= '1000'  0x08
	rtp_pt_1_seq_id discriminator : 4; // =:= '1001'  0x09
	rtp_pt_2_seq_ts discriminator : 4; // =:= '1101'  0x0D
	rtp_pt_2_seq_id discriminator : 5; // =:= '11000' 0x18
	rtp_pt_2_seq_both discriminator : 5; // =:= '11001' 0x19

#endif

decode_rtp_co_common:
	{
		profile_1_7_flags1_t profile_1_7_flags1;
		profile_1_flags2_t profile_1_flags2;
		u_int32_t ts_scaled;
		u_int32_t timestamp;
		u_int32_t ts_stride;
		u_int32_t time_stride;



// rtp_context->new_reorder_ratio;
// rtp_context->new_ts_stride;
// rtp_context->new_time_stride;
// rtp_context->new_ip_id_behavior;  // For all IP header of the compressed chain!

		rohc_debugf(3, "RTP profile co_common\n");
		mptr.rtp_co_common = c_base_header.rtp_co_common + 1;
		size = sizeof(rtp_co_common_t);
		if(c_base_header.rtp_co_common->flags1_indicator != 0)
		{
			profile_1_7_flags1.flags = mptr.profile_1_7_flags1->flags;
			++mptr.profile_1_7_flags1;
			size += 1 + profile_1_7_flags1.name.ttl_hopl_indicator +
			        profile_1_7_flags1.name.tos_tc_indicator;
		}
		else
		{
			profile_1_7_flags1.flags = 0;
		}
		if(c_base_header.rtp_co_common->flags2_indicator != 0)
		{
			profile_1_flags2.flags = mptr.profile_1_flags2->flags;
			++mptr.profile_1_flags2;
			size += 1 + profile_1_flags2.name.pt_indicator;
		}
		else
		{
			profile_1_flags2.flags = 0;
		}
		rohc_debugf(3, "profile_1_7_flags1 %Xh profile_1_flags2 %Xh\n",profile_1_7_flags1.flags,
		            profile_1_flags2.flags);
		fflush(stdout);
		/*
		header_crc = c_base_header.rtp_co_common->header_crc;
		c_base_header.rtp_co_common->header_crc = 0;
		c_base_header.rtp_co_common->header_crc = crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,decomp->crc_table_7);
		if( header_crc != c_base_header.rtp_co_common->header_crc )
		{
		   rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,c_base_header.rtp_co_common->header_crc,size);
		//			return ROHC_ERROR;
		}
		*/
		// =:= irregular(1)
		rtp->marker = c_base_header.rtp_co_common->marker;
		rtp_context->marker = c_base_header.rtp_co_common->marker;
		if(profile_1_7_flags1.flags != 0)
		{
			if(ip_inner_context.vx->version == IPV4)
			{
				if(profile_1_7_flags1.name.tos_tc_indicator != 0)
				{
					ip_inner_context.v4->tos_tc = *(mptr.uint8++);
				}
				base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
				if(profile_1_7_flags1.name.ttl_hopl_indicator != 0)
				{
					ip_inner_context.v4->ttl_hopl = *(mptr.uint8++);
				}
				base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
			}
			else
			{
				if(profile_1_7_flags1.name.tos_tc_indicator != 0)
				{
					#if __BYTE_ORDER == __LITTLE_ENDIAN
					ip_inner_context.v6->tos_tc1 = (*mptr.uint8) >> 4;
					ip_inner_context.v6->tos_tc2 = *(mptr.uint8++);
					#elif __BYTE_ORDER == __BIG_ENDIAN
					ip_inner_context.v6->tos_tc = *(mptr.uint8++);
					#endif
				}
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
				base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
				#elif __BYTE_ORDER == __BIG_ENDIAN
				base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
				#endif
				if(profile_1_7_flags1.name.ttl_hopl_indicator != 0)
				{
					ip_inner_context.v6->ttl_hopl = *(mptr.uint8++);
				}
				base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
			}
		}
		else
		{
			if(ip_inner_context.vx->version == IPV4)
			{
				base_header_inner.ipv4->tos_tc = ip_inner_context.v4->tos_tc;
				base_header_inner.ipv4->ttl_hopl = ip_inner_context.v4->ttl_hopl;
			}
			else
			{
				#if __BYTE_ORDER == __LITTLE_ENDIAN
				base_header_inner.ipv6->tos_tc1 = ip_inner_context.v6->tos_tc1;
				base_header_inner.ipv6->tos_tc2 = ip_inner_context.v6->tos_tc2;
				#elif __BYTE_ORDER == __BIG_ENDIAN
				base_header_inner.ipv6->tos_tc = ip_inner_context.v6->tos_tc;
				#endif
				base_header_inner.ipv6->ttl_hopl = ip_inner_context.v6->ttl_hopl;
			}
		}
		// payload_type =:= pt_irr_or_static(pt_indicator)
		if(profile_1_flags2.name.pt_indicator != 0)
		{
			rtp_context->payload_type = *(mptr.uint8++);
		}
		// sequence_number =:= sdvl_sn_lsb(sequence_number.ULENGTH)
		rohc_debugf(3, "for sequence number mptr : %2.2Xh %2.2Xh\n",mptr.uint8[0],mptr.uint8[1]);
		rtp->sequence_number = htons( d_sdvl_sn_lsb(&mptr) );
		rohc_debugf(3, "so sequence_number %4.4Xh\n",ntohs(rtp->sequence_number));

		if(ip_inner_context.vx->version == IPV4)
		{
			// ip_id =:= ip_id_sequential_variable(ip_id_behavior_innermost.UVALUE, ip_id_indicator.CVALUE)
			rohc_debugf(3, "ip_id_behavior %d ip_id_indicator %d\n",
			            ip_inner_context.v4->ip_id_behavior,
			            c_base_header.rtp_co_common->ip_id_indicator);
			d_ip_id_sequential_variable(&mptr,ip_inner_context.v4->ip_id_behavior,
			                            c_base_header.rtp_co_common->ip_id_indicator,
			                            ip_inner_context.v4->last_ip_id,&ip_id.uint16,
			                            v2_context->msn);
			base_header_inner.ipv4->ip_id = htons(ip_id.uint16);
			ip_inner_context.v4->last_ip_id.uint16 = ip_id.uint16;
		}
		fflush(stdout);
		rohc_debugf(3, "indicators tis %d tss %d tsc %d\n",
		            profile_1_flags2.name.time_stride_indicator,
		            c_base_header.rtp_co_common->tss_indicator,
		            c_base_header.rtp_co_common->tsc_indicator);
		// ts_scaled =:= variable_scaled_timestamp(tss_indicator.CVALUE, tsc_indicator.CVALUE, ts_stride.UVALUE, time_stride.UVALUE)
		rohc_debugf(3, "Decompress ts_scaled %2.2Xh %2.2Xh %2.2Xh\n",mptr.uint8[0],mptr.uint8[1],
		            mptr.uint8[2]);
		d_variable_scaled_timestamp(&mptr,rtp_context,c_base_header.rtp_co_common->tss_indicator,
		                            c_base_header.rtp_co_common->tsc_indicator,
		                            &ts_scaled);
		fflush(stdout);
		rohc_debugf(3, "Decompress timestamp %2.2Xh %2.2Xh %2.2Xh %2.2Xh %2.2Xh %2.2Xh\n",
		            mptr.uint8[0],mptr.uint8[1],mptr.uint8[2],mptr.uint8[3],mptr.uint8[4],
		            mptr.uint8[5]);
		// timestamp =:= variable_unscaled_timestamp(tss_indicator.CVALUE, tsc_indicator.CVALUE)
		d_variable_unscaled_timestamp(&mptr,c_base_header.rtp_co_common->tss_indicator,
		                              c_base_header.rtp_co_common->tsc_indicator,
		                              rtp_context->timestamp,
		                              &timestamp);
		rohc_debugf(3, "timestamp %d %Xh\n",timestamp,timestamp);

		rtp_context->timestamp = timestamp;
		rtp->timestamp = htonl(timestamp);
		rohc_debugf(3, "Decompress ts_tride %2.2Xh %2.2Xh %2.2Xh\n",mptr.uint8[0],mptr.uint8[1],
		            mptr.uint8[2]);

		fflush(stdout);
		// ts_stride =:= sdvl_or_static(tss_indicator.CVALUE)
		d_sdvl_or_static(&mptr,c_base_header.rtp_co_common->tss_indicator,rtp_context->ts_stride,
		                 &ts_stride);
		rohc_debugf(3, "ts_stride %d %Xh\n",ts_stride,ts_stride);
		fflush(stdout);
		rohc_debugf(3, "Decompress time_stride %2.2Xh %2.2Xh %2.2Xh\n",mptr.uint8[0],mptr.uint8[1],
		            mptr.uint8[2]);
		// time_stride =:= sdvl_or_static(tis_indicator.CVALUE)
		// 0 si c_base_header.rtp_co_common->flags2_indicator == 0 ????
		d_sdvl_or_static(&mptr,profile_1_flags2.name.time_stride_indicator,rtp_context->time_stride,
		                 &time_stride);
		rohc_debugf(3, "time_stride %d %Xh\n",time_stride,time_stride);
		fflush(stdout);
		if(profile_1_flags2.flags != 0)
		{
			rtp_context->pad_bit = profile_1_flags2.name.pad_bit;
			rtp_context->extension = profile_1_flags2.name.extension;
			// csrc_list =:= csrc_list_dynchain(list_present.CVALUE, cc.UVALUE)
			if(profile_1_flags2.name.list_indicator == 0)
			{
				rtp->cc = 0;
			}
			else
			{
				mptr.uint8 = d_csrc_list_dynchain(rtp_context,mptr,rtp);
				fflush(stdout);
			}
		}
		else
		{
			rtp->cc = 0;
		}
		rtp->payload_type = rtp_context->payload_type;
		rtp->pad_bit = rtp_context->pad_bit;
		rtp->extension = rtp_context->extension;
		size = mptr.uint8 - c_base_header.uint8;
		rohc_debugf(3, "return size %d\n",size);
		fflush(stdout);
		return size;
	}


decode_rtp_pt_0_crc3:
	rohc_debugf(3, "RTP profile pt_0_crc3\n");
	size = sizeof(rtp_pt_0_crc3_t);
	header_crc = c_base_header.rtp_pt_0_crc3->header_crc;
	c_base_header.rtp_pt_0_crc3->header_crc = 0;
	c_base_header.rtp_pt_0_crc3->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.rtp_pt_0_crc3->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_0_crc3->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto idem_decode_rtp_pt_0_crc7;

decode_rtp_pt_0_crc7:
	rohc_debugf(3, "RTP profile pt_0_crc7\n");
	size = sizeof(rtp_pt_0_crc7_t);
	header_crc = c_base_header.rtp_pt_0_crc7->header_crc;
	c_base_header.rtp_pt_0_crc7->header_crc = 0;
	c_base_header.rtp_pt_0_crc7->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.rtp_pt_0_crc7->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_0_crc7->header_crc,
		            size);
		return ROHC_ERROR;
	}
idem_decode_rtp_pt_0_crc7:
	if(ip_inner_context.vx->version == IPV4)
	{
		switch(ip_inner_context.v4->ip_id_behavior)
		{
			case IP_ID_BEHAVIOR_SEQUENTIAL:
				++ip_inner_context.v4->last_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(ip_inner_context.v4->last_ip_id.uint16);
				break;
			case IP_ID_BEHAVIOR_SEQUENTIAL_SWAPPED:
			{
				WB_t swapped_ip_id;
				swapped_ip_id.uint8[0] = ip_inner_context.v4->last_ip_id.uint8[1];
				swapped_ip_id.uint8[1] = ip_inner_context.v4->last_ip_id.uint8[0];
				++swapped_ip_id.uint16;
				base_header_inner.ipv4->ip_id = htons(swapped_ip_id.uint16);
				ip_inner_context.v4->last_ip_id.uint8[1] = swapped_ip_id.uint8[0];
				ip_inner_context.v4->last_ip_id.uint8[0] = swapped_ip_id.uint8[1];
			}
			break;
			default:
				break;
		}
	}
	goto end_decode_rtp_base_header;


decode_rtp_pt_1_rnd:
	rohc_debugf(3, "RTP profile pt_1_rnd\n");
	size = sizeof(rtp_pt_1_rnd_t);
	header_crc = c_base_header.rtp_pt_1_rnd->header_crc;
	c_base_header.rtp_pt_1_rnd->header_crc = 0;
	c_base_header.rtp_pt_1_rnd->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.rtp_pt_1_rnd->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_1_rnd->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto end_decode_rtp_base_header;


decode_rtp_pt_1_seq_id:
	rohc_debugf(3, "RTP profile pt_1_seq_id\n");
	size = sizeof(rtp_pt_1_seq_id_t);
	header_crc = c_base_header.rtp_pt_1_seq_id->header_crc;
	c_base_header.rtp_pt_1_seq_id->header_crc = 0;
	c_base_header.rtp_pt_1_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.rtp_pt_1_seq_id->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_1_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto end_decode_rtp_base_header;


decode_rtp_pt_1_seq_ts:
	rohc_debugf(3, "RTP profile pt_1_seq_ts\n");
	size = sizeof(rtp_pt_1_seq_ts_t);
	header_crc = c_base_header.rtp_pt_1_seq_ts->header_crc;
	c_base_header.rtp_pt_1_seq_ts->header_crc = 0;
	c_base_header.rtp_pt_1_seq_ts->header_crc =
	   crc_calculate(CRC_TYPE_3,c_base_header.uint8,size,CRC_INIT_3,
	                 decomp->crc_table_3);
	if(header_crc != c_base_header.rtp_pt_1_seq_ts->header_crc)
	{
		rohc_debugf(3, "crc3 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_1_seq_ts->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto end_decode_rtp_base_header;


decode_rtp_pt_2_rnd:
	rohc_debugf(3, "RTP profile pt_2_rnd\n");
	size = sizeof(rtp_pt_2_rnd_t);
	header_crc = c_base_header.rtp_pt_2_rnd->header_crc;
	c_base_header.rtp_pt_2_rnd->header_crc = 0;
	c_base_header.rtp_pt_2_rnd->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.rtp_pt_2_rnd->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_2_rnd->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto end_decode_rtp_base_header;


decode_rtp_pt_2_seq_id:
	rohc_debugf(3, "RTP profile pt_2_seq_id\n");
	size = sizeof(rtp_pt_2_seq_id_t);
	header_crc = c_base_header.rtp_pt_2_seq_id->header_crc;
	c_base_header.rtp_pt_2_seq_id->header_crc = 0;
	c_base_header.rtp_pt_2_seq_id->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.rtp_pt_2_seq_id->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_2_seq_id->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto end_decode_rtp_base_header;

decode_rtp_pt_2_seq_both:
	rohc_debugf(3, "RTP profile pt_2_seq_both\n");
	size = sizeof(rtp_pt_2_seq_both_t);
	header_crc = c_base_header.rtp_pt_2_seq_both->header_crc;
	c_base_header.rtp_pt_2_seq_both->header_crc = 0;
	c_base_header.rtp_pt_2_seq_both->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.rtp_pt_2_seq_both->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_2_seq_both->header_crc,
		            size);
		return ROHC_ERROR;
	}
	goto end_decode_rtp_base_header;

decode_rtp_pt_2_seq_ts:
	rohc_debugf(3, "RTP profile pt_2_seq_ts\n");
	size = sizeof(rtp_pt_2_seq_ts_t);
	header_crc = c_base_header.rtp_pt_2_seq_ts->header_crc;
	c_base_header.rtp_pt_2_seq_ts->header_crc = 0;
	c_base_header.rtp_pt_2_seq_ts->header_crc =
	   crc_calculate(CRC_TYPE_7,c_base_header.uint8,size,CRC_INIT_7,
	                 decomp->crc_table_7);
	if(header_crc != c_base_header.rtp_pt_2_seq_ts->header_crc)
	{
		rohc_debugf(3, "crc7 %Xh != %Xh length %d\n",header_crc,
		            c_base_header.rtp_pt_2_seq_ts->header_crc,
		            size);
		return ROHC_ERROR;
	}

end_decode_rtp_base_header:
	return size;
}


/**
 * @brief Get the reference MSN value of the context.
 *
 * This function is one of the functions that must exist in one profile for the
 * framework to work.
 *
 * @param context The decompression context
 * @return        The reference MSN value
 */
int d_v2_get_msn(struct d_context *context)
{
	struct d_generic_context *g_context = context->specific;
	struct d_v2_context *v2_context = g_context->specific;

	rohc_debugf(3, "return %Xh\n",v2_context->msn);

	return v2_context->msn;
}


/**
 * @brief Define the decompression part of the UDP profile as described
 *        in the RFC 3095.
 */
//struct d_profile d_rohc_v2_ip_profile =
d_v2_profile_t d_rohc_v2_ip_profile =
{
	{
		ROHCv2_PROFILE_IP,    /* profile ID (see RFC 5225 page 4 or 46) */
		"ROHCv2 IP / Decompressor", /* profile description */
		d_v2_decode,        /* profile handlers */
//	d_v2_decode_ir,
		d_v2_create,
		d_v2_destroy,
//	tcp_detect_ir_size,
//	NULL,
//	tcp_get_static_size,
		d_v2_get_msn
	},
	0,
//	sizeof(ipv6_context_t)
//	sizeof(ipv6_static2_t)
	0
};

/**
 * @brief Define the decompression part of the UDP profile as described
 *        in the RFC 3095.
 */
//struct d_profile d_rohc_v2_udp_profile =
d_v2_profile_t d_rohc_v2_udp_profile =
{
	{
		ROHCv2_PROFILE_UDP,    /* profile ID (see RFC 5225 page 4 or 46) */
		"ROHCv2 UDP / Decompressor", /* profile description */
		d_v2_decode,        /* profile handlers */
//	d_v2_decode_ir,
		d_v2_create,
		d_v2_destroy,
//	tcp_detect_ir_size,
//	NULL,
//	tcp_get_static_size,
		d_v2_get_msn
	},
	IPPROTO_UDP,
	sizeof(udp_static_t)
};

/**
 * @brief Define the decompression part of the RTP profile as described
 *        in the RFC 3095.
 */
//struct d_profile d_rohc_v2_rtp_profile =
d_v2_profile_t d_rohc_v2_rtp_profile =
{
	{
		ROHCv2_PROFILE_RTP,    /* profile ID (see RFC 5225 page 4 or 46) */
		"ROHCv2 RTP / Decompressor", /* profile description */
		d_v2_decode,        /* profile handlers */
//	d_v2_decode_ir,
		d_v2_create,
		d_v2_destroy,
//	tcp_detect_ir_size,
//	NULL,
//	tcp_get_static_size,
		d_v2_get_msn
	},
	IPPROTO_UDP,
	sizeof(udp_static_t) + sizeof(rtp_static_t)
};

/**
 * @brief Define the decompression part of the UDP-Lite profile as described
 *        in the RFC 3095.
 */
//struct d_profile d_rohc_v2_udp_lite_profile =
d_v2_profile_t d_rohc_v2_udp_lite_profile =
{
	{
		ROHCv2_PROFILE_UDPLITE,   /* profile ID (see RFC 5225 page 4 or 46) */
		"ROHCv2 UDP-Lite / Decompressor", /* profile description */
		d_v2_decode,        /* profile handlers */
//	d_v2_decode_ir,
		d_v2_create,
		d_v2_destroy,
//	tcp_detect_ir_size,
//	NULL,
//	tcp_get_static_size,
		d_v2_get_msn
	},
	IPPROTO_UDPLITE,
	sizeof(udp_lite_static_t)
};

/**
 * @brief Define the decompression part of the ESP profile as described
 *        in the RFC 3095.
 */
//struct d_profile d_rohc_v2_esp_profile =
d_v2_profile_t d_rohc_v2_esp_profile =
{
	{
		ROHCv2_PROFILE_ESP,    /* profile ID (see RFC 5225 page 4 or 46) */
		"ROHCv2 ESP / Decompressor", /* profile description */
		d_v2_decode,        /* profile handlers */
//	d_v2_decode_ir,
		d_v2_create,
		d_v2_destroy,
//	tcp_detect_ir_size,
//	NULL,
//	tcp_get_static_size,
		d_v2_get_msn
	},
	IPPROTO_ESP,
	sizeof(esp_static_t)
};


/**
 * @brief Define the decompression part of the UDP-Lite RTP profile as described
 *        in the RFC 3095.
 */
//struct d_profile d_rohc_v2_udp_lite_rtp_profile =
d_v2_profile_t d_rohc_v2_udp_lite_rtp_profile =
{
	{
		ROHCv2_PROFILE_UDPLITE_RTP,   /* profile ID (see RFC 5225 page 4 or 46) */
		"ROHCv2 UDP-Lite RTP / Decompressor", /* profile description */
		d_v2_decode,        /* profile handlers */
//	d_v2_decode_ir,
		d_v2_create,
		d_v2_destroy,
//	tcp_detect_ir_size,
//	NULL,
//	tcp_get_static_size,
		d_v2_get_msn
	},
	IPPROTO_UDPLITE,
	sizeof(udp_lite_static_t) + sizeof(rtp_static_t)
};

#endif // SUPPORT_ROHC_V2

